// CandidateChooser.cpp: implementation of the CCandidateChooser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "CandidateChooser.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCandidateChooser::CCandidateChooser()
{
	m_DSepGrid = NULL;
	m_Interpolator = NULL;
	m_VectorField = NULL;
	m_CaPerSP = DEF_CA_PER_SP;
	m_Normals = NULL;
}

CCandidateChooser::~CCandidateChooser()
{
	DeleteCandidates();

	if(m_Normals != NULL)
		delete [] m_Normals;
}


void CCandidateChooser::CreateCandidates(CStreamline *streamline)
{
	// Delete previous candidates.
	DeleteCandidates();

	CObList *samplePoints = streamline->GetSamplePoints();
	POSITION pos = samplePoints->GetHeadPosition();

	CPoint3D caPos;
	CPoint3D currentPos;

	float dSep = m_DSepGrid->GetDSep();

	while (pos != NULL)
	{
		CSamplePoint * currentPoint = (CSamplePoint *)samplePoints->GetNext(pos);
		currentPos = *currentPoint->GetPos();
		
		CVector3D::Normals(*currentPoint->GetUnitVec(),m_Normals,m_CaPerSP);
					
		for(int i=0;i<m_CaPerSP;i++)
		{
			caPos.m_XPos = currentPos.m_XPos + m_Normals[i].m_XComp * dSep;
			caPos.m_YPos = currentPos.m_YPos + m_Normals[i].m_YComp * dSep;
			caPos.m_ZPos = currentPos.m_ZPos + m_Normals[i].m_ZComp * dSep;

			if(caPos.Inside(m_VectorField->m_SeedMinPos,m_VectorField->m_SeedMaxPos))
			{
				CSamplePoint *candidate = 
					m_Interpolator->Interpolate(caPos,
							m_VectorField->GetSamplePoints(caPos));
			
				m_Candidates.AddHead(candidate);
			}
		}
	}
	
}

CObList * CCandidateChooser::GetCandidates()
{
	return &m_Candidates;
}

CCandidateChooser::CCandidateChooser(CDSepGrid *dSepGrid,
									 CInterpolator *interpolator, int caPerSp)
{
	m_DSepGrid = dSepGrid;
	m_Interpolator = interpolator;
	m_CaPerSP = caPerSp;
	
	m_Normals = new CVector3D[m_CaPerSP];

	m_VectorField = m_DSepGrid->GetVectorField();
}

void CCandidateChooser::SetInterpolator(CInterpolator *interpolator)
{
	m_Interpolator = interpolator;
}

void CCandidateChooser::SetDSepGrid(CDSepGrid *dSepGrid)
{
	m_DSepGrid = dSepGrid;
	m_VectorField = m_DSepGrid->GetVectorField();
}

void CCandidateChooser::SetCaPerSP(int caPerSP)
{
	m_CaPerSP = caPerSP;

	if(m_Normals != NULL)
		delete [] m_Normals;

	m_Normals = new CVector3D[m_CaPerSP];
}

void CCandidateChooser::DeleteCandidates()
{
	POSITION pos = m_Candidates.GetHeadPosition();

	while(pos != NULL)
	{
		CSamplePoint *s = (CSamplePoint *)m_Candidates.GetNext(pos);
		m_Candidates.RemoveHead();
		delete s;
	}
}
