// CandidateChooser.h: interface for the CCandidateChooser class.
//
//////////////////////////////////////////////////////////////////////

#include "Streamline.h"
#include "Defines.h"

#if !defined(AFX_CANDIDATECHOOSER_H__82BBB66E_D9C8_41EA_9B37_B0B0A4A49FA2__INCLUDED_)
#define AFX_CANDIDATECHOOSER_H__82BBB66E_D9C8_41EA_9B37_B0B0A4A49FA2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


//!  A class for getting candidates for a seedpoint of a new streamline.
/*!
	Used to compute a number of possible seedpoints from every samplepoint of a existing
	streamline in the distance "dSep" from the streamline. 
	The number of the candidates computed for every streamline-samplepoint is 'CaPerSP'
	(= "Candidates Per SamplePoint").
*/
class CCandidateChooser  
{
public:
	//! Clears list of candidates.
	void DeleteCandidates();
	
	//! Sets value of "Candidates per samplepoint".
	void SetCaPerSP(int caPerSP);

	//! Sets a pointer to a dSepGrid.
	void SetDSepGrid(CDSepGrid *dSepGrid);

	//! Sets a pointer to a interpolator-class.
	void SetInterpolator(CInterpolator *interpolator);

	//! Returns list of candidates.
	CObList * GetCandidates();
	//! Creates list of candidates for streamline.
	void CreateCandidates(CStreamline *streamline);

	CCandidateChooser(CDSepGrid *dSepGrid,CInterpolator *interpolator, 
					  int caPerSp);

	CCandidateChooser();
	virtual ~CCandidateChooser();

private:
	CInterpolator * m_Interpolator;
	CDSepGrid * m_DSepGrid;
	CVectorField *m_VectorField;
	//! Candidates per samplepoint.
	int m_CaPerSP;

	//! Array of normal vector for next candidates.
	CVector3D *m_Normals;
	//! List of candidates.
	CObList m_Candidates;
};

#endif // !defined(AFX_CANDIDATECHOOSER_H__82BBB66E_D9C8_41EA_9B37_B0B0A4A49FA2__INCLUDED_)
