// DSepGrid.h: interface for the CDSepGrid class.
//
//////////////////////////////////////////////////////////////////////
#include "Vectorfield.h"

#if !defined(AFX_DSEPGRID_H__31D244F2_E6D7_4CF5_96D7_F12A4D99648F__INCLUDED_)
#define AFX_DSEPGRID_H__31D244F2_E6D7_4CF5_96D7_F12A4D99648F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//! Class representing a dSep-grid.

/*!
	The dSep-grid is needed by the ESS-algorithm to determine
	if a seedpoint has at least a distance dSep from all the other
m	streamlines.
*/
class CDSepGrid : public CObject
{
public:
	bool IsValidIndex();
	void Clamp(int &x, int &y, int &z);
	bool IsValidIndex(int x, int y, int z);
	void Insert(CSamplePoint *samplePoint, int x, int y, int z);
	void GetXYZIndex(int &x, int &y, int &z, CPoint3D &pos);
	void CreateGrid();
	CVectorField * GetVectorField();

	void DeleteGrid();

	//! Create Grid with cell size equal to dSep, storing samplepoints in lists.
	void InitGrid();

	//! Returns dTest.
	float GetDTest();
	//! Returns dSep.
	float GetDSep();

	//! Sets dTest.
	void SetDTest(float dTest);
	//! Sets dTest.
	void SetDSep(float dSep);

	//! Returns x-index of samplepoint in grid.
	int GetXIndex(CSamplePoint *samplePoint);
	//! Returns y-index of samplepoint in grid.
	int GetYIndex(CSamplePoint *samplePoint);
	//! Returns z-index of samplepoint in grid.
	int GetZIndex(CSamplePoint *samplePoint);

	void SetVectorField(CVectorField *vectorField);

	//! Inserts samplepoint into grid.
	void Insert(CSamplePoint *samplePoint);

	//! Checks if a samplepoint is valid regarding dSep and field boundaries.
	bool TestValid(int type, CSamplePoint *samplePoint);

	//! Overloaded operator '='.
	CDSepGrid& operator=(const CDSepGrid &dSepGrid);

	//! Copy-constructor.
	CDSepGrid(const CDSepGrid &dSepGrid);
	CDSepGrid();
	CDSepGrid(CVectorField * vectorField, float dSep, float dTest);

	virtual ~CDSepGrid();

private:
	float m_QuadMinDist_line;
	float m_QuadMinDist_start;

	float m_DSep;
	float m_InvDSep;
	float m_DTest;
	int m_XDim;
	int m_YDim;
	int m_ZDim;
	int m_MaxXIndex;
	int m_MaxYIndex;
	int m_MaxZIndex;
	CPoint3D m_MinPos;
	CPoint3D m_MaxPos;

	CVectorField *m_VectorField;
	//! The DSep-Grid stores a list of samplepoints for every cell.
	CObList *** m_DSepGrid;
};

#endif // !defined(AFX_DSEPGRID_H__31D244F2_E6D7_4CF5_96D7_F12A4D99648F__INCLUDED_)
