//! File for storing all defines.
#if !defined DEFINES_H__
#define DEFINES_H__ 

// Pi.
#define PI							 3.14159265f

// binary values used for indexing in triliear interpolators.
#define B_000						 0
#define B_001						 1
#define B_010						 2
#define B_011						 3
#define B_100						 4
#define B_101						 5
#define B_110						 6
#define B_111						 7

// Default parameters for sampling equitations.
#define DEF_PARAM1					 5.0f
#define DEF_PARAM2					 5.0f
#define DEF_PARAM3					 0.2f

// Default field dimensions.
#define DEF_XDIM				     100
#define DEF_YDIM					 100
#define DEF_ZDIM					 100

// Default parameters for main-algorithm(= ESS)

// Default number of seedpoint-candidates computed for every samplepoint.
#define DEF_CA_PER_SP				 7

// Default step size of integrator
#define DEF_DT						 0.002f

// Default separating distance in percent of x-width of vectorfield.
#define DEF_DSEP_PERC				 5.0f

// Default absolute dsep.
#define DEF_DSEP					 7.0f

// Default max-value for streamline integration.
#define DEF_MAXITERATIONS			 2000

// Default value for test distance.
#define DEF_DTEST					 0.5f

// Default index values for radio button.
#define DEF_EULER_INDEX				 0
#define DEF_RK2_INDEX				 1
#define DEF_RK4_INDEX				 2

// Default index values for radio button.
#define DEF_NN_INDEX				 0
#define DEF_TRILINEAR_INDEX			 1
#define DEF_TRILINEARVL_INDEX		 2

// FocusContext recalc-parameter radio values.
#define DEF_NEWBOUND_INDEX			0
#define DEF_SEEDREG_INDEX			1

// Default diffuse correction value.
#define DEF_DIFFUSE_CORRECTION		 2.0f

// Default glossiness of streamlines.
#define DEF_GLOSSINESS				 70.0f

// Default material values.
#define DEF_MAT_AMB_R				 30
#define DEF_MAT_AMB_G				 50
#define DEF_MAT_AMB_B				 30

#define DEF_MAT_DIF_R				 50
#define DEF_MAT_DIF_G				 200
#define DEF_MAT_DIF_B				 50

#define DEF_MAT_SPEC_R				 200
#define DEF_MAT_SPEC_G				 255
#define DEF_MAT_SPEC_B				 200

// Default size of texture map.
#define DEF_TEXTURE_SIZE			 256

// Hints for UpdateAllViews call.
#define DEF_HINT_DRAW_NEW_STREAMLINE 1
#define DEF_HINT_INIT_SCENE			 2
#define DEF_HINT_REDRAW_SCENE		 4

// minimal values for sliders.
#define DEF_DSEPSLIDER_MIN			 10
#define DEF_DTESTSLIDER_MIN			 100
#define DEF_DTSLIDER_MIN			 1
#define DEF_MAXITERATIONSLIDER_MIN	 10
#define DEF_MAXSTREAMLINESLIDER_MIN	 0
#define DEF_CAPERSPSLIDER_MIN		 1

#define DEF_LINEWIDTHSLIDER_MIN		 10
#define DEF_INTERPOSSLIDER_MIN		 0
#define DEF_REGIONSLIDER_MIN		 0
#define DEF_HIGHLIGHTSLIDER_MIN		 1
#define DEF_DIFFUSECORRSLIDER_MIN	 100
#define DEF_RESNUMSLIDER_MIN		 1
#define DEF_ENDTAP_WIDTHSLIDER_MIN	 1

#define DEF_P1SLIDER_MIN			 -200			 
#define DEF_P2SLIDER_MIN			 -200
#define	DEF_P3SLIDER_MIN			 -200

#define DEF_XDIMSLIDER_MIN			 1
#define DEF_YDIMSLIDER_MIN			 1
#define	DEF_ZDIMSLIDER_MIN			 1

#define DEF_PERIODLENGTHSLIDER_MIN	 2
#define DEF_ABSORPTIONSLIDER_MIN	 0

#define DEF_GAPSLIDER_MIN			 0
#define DEF_OPACITYSLIDER_MIN		 0
#define DEF_MINDISTANCESLIDER_MIN	 0
#define DEF_PERCENTSLIDER_MIN		 0

#define DEF_SPEEDSLIDER_MIN			 1
#define DEF_INTENSITYSLIDER_MIN		 0
#define DEF_ARROWLENGTHSLIDER_MIN	 10
#define DEF_ARROWWIDTHSLIDER_MIN	 10
#define DEF_BLOCK_ZOFFSSLIDER_MIN	 0

#define DEF_AR_PERIODSLIDER_MIN		 0

// Maximum values for sliders.
#define DEF_ARROWWIDTHSLIDER_MAX	 50
#define DEF_ARROWLENGTHSLIDER_MAX	 100
#define DEF_SPEEDSLIDER_MAX			 200
#define DEF_DSEPSLIDER_MAX			 1000
#define DEF_DTESTSLIDER_MAX			 1000
#define DEF_DTSLIDER_MAX			 1000
#define DEF_GAPSLIDER_MAX			 500
#define DEF_PERCENTSLIDER_MAX		 100
#define DEF_RESNUMSLIDER_MAX		 100
 
#define DEF_MAXITERATIONSLIDER_MAX	 10000
#define DEF_MAXSTREAMLINESLIDER_MAX	 10000
#define DEF_CAPERSPSLIDER_MAX		 100

#define DEF_LINEWIDTHSLIDER_MAX		 100
#define DEF_INTERPOSSLIDER_MAX		 1000
#define DEF_REGIONSLIDER_MAX		 1000
#define DEF_HIGHLIGHTSLIDER_MAX		 200
#define DEF_DIFFUSECORRSLIDER_MAX	 1000
#define DEF_ENDTAP_WIDTHSLIDER_MAX	 30

#define DEF_P1SLIDER_MAX			 200
#define DEF_P2SLIDER_MAX			 200
#define	DEF_P3SLIDER_MAX			 200

#define DEF_XDIMSLIDER_MAX			 300
#define DEF_YDIMSLIDER_MAX			 300	
#define	DEF_ZDIMSLIDER_MAX			 300

#define DEF_PERIODLENGTHSLIDER_MAX	 500
#define DEF_ABSORPTIONSLIDER_MAX	 255
#define DEF_OPACITYSLIDER_MAX		 255
#define DEF_MINDISTANCESLIDER_MAX	 3000
#define DEF_INTENSITYSLIDER_MAX		 255
#define DEF_BLOCK_ZOFFSSLIDER_MAX	 500

#define DEF_AR_PERIODSLIDER_MAX		 100

// Factors used to map slider values to corresponding variables.
#define DEF_DSEPSLIDER_FACTOR		 20.0f
#define DEF_DTESTSLIDER_FACTOR		 1000.0f
#define DEF_DTSLIDER_FACTOR			 10000.0f
#define DEF_LINEWIDTHSLIDER_FACTOR	 10.0f
#define DEF_DIFFUSECORRSLIDER_FACTOR 100.0f
#define DEF_HIGHLIGHTSLIDER_FACTOR   1.0f
#define DEF_MINDISTANCESLIDER_FACTOR 1000
#define DEF_P1SLIDER_FACTOR			 10.0f			 
#define DEF_P2SLIDER_FACTOR			 10.0f
#define	DEF_P3SLIDER_FACTOR			 10.0f

#define DEF_GAPSLIDER_FACTOR		 100.0f
#define DEF_ARROWWIDTHSLIDER_FACTOR  10.0f
#define DEF_ARROWLENGTHSLIDER_FACTOR 10.0f

#define DEF_BLOCK_ZOFFSSLIDER_FACTOR 100.0f

// Multiplicative factor used to set up texture matrix
#define DEF_TEXMATRIX_FACTOR		 0.5f

// Multiplicative factor used to regulate z-offset of vectorfield.
#define DEF_Z_OFFS_FACTOR			 2.0f

// Multiplicative factor used to regulate rotation speed
// during r-mouse button rotation.
#define DEF_ONTIMER_ANGLEINCR_FACTOR 0.005f

// Multiplicative factor used to regulate rotation speed during l- button
// vectorfield rotation.
#define DEF_MOUSEMV_ANGLEINCR_FACTOR 0.1f  

// Multiplicative factor used to regulate rotation speed of vectorfield during
// streamline calculation
#define DEF_ONUPD_ANGLEINCR_FACTOR   1.0f

// Multiplicative factor used to regulate position increment during seedpoint-
// placement mode.
#define DEF_MOUSEMOV_POSINCR_FACTOR  0.3f

// Multiplicative factor used to regulate speed during movement along z-axis.
#define DEF_DISTINCR_FACTOR			 0.3f

// Multiplicative factor used to regulate zoffs of block(only block-dataset).
#define DEF_BLOCK_ZOFFS_FACTOR		 0.01f;

// Number of material regions which can be defined in additional info dialog.
#define DEF_REGIONS_NUM				 5

// Minimal position where region can start.
#define DEF_REGIONPOS_MIN			 0.0f

// Index values mostly for various controls:

// Light type radio button values.
#define DEF_HEADLIGHT_INDEX			 0
#define DEF_POINTLIGHT_INDEX		 1

// Additional info radio button values.
#define DEF_CONTTRANSF_INDEX		 0
#define DEF_REGIONS_INDEX			 1

// Depth coding type radio button values.
#define DEF_DEPTHCUEING_INDEX		 0
#define DEF_DEPTHCOLOR_INDEX		 1

// Sampling equations radio button values.
#define DEF_ROESSLERSAMPLER_INDEX	 0
#define DEF_LORENZSAMPLER_INDEX		 1
#define DEF_TESTSAMPLER_INDEX		 2

// Arrow type radio button values.
#define DEF_AR_LEND_INDEX			 0
#define DEF_AR_PERIOD_INDEX			 1

// Index values for combo box in additional info dialog.
#define DEF_VELOCITY_INDEX			 0
#define DEF_ORIENT_X_INDEX			 1
#define DEF_ORIENT_Y_INDEX			 2
#define DEF_ORIENT_Z_INDEX			 3
#define DEF_POS_X_INDEX				 4
#define DEF_POS_Y_INDEX				 5
#define DEF_POS_Z_INDEX				 6
#define DEF_DIF_ORIENT_INDEX		 7
#define DEF_DIF_VLEN_INDEX			 8

#define DEF_SCALAR_NUM				 9
#define DEF_SCALAR_OVERALL			 DEF_SCALAR_NUM + CSamplePoint::GetAdditionalSize()


// Index values for transparency function radio buttons.
#define DEF_HAZE_LIN_INDEX			  0
#define DEF_HAZE_EXP_INDEX			  1
#define DEF_SIN_INDEX				  2

// Index values for magic volume type radio buttons.
#define DEF_MCUBE_INDEX				  0
#define DEF_MSPHERE_INDEX			  1
#define DEF_MRECT_INDEX				  2

// Default magic volume index value.
#define DEF_MAGICVOL_INDEX			  2

// Index values for light type radio buttons.
#define DEF_HEADLIGHT_INDEX			  0
#define DEF_POINTLIGHT_INDEX		  1

// Index values for timers.
#define DEF_ROTATIONTIMER_INDEX		  1
#define DEF_ANIMATIONTIMER_INDEX	  2

// Index values used to choose list most closely resembling view direction.
#define DEF_XLIST_INDEX				  0
#define DEF_YLIST_INDEX				  1
#define DEF_ZLIST_INDEX				  2
#define DEF_XLIST_REVERSE_INDEX		  3
#define DEF_YLIST_REVERSE_INDEX		  4
#define DEF_ZLIST_REVERSE_INDEX		  5

// Number of different main view directions.
#define DEF_DIRECTIONS_NUM			  6

// Strings displayed in additional info combo box.
#define DEF_VELOCITY_STR			  "Velocity(= vector length)"
#define DEF_ORIENT_X_STR			  "Arccos X-comp. [0..PI]"
#define DEF_ORIENT_Y_STR			  "Arccos Y-comp. [0..PI]"
#define DEF_ORIENT_Z_STR			  "Arccos Z-comp. [0..PI]"
#define DEF_POS_X_STR				  "Position X"
#define DEF_POS_Y_STR				  "Position Y"
#define DEF_POS_Z_STR				  "Position Z"
#define DEF_ADDINFO_STR				  "VField scalar"
#define DEF_DIF_ORIENT_STR			  "Dif. orientation"
#define DEF_DIF_VLEN_STR			  "Dif. velocity"
#define DEF_DEPTH_STR				  "Depth"

// Defines for color code transfer function:

// Default position of intermediate point in continuous transfer function.
#define	DEF_INTERPOS				  0.5f

// Default minimal rgba value in color code-transfer function.
#define DEF_MINRGBA					  {0,0,255,255}

// Default intermediate color values in continuous transfer function.
#define DEF_INTERRGBA				  {0,255,0,255}

// Default maximum color values in color code-transfer function.
#define DEF_MAXRGBA					  {255,0,0,255}

// Minimal and maximal color values
#define DEF_COLOR_MIN				  0
#define DEF_COLOR_MAX				  255

// Gl-initialisation values.
#define DEF_NEARPLANE				  0.1f
#define DEF_FARPLANE				  500.0f
#define DEF_VIEWANGLE				  45.0f

// Defines used in line style dialog:

// Default arrow period length
#define DEF_AR_PERIOD_LENGHT		  10

// Default period length of transparency function(haze lin, sin).
#define DEF_PERIOD_LENGTH			  80

// Default absorption of exponential particle haze transparency function.
#define DEF_ABSORPTION				  10

// Default line width.
#define DEF_LINEWIDTH				  5.0f

// Default gap size between streamlines.
#define DEF_GAP_SIZE				  2.0f

// 0-degree angle.
#define DEF_ANGLE_0F				  0.0f

// 90-degree angle
#define DEF_ANGLE_90F				  90.0f

// 180-degree angle
#define DEF_ANGLE_180F				  180.0f

// 270-degree angle
#define DEF_ANGLE_270F				  270.0f
// 360 degree angle.
#define DEF_ANGLE_360F				  360.0f

// 360 degree angle.
#define DEF_ANGLE_360				  360

#define DEF_SINTABLE_WIDTH			  360
#define DEF_COSTABLE_WIDTH			  360

// Min and max values for opaciy.
#define DEF_OPACITY_MIN				  0
#define DEF_OPACITY_MAX				  255

// Default opacity-value used for streamlines.
#define DEF_OPACITY					  255

// Time between two timer ticks(in milliseconds).
#define DEF_ROTATIONTIMER_ELAPSE	  20
#define DEF_ANIMATIONTIMER_ELAPSE	  20	

// Minimal difference value when 2 float values are considered to be different.
#define DEF_FLT_CHANGE_MIN			  0.0001f

// Max values for progress control.
#define DEF_UPDATEDLISTSPROGRESS_MAX  60
#define DEF_UPDATEPLISTSPROGRESS_MAX  1000
#define DEF_REPLTEXTURESPROGRESS_MAX  60
#define DEF_RECALCPROGRESS_MAX		  200
#define DEF_CREATEFIELDPROGRESS_MAX   m_XDim*10
#define DEF_SAVELOADDATA_MAX		  m_XDim*2

// Title string for progress dialog.
#define DEF_UPDATEPLISTSPROGRESS_STR  "Updating data"
#define DEF_UPDATEDLISTSPROGRESS_STR  "Updating display lists"
#define DEF_REPLTEXTURESPROGRESS_STR  "Updating texture maps"
#define DEF_CREATEFIELDPROGRESS_STR	  "Calculating sample field"
#define DEF_RECALCPROGRESS_STR		  "Calculating streamlines"
#define DEF_LOADDATA_STR			  "Loading data"
#define DEF_SAVEDATA_STR			  "Saveing data"

// Default view direction vector, light direction vector.
#define DEF_VIEW					  CVector3D(0.0f,0.0f,-1.0f)
#define DEF_LIGHT					  DEF_VIEW

// Default axis.
#define DEF_AXIS_X					  CVector3D(1.0f,0.0f,0.0f)
#define DEF_AXIS_Y					  CVector3D(0.0f,1.0f,0.0f)
#define DEF_AXIS_Z					  CVector3D(0.0f,0.0f,1.0f)

// Default flag values.
#define DEF_ISKEYPRESSED			  false
#define DEF_ISINCREMENTIAL			  true
#define DEF_ISDRAWBOUNDARIES		  true
#define DEF_ISDEPTHCUEING			  true
#define DEF_ISROTATING				  false
#define DEF_ISPLACESEEDPOINT		  false
#define DEF_ISENDTAP				  true

#define DEF_ISLBUTTONDOWN			  false
#define DEF_ISRBUTTONDOWN			  false

#define DEF_ISBUSY					  false
#define DEF_ISANTIALIASING			  false
#define DEF_ISSHOWAXIS				  false
#define DEF_ISTRANSPFUNC			  false
#define DEF_ISMODIFIED				  true
#define DEF_ISANIMATION				  false
#define DEF_ISGAP					  true

#define DEF_ISMAGICVOLUME			  true
#define DEF_USEDISPLAYLISTS			  false
#define DEF_ISCORRECTORDER			  true
#define DEF_ISSAMPLEADDINFO			  false
#define DEF_ISTAPERING				  false
#define DEF_ISADDINFO				  false
#define DEF_ISFOCUSCONTEXT			  false
#define DEF_ISSHOWLEGEND			  true
#define DEF_ISARROWS				  false

#define DEF_ISSHOWBLOCK				  false
#define DEF_ISTEX					  true

// Default flag which indicates if context streamlines should
// be cut in focus.
#define DEF_ISCUT_LINES				  true

// Default resolution flags.
#define DEF_ISRES_0					  true
#define DEF_ISRES_1					  true
#define DEF_ISRES_2					  true
#define DEF_ISRES_3					  true
#define DEF_ISRES_4					  true

// 100%.
#define DEF_PERC_100				  100.0f

// Default title of error message box.
#define DEF_ERROR_STR				  "Error"

// Default status bar string.
#define DEF_READY_STR				  "Ready"

// Default minimal distance between streamline samplepoints
// in % of field x-range
#define DEF_MINDIST_PERC			  0.80f
	
// X,y,z-dimension field range.

#define DEF_XRANGE					  GetDocument()->GetXRange()
#define DEF_YRANGE					  GetDocument()->GetYRange()
#define DEF_ZRANGE					  GetDocument()->GetZRange()

// half field range.
#define DEF_XRANGEDIV2				  DEF_XRANGE * 0.5f
#define DEF_YRANGEDIV2				  DEF_YRANGE * 0.5f
#define DEF_ZRANGEDIV2				  DEF_ZRANGE * 0.5f

// Offset values
#define DEF_XOFFS					  (DEF_XRANGEDIV2 + GetDocument()->GetMinXPos())
#define DEF_YOFFS					  (DEF_YRANGEDIV2 + GetDocument()->GetMinYPos())
#define DEF_ZOFFS					  (DEF_ZRANGEDIV2 + GetDocument()->GetMinZPos())

// Dimension with minimal range between specified dimensions.
#define DEF_MINRANGE_DIM_XY			  min(DEF_XRANGE,DEF_YRANGE)
#define DEF_MINRANGE_DIM_XZ			  min(DEF_XRANGE,DEF_ZRANGE)
#define DEF_MINRANGE_DIM_YZ			  min(DEF_YRANGE,DEF_ZRANGE)

#define DEF_MINRANGE_DIM_XYZ		  min(DEF_MINRANGE_DIM_XY,DEF_MINRANGE_DIM_XZ)

// Default magic volumes size.
#define DEF_MAGICCUBE_SIZE			  DEF_MINRANGE_DIM_XYZ / 3.0f
#define DEF_MAGICSPHERE_RADIUS		  DEF_MINRANGE_DIM_XYZ / 6.0f
#define DEF_MAGICRECT_LENGTH		  DEF_XRANGE / 3.0f
#define DEF_MAGICRECT_WIDTH			  DEF_YRANGE / 3.0f
#define DEF_MAGICRECT_HEIGHT		  DEF_ZRANGE / 3.0f

// Boundaries line width.
#define DEF_BOUNDARIES_LINEWIDTH	 2.0f
// Magic volume line width.
#define DEF_MAGICVOL_LINEWIDTH		 2.0f
// Axis line-widths.
#define DEF_AXIS_LINEWIDTH			 2.0f
// Linewidth when key is pressed.
#define DEF_KEYPR_LINEWIDTH			 3.5f

// Boundaries rgba-values.
#define DEF_BOUNDARIES_RGBA			 {255,0,0,255}
// Magic volume rgba-values.
#define DEF_MAGICVOL_RGBA			 {0,255,0,255}

// Axis rgba-values.
#define DEF_AXIS_X_RGBA				 {255,255,0,255}
#define DEF_AXIS_Y_RGBA				 {255,0,255,255}
#define DEF_AXIS_Z_RGBA				 {0,255,255,255}

// Rgba-value when key is pressed.
#define DEF_KEYPR_RGBA			 {255,255,255,255}

// Offset values.
#define DEF_AXIS_OFFS				 20.0f
#define DEF_SEEDPCROSS_OFFS			 0.0f
#define DEF_MAGICVOL_AXIS_OFFS		 0.0f

// DSepGrid-Validation types:

// Startpoint of new streamline: Test with dSep.
#define DEF_TYPE_START				 0
// Normal streamline samplepoint: Test with dSep * dTest.
#define DEF_TYPE_LINE				 1

// Default context streamline resolution in %.
#define DEF_CONTEXT_RES				 70
// Default context opacity
#define DEF_CONTEXT_OPACITY			 100
// Default context lineWidth
// Default context opacity
#define DEF_CONTEXT_LWIDTH			 100

// Highest and lowest resolutions.
#define DEF_RES_MIN					 0
#define DEF_RES_MAX					 100

// Default number of different streamline resolutions.
#define DEF_RESOLUTION_NUM			 10

// Bitmap fonts height and width.
#define DEF_CHAR_HEIGHT				 12
#define DEF_CHAR_WIDTH				 10

// Legend lines length.
#define DEF_LINES_LENGTH			 50

// Default legend xy-pos
#define DEF_LEGEND_XPOS				 10
#define DEF_LEGEND_YPOS				 220

// Default legend text rgba-value.
#define DEF_TEXT_RGBA				 {200,200,255,255}

// Default background color.
#define DEF_BK_COLOR_R				 0
#define DEF_BK_COLOR_G				 0
#define DEF_BK_COLOR_B				 0.2

//! Default size of additional array of samplepoints.
#define DEF_ADDITIONAL_SIZE			 0

//! Default regions material.
#define DEF_REGION1_AMB				 {0,0,50}
#define DEF_REGION1_DIF				 {0,0,255}
#define DEF_REGION1_SPEC			 {255,255,255}

#define DEF_REGION2_AMB				 {0,50,0}
#define DEF_REGION2_DIF				 {0,255,0}
#define DEF_REGION2_SPEC			 {255,255,255}

#define DEF_REGION3_AMB				 {50,0,0}
#define DEF_REGION3_DIF				 {255,0,0}
#define DEF_REGION3_SPEC			 {255,255,255}

// Default material for block(block dataset only).
#define DEF_BLOCK_AMB				 {100,100,100}
#define DEF_BLOCK_DIF				 {255,255,255}
#define DEF_BLOCK_SPEC				 {255,255,255}

// Default positions for regions.
#define DEF_REGION1_POS				 0.33
#define DEF_REGION2_POS				 0.66
#define DEF_REGION3_POS				 1.0

// Default end tapering width.
#define DEF_ENDTAPWIDTH				 5

// Default background color and bar color for histograms.
#define DEF_HISTO_BKCOLOR			 {100,100,150}
#define DEF_HISTO_BARCOLOR			 {100,200,100}

// Number of intervals for histograms
#define DEF_INTERV_NUM				 100

// CHistoDataStream define, stands for 
// "take scalars for all regions".
#define DEF_ALL_REGIONS				 -1

// Index of focus in CHistoDataStream.
#define DEF_FOCUS_INDEX				DEF_REGIONS_NUM
// Index of context in CHistoDataStream.
#define DEF_CONTEXT_INDEX			DEF_REGIONS_NUM + 1

// Colors for histogram bar colors in "Edit Regions Dialog".
#define DEF_BARCOL_REG1				{255, 0, 0}
#define DEF_BARCOL_REG2				{0,255,0}
#define DEF_BARCOL_REG3				{0,0,255}
#define DEF_BARCOL_REG4				{255,255,0}
#define DEF_BARCOL_REG5				{0,255,255}

// Number of histogram images.
#define DEF_HISTOIMG_NUM			DEF_REGIONS_NUM + 3

// Number of values defining the continous transfer function.
#define DEF_CONTTRANSFVALUES_NUM	3

// Indeces for min-max-positions in 
// "Continous Transfer Function Dialog".
#define DEF_MINPOS_INDEX			0
#define DEF_INTERPOS_INDEX			1
#define DEF_MAXPOS_INDEX			2

// Default values for continuous transfer function 
// streamline resolution.
#define DEF_CONTTRANSF_RES			100

// Number of keys.
#define DEF_KEYS_NUM				255

// Maximal and minimal linewidth.
#define DEF_LINEWIDTH_MAX			10.0f
#define DEF_LINEWIDTH_MIN			1.0f

// Type of animation.
#define DEF_GROWING_INDEX			0
#define DEF_TRANSP_INDEX			1

#define DEF_ALPHATBL_MAXSIZE		1000

#define DEF_DEINTENSITY_FACTOR		0.025f
#define DEF_INTENSITY_INCR_FACTOR	1.0f
#define DEF_DEPTH_INTENSITY			155
#define DEF_INTENSITY_MIN			DEF_INTENSITYSLIDER_MIN	
#define DEF_INTENSITY_MAX			DEF_INTENSITYSLIDER_MAX

#define DEF_DATA_SIZE				4
#define DEF_HEADER_SIZE				80
#define DEF_BUF_SIZE				40

#define DEF_CONE_SLICES_NUM			20
#define DEF_CONE_STACKS_NUM			1

#define DEF_ARROW_LENGTH			4.0f
#define DEF_ARROW_WIDTH				2.0f

#define DEF_PERIODLENGTH_FACTOR		2.0f
#define DEF_ABSORPTION_FACTOR		1.0f

// Context changing factor.
#define DEF_CONTEXTCHANGE_FACTOR	1.0f

// Number of region properties
#define DEF_REGPROP_NUM				DEF_REGIONS_NUM + 1

// Offset for dsep-grid to assure that new samplepoint falls into valid range.
#define DEF_DSEPGRID_OFFS			1

// Maximal range in xyz-dimension.
#define DEF_MAXRANGE				max(DEF_ZRANGE,max(DEF_XRANGE,DEF_YRANGE))

// Default string size.
#define DEF_STR_SIZE				255

// Block texture height and width.
#define DEF_BLTEX_HEIGHT			128
#define DEF_BLTEX_WIDTH				128

// Texture name.
#define DEF_BLTEX_NAME				"tex/block.bmp"

// Bitmap header size
#define DEF_BMP_HEADER_SIZE				54

#endif /* !DEFINES_H__ */
