// Flow3DView.cpp : implementation of the CFlow3DView class
//

#include "stdafx.h"
#include "Flow3D.h"

#include "Flow3DDoc.h"
#include "Flow3DView.h"
#include "MyHighlightDialog.h"
#include "MyDiffuseCorrectionDialog.h"
#include "MyAddInfoDialog.h"
#include "MyLightTypeDialog.h"
#include "MyLineStyleDialog.h"
#include "MyFieldSamplingDialog.h"
#include "MyStatisticsDialog.h"
#include "Trilinear.h"
#include "MyProgressDialog.h"
#include "MyPerformanceOptDialog.h"
#include "MyFocusContextDialog.h"
#include "MyDepthCodingDialog.h"
#include "MyAnimationDialog.h"
#include "MyViewVecDialog.h"
#include "MyBlockDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Texture matrix.
GLfloat texMatrix[16] = {0,   0,   0,  0,
						 0,   0,   0,  0,
						 0,   0,   0,  0,
						 0.5f,0.5f,0,  1};

// Current rotation matrix.
GLfloat rotMatrix[16] = {1, 0, 0, 0,
						 0, 1, 0, 0,
						 0, 0, 1, 0,
						 0,	0, 0, 1};

// Old rotation matrix.
GLfloat oldRotMatrix[16] = {1, 0, 0, 0,
							0, 1, 0, 0,
						    0, 0, 1, 0,
						    0, 0, 0, 1};

//static GLubyte blTex[DEF_BLTEX_HEIGHT*DEF_BLTEX_WIDTH*4];

CSinTable CFlow3DView::m_SinTable(DEF_ANGLE_0F, DEF_ANGLE_360F, DEF_SINTABLE_WIDTH);

// Axis rgba-values
GLubyte xAxisRgba[4] = DEF_AXIS_X_RGBA;
GLubyte yAxisRgba[4] = DEF_AXIS_Y_RGBA;
GLubyte zAxisRgba[4] = DEF_AXIS_Z_RGBA;

// Rotation factor: for conversion between deg and rad.
const float ROTFACTOR = DEF_ANGLE_180F/PI;
// Float factor: for conversion between ubyte and float.
const float FLOATFACTOR = 255.0f;

/////////////////////////////////////////////////////////////////////////////
// CFlow3DView

IMPLEMENT_DYNCREATE(CFlow3DView, CView)

BEGIN_MESSAGE_MAP(CFlow3DView, CView)
	//{{AFX_MSG_MAP(CFlow3DView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_DRAWINGSTYLE_SHOWHIDEBOUNDARIES, OnShowHideBoundaries)
	ON_COMMAND(ID_OPTIONS_ENABLEDISABLEDEPTHCUEING, OnEnableDisableDepthCueing)
	ON_WM_TIMER()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_OPTIONS_PLACESEEDPOINT, OnPlaceSeedPoint)
	ON_COMMAND(ID_OPTIONS_SETADDITIONALINFO, OnSetAddInfo)
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_COMMAND(ID_OPTIONS_SETMATERIAL, OnSetMaterial)
	ON_COMMAND(ID_OPTIONS_SETHIGHLIGHT, OnSetHighlight)
	ON_COMMAND(ID_OPTIONS_SETDIFFUSECORRECTION, OnSetDiffuseCorrection)
	ON_COMMAND(ID_OPTIONS_SHOWHIDEADDITIONALINFO, OnShowHideAddInfo)
	ON_COMMAND(ID_OPTIONS_SETLIGHTTYPE, OnSetLightType)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_ENABLEDISABLEDEPTHCUEING, OnUpdateEnableDisableDepthCueing)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_PLACESEEDPOINT, OnUpdatePlaceSeedPoint)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SHOWHIDEADDITIONALINFO, OnUpdateShowHideAddInfo)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_SHOWHIDEBOUNDARIES, OnUpdateShowHideBoundaries)
	ON_COMMAND(ID_OPTIONS_SETLINESTYLE, OnSetLineStyle)
	ON_COMMAND(ID_DRAWINGSTYLE_SHOWHIDEAXIS, OnShowHideAxis)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_SHOWHIDEAXIS, OnUpdateShowHideAxis)
	ON_COMMAND(ID_OPTIONS_STATISTICS, OnStatistics)
	ON_COMMAND(ID_DRAWINGSTYLE_ENABLEDISABLEANIMATION, OnEnableDisableAnimation)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_ENABLEDISABLEANIMATION, OnUpdateEnableDisableAnimation)
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_COMMAND(ID_OPTIONS_SETPERFORMANCEOPTIONS, OnSetPerformanceOptions)
	ON_COMMAND(ID_DRAWINGSTYLE_SHOWHIDEMAGICVOLUME, OnShowHideMagicVolume)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_SHOWHIDEMAGICVOLUME, OnUpdateShowHideMagicVolume)
	ON_COMMAND(ID_DRAWINGSTYLE_SETFOCUSCONTEXTOPTIONS, OnSetFocusContextOptions)
	ON_COMMAND(ID_DRAWINGSTYLE_STARTENDFOCUSCONTEXTMODE, OnStartEndFocusContextMode)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_STARTENDFOCUSCONTEXTMODE, OnUpdateStartEndFocusContextMode)
	ON_WM_RBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_COMMAND(ID_OPTIONS_SHOWHIDELEGEND, OnShowHideLegend)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SHOWHIDELEGEND, OnUpdateShowHideLegend)
	ON_COMMAND(ID_DRAWINGSTYLE_SETBACKGROUND, OnSetBackground)
	ON_COMMAND(ID_DRAWINGSTYLE_ZOOMIN, OnZoomIn)
	ON_COMMAND(ID_DRAWINGSTYLE_RESETSIZE, OnResetSize)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_DIFORIENT, OnShowHistoDifOrient)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_ORIENTATIONX, OnShowHistoOrientX)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_ORIENTATIONY, OnShowHistoOrientY)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_ORIENTATIONZ, OnShowHistoOrientZ)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_POSITIONX, OnShowHistoPosX)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_POSITIONY, OnShowHistoPosY)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_POSITIONZ, OnShowHistoPosZ)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_VELOCITY, OnShowHistoVelocity)
	ON_COMMAND(ID_OPTIONS_SHOWHISTOGRAM_DIFVELOCITY, OnShowHistoDifVelocity)
	ON_COMMAND(ID_DRAWINGSTYLE_SETDEPTHCODING, OnSetDepthCoding)
	ON_COMMAND(ID_DRAWINGSTYLE_SETANIMATIONOPTIONS, OnSetAnimationOptions)
	ON_COMMAND(ID_OPTIONS_SETVIEWDIRECTION, OnSetViewDirection)
	ON_COMMAND(ID_DRAWINGSTYLE_SETBLOCKPARAMETERS, OnSetBlockParameters)
	ON_COMMAND(ID_DRAWINGSTYLE_SHOWHIDEBLOCK, OnShowHideBlock)
	ON_UPDATE_COMMAND_UI(ID_DRAWINGSTYLE_SHOWHIDEBLOCK, OnUpdateShowHideBlock)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFlow3DView construction/destruction

CFlow3DView::CFlow3DView()
{
	// TODO: add construction code here
	m_pDC = NULL;
}

CFlow3DView::~CFlow3DView()
{
	if(doc->m_AlphaTable != NULL)
		delete doc->m_AlphaTable;
	
	if(doc->m_MagicVolume != NULL)
		delete doc->m_MagicVolume;

	m_HistoDialog.DestroyWindow();
}

BOOL CFlow3DView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

    return CView::PreCreateWindow(cs);
}

void CFlow3DView::Init()
{
	// Init opengl.
	GLInit();

	// Init member variables.

	// Init view and light vector.
	m_View = DEF_VIEW;
	m_Light = DEF_LIGHT;

	// doc is a pointer to document(So I don't have to call GetDocument()
	// all the time.
	doc = GetDocument();

	// Set hightlight-shininess and diffuse correction factor.
	m_Glossiness = DEF_GLOSSINESS;
	m_DiffuseCorr = DEF_DIFFUSE_CORRECTION;

	// Initialise flags.
	m_IsDrawBoundaries = DEF_ISDRAWBOUNDARIES;
	m_IsDepthCueing = DEF_ISDEPTHCUEING;
	m_IsRotating = DEF_ISROTATING;
	m_IsLButtonDown = DEF_ISLBUTTONDOWN;
	m_IsRButtonDown = DEF_ISRBUTTONDOWN;
	m_IsBusy = DEF_ISBUSY;
	m_IsAntiAliasing = DEF_ISANTIALIASING;
	m_IsShowAxis = DEF_ISSHOWAXIS;
	m_IsTranspFunc = DEF_ISTRANSPFUNC;
	m_Modified = DEF_ISMODIFIED;
	m_IsGap = DEF_ISGAP;
	m_IsShowLegend = DEF_ISSHOWLEGEND;

	// Init key-array.
	for(int i=0;i<DEF_KEYS_NUM;i++)
		m_IsKeyPressed[i] = DEF_ISKEYPRESSED;

	// Init display list indexes.
	m_TheStreamlines = 0;
	m_TheVField = 0;

	// Initialise rotation angle.
	m_RotAngle = DEF_ANGLE_0F;
	// Set default rotation axis to x-axis.
	m_RotAxis = DEF_AXIS_X;
	
	// Initialise light type with headlight.
	m_LightType = DEF_HEADLIGHT_INDEX;

	// Set default gap size.
	m_GapSize = DEF_GAP_SIZE;
	
	// Init transparency function options and create alpha table.
	m_TranspFuncType = DEF_HAZE_EXP_INDEX;
	m_Absorption = DEF_ABSORPTION;
	// Initialise transparency function period length.
	doc->m_PeriodLength = DEF_PERIOD_LENGTH;
	doc->m_AlphaTable = CreateAlphaTable();

	// Initialise field boundaries options.
	GLubyte bRgba[4] = DEF_BOUNDARIES_RGBA;
	doc->m_Boundaries.SetRgba(bRgba);
	doc->m_Boundaries.SetLineWidth(DEF_BOUNDARIES_LINEWIDTH);

	// Initialise coordinate system axis object.
	m_Axis.SetXAxisRgba(xAxisRgba);
	m_Axis.SetYAxisRgba(yAxisRgba);
	m_Axis.SetZAxisRgba(zAxisRgba);

	m_Axis.SetLineWidth(DEF_AXIS_LINEWIDTH);
	m_Axis.SetOffs(DEF_AXIS_OFFS);

	// Initialise focus/context objects
	InitFoCoObj();

	// Set magic volume color and line width.
	GLubyte mcRgba[4] = DEF_MAGICVOL_RGBA;
	doc->m_MagicVolume->SetRgba(mcRgba);
	doc->m_MagicVolume->SetLineWidth(DEF_MAGICVOL_LINEWIDTH);

	// Initialise background color.
	m_BKColor[0] = DEF_BK_COLOR_R;
	m_BKColor[1] = DEF_BK_COLOR_G;
	m_BKColor[2] = DEF_BK_COLOR_B;
	m_BKColor[3] = 1.0f;

	// Init depth coding parameters.
	m_DepthType = DEF_DEPTHCUEING_INDEX;
	
	for(i=0;i<3;i++)
		m_DepthColor[i] = (GLubyte)(m_BKColor[i] * DEF_COLOR_MAX);
	
	// Initialise histogram dialog data stream.
	m_HistoDialog.InitDataStreams(doc->GetXList());
	
	// Create streamline illumination texture.
	m_TexGenerator.SetMapSize(DEF_TEXTURE_SIZE);
	CreateTextures();

	// Init old member variables(needed in mouse move routine).
	InitOldMembers();

	// Initialise field angle dialog parameters.
	m_XAngle = DEF_ANGLE_0F;
	m_YAngle = DEF_ANGLE_0F;
	m_ZAngle = DEF_ANGLE_0F;

	// Initialise arrow parameters.
	m_ArrowWidth = DEF_ARROW_WIDTH;
	m_ArrowLength = DEF_ARROW_LENGTH;
	m_ArPeriod = DEF_AR_PERIOD_LENGHT;
	m_ArType = DEF_AR_LEND_INDEX;

	// Init block parameters(only important for block-dataset).
	InitBlockParams();
}

BOOL CFlow3DView::bSetupPixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
		1,                              // version number
		PFD_DRAW_TO_WINDOW |            // support window
		  PFD_SUPPORT_OPENGL |          // support OpenGL
		  PFD_DOUBLEBUFFER,             // double buffered
		PFD_TYPE_RGBA,                  // RGBA type
		24,                             // 24-bit color depth
		0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // no alpha buffer
		0,                              // shift bit ignored
		0,                              // no accumulation buffer
		0, 0, 0, 0,                     // accum bits ignored
		32,                             // 32-bit z-buffer
		0,                              // no stencil buffer
		0,                              // no auxiliary buffer
		PFD_MAIN_PLANE,                 // main layer
		0,                              // reserved
		0, 0, 0                         // layer masks ignored
	};

	int pixelformat;

	if ( (pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0 )
	{
		MessageBox("ChoosePixelFormat failed");
		return FALSE;
	}

	if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
	{
		MessageBox("SetPixelFormat failed");
		return FALSE;
	}

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CFlow3DView drawing

void CFlow3DView::OnDraw(CDC* pDC)
{
	CFlow3DDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
	DrawScene();	
}

void CFlow3DView::DrawStreamlines(int direction)
{
	bool reverse = false;

	CObList * pieces = GetPiecesList(reverse,direction);
	
	POSITION pos;

	if(reverse)
		pos = pieces->GetTailPosition();
	else
		pos = pieces->GetHeadPosition();

	// to display gap size of m_GapSize on both sides of the line, 
	// the black gap line width
	// must be line width + gap size * 2;
	float doubleGapSize = m_GapSize * 2.0f;

	bool isFoContext = doc->GetIsFocusContext();
	int contextOpacity = doc->m_ContextProp.opacity;
	float contextLineWidth = doc->m_ContextProp.lineWidth;
	int alphaWidth = doc->m_AlphaTblWidth;

	bool isEndTap = doc->GetIsEndTap();
	float *endTapTable = doc->m_EndTapTable;
	bool isTranspF = (m_IsTranspFunc || 
					  (doc->m_IsAnimation && doc->m_AnimType == DEF_TRANSP_INDEX));
	
	bool isArrows = doc->m_IsArrows;
	float lineWidth = 0;

	float arrowFactor = DEF_XRANGE / DEF_PERC_100;

	float arrowLength =m_ArrowLength * arrowFactor;
	float arrowWidth = m_ArrowWidth * arrowFactor;

	CSamplePoint *start;
	CSamplePoint *end;
	CStreamlinePiece *piece;

	GLubyte *table = doc->m_AlphaTable;
	GLubyte rgba[4];

	//int n = pieces->GetCount();int i;
	CStreamlineMaterial *coneMaterial;

	while(pos != NULL)
	{
		if(reverse)
			piece = (CStreamlinePiece *)pieces->GetPrev(pos);
		else
			piece = (CStreamlinePiece *)pieces->GetNext(pos);
		
		if(piece->m_IsDraw)
		{
			start = piece->m_StartPoint;
			end = piece->m_EndPoint;

			lineWidth = piece->m_LineWidth;

			for(int i=0;i<4;i++)
				rgba[i] = piece->m_Color[i];
		
			// Transparency function.
			if(((!isFoContext) || piece->m_IsInside) && isTranspF)
				rgba[3] = 
					rgba[3]*table[piece->m_Index%alphaWidth]/DEF_OPACITY_MAX;

			if(isFoContext && (!piece->m_IsInside))
			{
				lineWidth = contextLineWidth;
				glBindTexture(GL_TEXTURE_2D, texName[DEF_REGPROP_NUM]);
				coneMaterial = &doc->m_ContextProp.material;

				for(int i=0;i<3;i++)
					rgba[i] = DEF_COLOR_MAX;
				rgba[3] = contextOpacity;
			}	
			else
			{
				glBindTexture(GL_TEXTURE_2D, texName[piece->m_RegIndex]);
				coneMaterial = &doc->m_RegProp[piece->m_RegIndex].material;
			}

			// End tapering.
			if(isEndTap)
				lineWidth *= endTapTable[piece->m_EndTapIndex];
			
//baseColor[0] = 255;baseColor[1] = i*255/n;
//baseColor[2] = baseColor[1];baseColor[3] = 255;
//i++;
			glColor4ubv(rgba);
			glLineWidth(lineWidth);
			
			// Now draw streamline.
			glBegin(GL_LINES);
				glTexCoord3f(start->GetUnitXVec(),start->GetUnitYVec(),
							 start->GetUnitZVec()); 
				glVertex3f(start->GetXPos(),start->GetYPos(),start->GetZPos());
		   		glTexCoord3f(end->GetUnitXVec(),end->GetUnitYVec(),
							 end->GetUnitZVec()); 
				glVertex3f(end->GetXPos(),end->GetYPos(),end->GetZPos());
			glEnd();
		
			// Gap = black line behind illuminated line.
			if(m_IsGap)
			{
				glLineWidth(lineWidth + doubleGapSize);
				glColor4ub(0,0,0,rgba[3]);

				glBegin(GL_LINES);
					glVertex3f(start->GetXPos(),start->GetYPos(),start->GetZPos());
					glVertex3f(end->GetXPos(),end->GetYPos(),end->GetZPos());
				glEnd();
			}

			// Now draw arrows.
			if(isArrows && 
			   (((m_ArType == DEF_AR_LEND_INDEX) && piece->m_IsLineStart) ||
			    ((m_ArType == DEF_AR_PERIOD_INDEX) && ((piece->m_Index%m_ArPeriod) == 0))))
			{
				DrawCone(start,*coneMaterial,rgba,arrowWidth,arrowLength);
			}
		}
    };

	glLineWidth(DEF_LINEWIDTH);
}

void CFlow3DView::DrawScene()
{
	m_IsBusy = true;

	glClearColor(m_BKColor[0], m_BKColor[1], m_BKColor[2], m_BKColor[3]);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	if(m_IsShowLegend)
		DrawLegend();

	m_View = DEF_VIEW;

	// Rotate view vector with current rotation matrix.
	m_View.Mult(rotMatrix);
	
	if(m_LightType == DEF_HEADLIGHT_INDEX)
		m_Light = m_View;

	// Set up texture matrix.
	SetTextureMatrix();

	// Use texture matrix for illumination.
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(texMatrix);

	glMatrixMode(GL_MODELVIEW);

	glPushMatrix();

	// Translate field back by value z-Offs.
	glTranslatef(0,0,-m_ZDist);

	// Rotate field with current rotation matrix.
	glMultMatrixf(rotMatrix);

	// The lighting is needed for the cones and the block.
	// Position must be -m_Light to get same light vector.
	GLfloat light_position[] = 
		{-m_Light.m_XComp,-m_Light.m_YComp,-m_Light.m_ZComp, 0.0 };
		//{0,0,1, 0.0 };

	// Hightlighting for the streamlines is also used for solid geometry.
    glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &m_Glossiness);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

	// Depth cueing.
	if(m_IsDepthCueing)
		SetDepthCueing();
	
	// Line antialiasing.
	if(m_IsAntiAliasing)
		glEnable(GL_LINE_SMOOTH);

	// Rectangle around vector field boundaries.
	if(m_IsDrawBoundaries)
		doc->m_Boundaries.Draw();

	// Color value if key is pressed.
	GLubyte keyPrRgba[4] = DEF_KEYPR_RGBA;

	// Seedpoint placement mode.
	if(doc->m_IsPlaceSeedPoint)
	{
		MyAxisCross *seedPCross = doc->GetSeedPCross();

		seedPCross->SetLineWidth(DEF_AXIS_LINEWIDTH);

		seedPCross->SetXAxisRgba(xAxisRgba);
		seedPCross->SetYAxisRgba(yAxisRgba);
		seedPCross->SetZAxisRgba(zAxisRgba);

		if(m_IsKeyPressed['X'])
		{
			seedPCross->SetXAxisRgba(keyPrRgba);
			seedPCross->SetXLineWidth(DEF_KEYPR_LINEWIDTH);
		}
		else if(m_IsKeyPressed['Y'])
		{
			seedPCross->SetYAxisRgba(keyPrRgba);
			seedPCross->SetYLineWidth(DEF_KEYPR_LINEWIDTH);
		}
		else if(m_IsKeyPressed['Z'])
		{
			seedPCross->SetZAxisRgba(keyPrRgba);
			seedPCross->SetZLineWidth(DEF_KEYPR_LINEWIDTH);
		}

		seedPCross->Draw();
	}
	else
	// Focus/context.
	if(doc->GetIsFocusContext() && doc->GetIsMagicVol())
	{
		GLubyte rgba[4] = DEF_MAGICVOL_RGBA;

		doc->m_MagicVolume->SetRgba(rgba);
		doc->m_MagicVolume->SetLineWidth(DEF_MAGICVOL_LINEWIDTH);

		if(m_IsRButtonDown)
		{
			doc->m_MagicVolume->SetRgba(keyPrRgba);
			doc->m_MagicVolume->SetLineWidth(DEF_KEYPR_LINEWIDTH);
		
			if((doc->m_MagicVolType == DEF_MRECT_INDEX)&&		
			   (m_IsKeyPressed['X'] || m_IsKeyPressed['Y'] || m_IsKeyPressed['Z']))
			{
				doc->m_MagicVolume->SetRgba(keyPrRgba);
				doc->m_MagicVolume->SetLineWidth(DEF_KEYPR_LINEWIDTH);
			}

		}
	
		doc->m_MagicVolume->Draw();

		m_FoContextCross.SetXAxisRgba(xAxisRgba);
		m_FoContextCross.SetYAxisRgba(yAxisRgba);
		m_FoContextCross.SetZAxisRgba(zAxisRgba);
		m_FoContextCross.SetLineWidth(DEF_AXIS_LINEWIDTH);

		if(m_IsKeyPressed['X'])
		{
			m_FoContextCross.SetXLineWidth(DEF_KEYPR_LINEWIDTH);
			m_FoContextCross.SetXAxisRgba(keyPrRgba);
		}
		else if(m_IsKeyPressed['Y'])
		{
			m_FoContextCross.SetYLineWidth(DEF_KEYPR_LINEWIDTH);
			m_FoContextCross.SetYAxisRgba(keyPrRgba);
		}
		else if(m_IsKeyPressed['Z'])
		{
			m_FoContextCross.SetZLineWidth(DEF_KEYPR_LINEWIDTH);
			m_FoContextCross.SetZAxisRgba(keyPrRgba);
		}

		m_FoContextCross.Draw();
	}
	else
	// Draw xyz-axis.
	if(m_IsShowAxis)
		m_Axis.Draw();
	
	// Draws block in block-dataset.
	if(m_IsShowBlock)
		DrawBlock();

	// Draw streamlines.
	if(!doc->m_IsPlaceSeedPoint)
	{
		// Used for drawing arrows(= glutSolidCone).
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		glEnable(GL_TEXTURE_2D);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		int direction = GetMainDirection();

		glPushMatrix();

		// Translation to place streamlines inside vectorfield.
		glTranslatef(-DEF_XOFFS,-DEF_YOFFS,-DEF_ZOFFS);

		if(doc->GetUseDisplayLists() && (!doc->m_IsAnimation) && 
		   (!doc->GetIsFocusContext()))
		{
			if(m_Modified)
			{
				m_Modified = false;
				
				MyProgressDialog progressDialog;
				progressDialog.Create(IDD_PROGRESSDIALOG,this);
				progressDialog.InitProgressDialog(DEF_UPDATEDLISTSPROGRESS_STR,
												  DEF_UPDATEDLISTSPROGRESS_MAX);
				progressDialog.ShowWindow(SW_SHOW);
								
				glDeleteLists(m_TheStreamlines, DEF_DIRECTIONS_NUM);

				m_TheStreamlines = glGenLists(DEF_DIRECTIONS_NUM);

				for(int i=0; i<DEF_DIRECTIONS_NUM; i++)
				{
					progressDialog.m_Progress.StepIt();
					glNewList(m_TheStreamlines + i, GL_COMPILE);
						DrawStreamlines(i);
					glEndList();
				}

				progressDialog.DestroyWindow();
				
			}
			glCallList(m_TheStreamlines + direction);
		}
		else
		{
			DrawStreamlines(direction);
		}

		glPopMatrix();
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);
	}

	if(m_IsAntiAliasing)
		glDisable(GL_LINE_SMOOTH);

	if(m_IsDepthCueing)
		glDisable(GL_FOG);

	glPopMatrix();

	glFinish();
	SwapBuffers(wglGetCurrentDC());

	m_IsBusy = false;
}

/*!
	Clamps value to newVal1 if value < min
	Clamps value to newVal2 if value >= max
*/
void CFlow3DView::Clamp(float &value, float min, float max, float newVal1, float newVal2)
{
	if(value < min)
		value = newVal1;
	else if(value >= max)
		value = newVal2;
}

/////////////////////////////////////////////////////////////////////////////
// CFlow3DView diagnostics

#ifdef _DEBUG
void CFlow3DView::AssertValid() const
{
	CView::AssertValid();
}

void CFlow3DView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CFlow3DDoc* CFlow3DView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFlow3DDoc)));
	return (CFlow3DDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFlow3DView message handlers


void CFlow3DView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	// TODO: Add your specialized code here and/or call the base class

	// Called during streamline calculation
	if(lHint == DEF_HINT_DRAW_NEW_STREAMLINE)
	{
			// rotate while adding new streamlines
			m_RotAxis = DEF_AXIS_Y;
			m_RotAngle = DEF_ANGLE_0F;
			m_RotAngle+= DEF_ONUPD_ANGLEINCR_FACTOR;
			
			Clamp(m_RotAngle,DEF_ANGLE_0F,DEF_ANGLE_360F,
				  m_RotAngle+DEF_ANGLE_360F,m_RotAngle-DEF_ANGLE_360F);
		
			SetRotMatrix(oldRotMatrix);
			SetRotMatrix(rotMatrix);

			doc->m_IsPlaceSeedPoint = false;
			m_Modified = true;
		
			DrawScene();
	
	}
	// Called if new vectorfield is specified.
	else if(lHint == DEF_HINT_INIT_SCENE)
	{
		// Set projection matrix.
		float fAspect;

		if (m_oldRect.bottom)
			fAspect = (GLfloat)m_oldRect.right/m_oldRect.bottom;
		else    // Don't divide by zero, not that we should ever run into that...
			fAspect = 1.0f;

		float farPlane = DEF_NEARPLANE + DEF_MAXRANGE + DEF_FARPLANE;

		glMatrixMode(GL_PROJECTION);	
		glLoadIdentity();
		gluPerspective(DEF_VIEWANGLE, fAspect, DEF_NEARPLANE, farPlane);
	
		glMatrixMode(GL_MODELVIEW);
	
		// Set rotation matrix and old rotation matrix to identity.
		glLoadIdentity();
		glGetFloatv(GL_MODELVIEW_MATRIX,rotMatrix);
		glGetFloatv(GL_MODELVIEW_MATRIX,oldRotMatrix);
		m_RotAxis = CVector3D(1.0f,0.0f,0.0f);
		m_RotAngle = 0.0f;

		InitObjects();	

		m_OldMinPos = doc->m_MinPos;
		m_OldMaxPos = doc->m_MaxPos;

		doc->UpdatePiecesLists();
					
		DrawScene();
	} 
	else if(lHint == DEF_HINT_REDRAW_SCENE)
	{
		doc->UpdatePiecesLists();
		m_Modified = true;
		m_OldMinPos = doc->m_MinPos;
		m_OldMaxPos = doc->m_MaxPos;

		DrawScene();
	}
}

int CFlow3DView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	Init(); // initialize OpenGL

   	return 0;
}

void CFlow3DView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	if(cy > 0)
	{
		glViewport(0, 0, cx, cy);

		if((m_oldRect.right > cx) || (m_oldRect.bottom > cy))
			RedrawWindow();

			m_oldRect.right = cx;
			m_oldRect.bottom = cy;

			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(DEF_VIEWANGLE, (GLdouble)cx/cy, DEF_NEARPLANE, DEF_FARPLANE);
			glMatrixMode(GL_MODELVIEW);
	}
}

void CFlow3DView::OnDestroy() 
{
	
	// TODO: Add your message handler code here
	HGLRC   hrc;

	hrc = ::wglGetCurrentContext();

	::wglMakeCurrent(NULL,  NULL);

	if (hrc)
		::wglDeleteContext(hrc);

	if (m_pDC)
		delete m_pDC;
	
	CView::OnDestroy();
}


BOOL CFlow3DView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	//return CView::OnEraseBkgnd(pDC);
	return TRUE;
}

void CFlow3DView::OnSetMaterial() 
{
	// TODO: Add your command handler code here

	MyMaterialDialog materialDialog;

	// Set material with highest region index to default material.
	materialDialog.SetMaterial(doc->m_RegProp[DEF_REGIONS_NUM].material);

	if(materialDialog.DoModal()==IDOK)
	{
		doc->m_RegProp[DEF_REGIONS_NUM].material = materialDialog.GetMaterial();
		ReplaceTextures();

		DrawScene();
	}
}

void CFlow3DView::CreateTextures()
{
	MyProgressDialog progressDialog;

	progressDialog.Create(IDD_PROGRESSDIALOG,this);

	progressDialog.InitProgressDialog(DEF_REPLTEXTURESPROGRESS_STR,
									  DEF_REPLTEXTURESPROGRESS_MAX);
	progressDialog.ShowWindow(SW_SHOW);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	// Generate textures for regions and one for context.
	glGenTextures(DEF_REGPROP_NUM + 1, texName);

	for(int i=0;i<DEF_REGPROP_NUM + 1;i++)
	{
		progressDialog.m_Progress.StepIt();

		if(i < DEF_REGPROP_NUM)
			m_TexGenerator.GenTextureMap(doc->m_RegProp[i].material,
										 m_DiffuseCorr,m_Glossiness);
		else
			m_TexGenerator.GenTextureMap(doc->m_ContextProp.material,
										 m_DiffuseCorr,m_Glossiness);
	
		glBindTexture(GL_TEXTURE_2D, texName[i]);
	
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_TexGenerator.GetMapSize(), 
					 m_TexGenerator.GetMapSize(), 0, GL_RGBA, GL_UNSIGNED_BYTE, 
					 m_TexGenerator.GetTextureMap());
	}

	progressDialog.DestroyWindow();
}	


void CFlow3DView::ReplaceTextures()
{
	MyProgressDialog progressDialog;
	progressDialog.Create(IDD_PROGRESSDIALOG,this);
	progressDialog.InitProgressDialog(DEF_REPLTEXTURESPROGRESS_STR,
									  DEF_REPLTEXTURESPROGRESS_MAX);

	progressDialog.ShowWindow(SW_SHOW);

	for(int i=0;i<DEF_REGPROP_NUM + 1;i++)
	{
		progressDialog.m_Progress.StepIt();

		if(i < DEF_REGPROP_NUM)
			m_TexGenerator.GenTextureMap(doc->m_RegProp[i].material,
										 m_DiffuseCorr,m_Glossiness);
		else
			m_TexGenerator.GenTextureMap(doc->m_ContextProp.material,
										 m_DiffuseCorr,m_Glossiness);

		glBindTexture(GL_TEXTURE_2D, texName[i]);
		
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, m_TexGenerator.GetMapSize(), 
						m_TexGenerator.GetMapSize(), GL_RGBA, GL_UNSIGNED_BYTE, 
						m_TexGenerator.GetTextureMap());

	}
	progressDialog.DestroyWindow();
}

void CFlow3DView::OnShowHideBoundaries() 
{
	// TODO: Add your command handler code here
	m_IsDrawBoundaries = (m_IsDrawBoundaries==false);
	DrawScene();
}

void CFlow3DView::OnEnableDisableDepthCueing() 
{
	// TODO: Add your command handler code here
	m_IsDepthCueing = (m_IsDepthCueing==false);
	DrawScene();
}

void CFlow3DView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	
	switch(nIDEvent)
	{
		case DEF_ROTATIONTIMER_INDEX:
			
			SetRotMatrix(oldRotMatrix);
			// take dragging startpoint - endpoint as vector: normal vector = axis
			m_RotAxis = CVector3D(m_CurrentMousePos.y - m_RButDblClkPos.y,
								  m_CurrentMousePos.x - m_RButDblClkPos.x, 0);
			
			// normal vector lenght = angle
			m_RotAngle = m_RotAxis.Magnitude()*DEF_ONTIMER_ANGLEINCR_FACTOR;

			m_RotAxis.Normalize();

			// also rotate rotation axis
			m_RotAxis.Mult(rotMatrix);
			
			// clamp rotangle into [0,360]
			Clamp(m_RotAngle,DEF_ANGLE_0F,DEF_ANGLE_360F,
				  m_RotAngle+DEF_ANGLE_360F,m_RotAngle-DEF_ANGLE_360F);

			SetRotMatrix(rotMatrix);
			if((m_HasFocus)&&(!m_IsBusy))
				DrawScene();
			break;

		case DEF_ANIMATIONTIMER_INDEX:
			doc->DoInsideTest();
			if((m_HasFocus)&&(!m_IsBusy)&&(!m_IsRotating))
				DrawScene();
			
			break;

	}
		
	CView::OnTimer(nIDEvent);
}

void CFlow3DView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	if((!m_IsRotating) && m_IsLButtonDown)
	{
		CVector3D vec = 
			CVector3D(point.x-m_LButtonDownPos.x,m_LButtonDownPos.y-point.y,0);

		if(IsLButtonRot())
		{
			// take dragging startpoint - endpoint as vector: normal vector = axis
			m_RotAxis = CVector3D(point.y - m_LButtonDownPos.y,
								  point.x - m_LButtonDownPos.x,0);

			RotateField();
			
		}
		else if(m_IsKeyPressed['D'])
		{
			m_ZDist = m_OldZDist + (vec.m_XComp + vec.m_YComp)* 
				DEF_DISTINCR_FACTOR * CalcSizePerc() / DEF_PERC_100;
		}
		else if(m_IsKeyPressed['R'])
		{
			float factor = DEF_CONTEXTCHANGE_FACTOR;
			float res = m_OldContextProp.resPerc + 
						(vec.m_XComp + vec.m_YComp) * factor; 
			
			Clamp(res,DEF_RES_MIN,DEF_RES_MAX,DEF_RES_MIN,DEF_RES_MAX);
			doc->m_ContextProp.resPerc = (int)res;
			doc->UpdateContextProp();
			doc->DoInsideTest();
			m_Modified = true;
		}
		else if(m_IsKeyPressed['O'])
		{
			float factor = DEF_CONTEXTCHANGE_FACTOR;
			float opacity = m_OldContextProp.opacityPerc + 
							(vec.m_XComp + vec.m_YComp) * factor; 
			
			Clamp(opacity,0,DEF_PERC_100,0,DEF_PERC_100);
			doc->m_ContextProp.opacityPerc = (int)opacity;
			doc->UpdateContextProp();
			m_Modified = true;
		}
		else if(m_IsKeyPressed['L'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);

			float factor = DEF_CONTEXTCHANGE_FACTOR;
			float lineWidth = m_OldContextProp.lineWidthPerc + 
							  (vec.m_XComp + vec.m_YComp) * factor; 
		
			Clamp(lineWidth,0,DEF_PERC_100,0,DEF_PERC_100);
			doc->m_ContextProp.lineWidthPerc = (int)lineWidth;
			doc->UpdateContextProp();
			m_Modified = true;
		}
		else if(m_IsKeyPressed['S'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);
			float factor = DEF_CONTEXTCHANGE_FACTOR;
			float speed = m_OldAnimSpeed - 
						  (vec.m_XComp + vec.m_YComp) * factor; 
	
			Clamp(speed,DEF_SPEEDSLIDER_MIN,DEF_SPEEDSLIDER_MAX,
				  DEF_SPEEDSLIDER_MIN,DEF_SPEEDSLIDER_MAX);

			doc->m_AnimSpeed = (int)speed;

			if(doc->m_IsAnimation)
				SetTimer(DEF_ANIMATIONTIMER_INDEX,doc->m_AnimSpeed,NULL);
		}
		else if(m_IsKeyPressed['I'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);

			float factor = DEF_INTENSITY_INCR_FACTOR;

			float intensity = m_OldDeIntensity + 
						      (vec.m_XComp + vec.m_YComp) * factor; 
	
			Clamp(intensity,DEF_INTENSITY_MIN,DEF_INTENSITY_MAX,
				  DEF_INTENSITY_MIN,DEF_INTENSITY_MAX);

			m_DeIntensity = intensity;
		}
		else if(m_IsKeyPressed['P'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);

			float factor = DEF_PERIODLENGTH_FACTOR;

			float periodLength = m_OldPeriodLength + 
						      (vec.m_XComp + vec.m_YComp) * factor; 
	
			Clamp(periodLength,DEF_PERIODLENGTHSLIDER_MIN,DEF_PERIODLENGTHSLIDER_MAX,
				  DEF_PERIODLENGTHSLIDER_MIN,DEF_PERIODLENGTHSLIDER_MAX);

			doc->m_PeriodLength = (int)periodLength;
			
			// Update alpha table.
			if(doc->m_AlphaTable != NULL)
				delete [] doc->m_AlphaTable;

			doc->m_AlphaTable = CreateAlphaTable();
		}
		else if(m_IsKeyPressed['A'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);

			float factor = DEF_ABSORPTION_FACTOR;

			float absorption = m_OldAbsorption - 
						      (vec.m_XComp + vec.m_YComp) * factor; 
	
			Clamp(absorption,DEF_ABSORPTIONSLIDER_MIN,DEF_ABSORPTIONSLIDER_MAX,
				  DEF_ABSORPTIONSLIDER_MIN,DEF_ABSORPTIONSLIDER_MAX);

			m_Absorption = (GLubyte)absorption;
			
			// Update alpha table.
			if(doc->m_AlphaTable != NULL)
				delete [] doc->m_AlphaTable;

			doc->m_AlphaTable = CreateAlphaTable();
		}
		else if(m_IsKeyPressed['B'])
		{
			CVector3D vec = CVector3D(point.x - m_LButtonDownPos.x,
									  m_LButtonDownPos.y - point.y,0);

			float factor = DEF_BLOCK_ZOFFS_FACTOR;

			float min = DEF_BLOCK_ZOFFSSLIDER_MIN / 
					    DEF_BLOCK_ZOFFSSLIDER_FACTOR;
			float max = DEF_BLOCK_ZOFFSSLIDER_MAX / 
					    DEF_BLOCK_ZOFFSSLIDER_FACTOR;

			float zOffs  = m_OldBlZOffs +
						  (vec.m_XComp + vec.m_YComp) * factor; 
	
			Clamp(zOffs,min,max,min,max);
				 
			doc->m_BlZOffs = zOffs;
		}
		else
		{
			if(doc->m_IsPlaceSeedPoint)
			{
				// Update seedpoint cross
				CPoint3D newPos = CalcNewPos(m_OldSeedPCross.GetPosition(),vec);
				doc->GetSeedPCross()->SetPosition(newPos);

			}
			else if(doc->GetIsFocusContext())
			{
				CPoint3D newPos = CalcNewPos(m_OldMagicVolume->GetPosition(),vec);
							
				doc->m_MagicVolume->SetPosition(newPos);
				m_FoContextCross.SetPosition(newPos);

				doc->DoInsideTest();
			}
		}
	}
	else if( m_IsRButtonDown && doc->GetIsFocusContext())
	{
		CVector3D vec = CVector3D(point.x - m_RButtonDownPos.x,
								  m_RButtonDownPos.y - point.y,0);

		float newSize = 0; 

		float newHeight = 0;
		float newWidth = 0;
		float newLength = 0;

		float newRadius = 0;

		if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
			newSize = ((MyCube *)m_OldMagicVolume)->GetSize();
		else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
			newRadius = ((MySphere *)m_OldMagicVolume)->GetRadius();
		else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
		{
			newHeight = ((MyRectangle *)m_OldMagicVolume)->GetHeight();
			newLength = ((MyRectangle *)m_OldMagicVolume)->GetLength();
			newWidth = ((MyRectangle *)m_OldMagicVolume)->GetWidth();
		}

		CalcNewSize(newSize,newRadius,newLength,newWidth,newHeight,vec);		

		if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
			((MyCube *)doc->m_MagicVolume)->SetSize(newSize);
		else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
			((MySphere *)doc->m_MagicVolume)->SetRadius(newRadius);
		else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
		{
			((MyRectangle *)doc->m_MagicVolume)->SetLength(newLength);
			((MyRectangle *)doc->m_MagicVolume)->SetHeight(newHeight);
			((MyRectangle *)doc->m_MagicVolume)->SetWidth(newWidth);
		
		}
		doc->DoInsideTest();
	}
	
	m_CurrentMousePos = point;
	
	if((m_HasFocus)&&(!m_IsBusy))
		DrawScene();

	CView::OnMouseMove(nFlags, point);
}

void CFlow3DView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_IsRButtonDown = false;

	CView::OnRButtonUp(nFlags, point);
}

void CFlow3DView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_IsLButtonDown = false;
		
	if(!m_IsRotating)
	{
		// Set rotation matrix
		SetRotMatrix(oldRotMatrix);

		// Initialise angle + axis
		m_RotAxis = DEF_AXIS_X;
		m_RotAngle = DEF_ANGLE_0F;
	}
	
	DrawScene();

	CView::OnLButtonUp(nFlags, point);
}

void CFlow3DView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDown(nFlags, point);
	m_LButtonDownPos = point;
	m_IsLButtonDown = true;

	// Initialise position values.
	m_OldSeedPCross.SetPosition(doc->GetSeedPCross()->GetPosition());
	m_OldMagicVolume->Init(doc->m_MagicVolume);
	m_OldZDist = m_ZDist;
	m_OldAnimSpeed = doc->m_AnimSpeed;
	m_OldDeIntensity = m_DeIntensity;
	m_OldPeriodLength = doc->m_PeriodLength;
	m_OldAbsorption = m_Absorption;
	m_OldBlZOffs = doc->m_BlZOffs;

	m_OldContextProp.resPerc = doc->m_ContextProp.resPerc;
	m_OldContextProp.lineWidthPerc = doc->m_ContextProp.lineWidthPerc;
	m_OldContextProp.opacityPerc = doc->m_ContextProp.opacityPerc;
	
	DrawScene();
}

void CFlow3DView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if(m_LightType == DEF_POINTLIGHT_INDEX)
		m_Light = m_View;

	CView::OnLButtonDblClk(nFlags, point);
}

void CFlow3DView::OnPlaceSeedPoint() 
{
	// TODO: Add your command handler code here
	doc->m_IsPlaceSeedPoint = !doc->m_IsPlaceSeedPoint;
	
	DrawScene();
}

void CFlow3DView::OnSetAddInfo() 
{
	// TODO: Add your command handler code here
	MyAddInfoDialog addInfoDialog;

	for(int i = 0;i<DEF_CONTTRANSFVALUES_NUM;i++)
	{
		ContTransfProp prop;
		doc->GetContTransfProp(i,prop);
		addInfoDialog.m_ContTransfDialog.SetContProp(i, prop);
	}

	addInfoDialog.SetHistoData(&m_HistoDialog.m_HistoData[0]);

	addInfoDialog.m_MinScalars = doc->m_MinScalars;
	addInfoDialog.m_MaxScalars = doc->m_MaxScalars;

	addInfoDialog.m_ContTransfDialog.SetInterPos(doc->GetInterPos());
	addInfoDialog.SetAddInfoType(doc->GetAddInfoType());
	addInfoDialog.SetAddInfoRepr(doc->GetAddInfoRepr());
	
	addInfoDialog.m_RegionsDialog.
		SetDefaultProp(doc->m_RegProp[DEF_REGIONS_NUM]);
	
	for(i=0;i<DEF_REGIONS_NUM;i++)
		addInfoDialog.m_RegionsDialog.SetRegProp(i,doc->m_RegProp[i]);

	if(addInfoDialog.DoModal()==IDOK)
	{
		for(i=0;i<DEF_REGIONS_NUM;i++)
		{
			addInfoDialog.m_RegionsDialog.GetRegProp(i,doc->m_RegProp[i]);

			// Calculate resolution index.
			doc->UpdateResIndex(doc->m_RegProp[i]);
		}
		
		for(int i = 0;i<DEF_CONTTRANSFVALUES_NUM;i++)
		{
			ContTransfProp prop;
			addInfoDialog.m_ContTransfDialog.GetContProp(i, prop);
			doc->SetContTransfProp(i, prop);
		}

		doc->SetInterPos(addInfoDialog.m_ContTransfDialog.GetInterPos());
		doc->SetAddInfoType(addInfoDialog.GetAddInfoType());

		doc->SetAddInfoRepr(addInfoDialog.GetAddInfoRepr());
		
	
		//! If material regions are enabled => update texture maps.
		if(doc->GetAddInfoRepr() == DEF_REGIONS_INDEX)
			ReplaceTextures();
			
		m_Modified = true;
		
		doc->UpdatePiecesLists();
		DrawScene();
	}
}

void CFlow3DView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	m_IsKeyPressed[nChar] = true;

	DrawScene();

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CFlow3DView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	for(int i=0;i<DEF_KEYS_NUM;i++)
		m_IsKeyPressed[i] = false;

	DrawScene();

	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CFlow3DView::OnSetHighlight() 
{
	// TODO: Add your command handler code here
	MyHighlightDialog highlightDialog;
	highlightDialog.SetHighlight(m_Glossiness);
	
	if(highlightDialog.DoModal()==IDOK)
	{
		m_Glossiness = highlightDialog.GetHighlight();
		ReplaceTextures();
		DrawScene();
	}
}

void CFlow3DView::OnSetDiffuseCorrection() 
{
	// TODO: Add your command handler code here
	MyDiffuseCorrectionDialog diffuseCorrDialog;
	diffuseCorrDialog.SetDiffuseCorr(m_DiffuseCorr);

	if(diffuseCorrDialog.DoModal()==IDOK)
	{
		m_DiffuseCorr = diffuseCorrDialog.GetDiffuseCorr();

		ReplaceTextures();
		DrawScene();
	}
}

void CFlow3DView::OnShowHideAddInfo() 
{
	// TODO: Add your command handler code here
	doc->SetIsAddInfo(!doc->GetIsAddInfo());
	doc->UpdatePiecesLists();
	m_Modified = true;
	DrawScene();
}

void CFlow3DView::OnSetLightType() 
{
	// TODO: Add your command handler code here
	MyLightTypeDialog lightTypeDialog;

	lightTypeDialog.SetLightType(m_LightType);
	
	lightTypeDialog.DoModal();

	m_LightType = lightTypeDialog.GetLightType();
	DrawScene();
}

void CFlow3DView::OnUpdateEnableDisableDepthCueing(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_IsDepthCueing);
}

void CFlow3DView::OnUpdatePlaceSeedPoint(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(doc->m_IsPlaceSeedPoint);
}

void CFlow3DView::OnUpdateShowHideAddInfo(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(doc->GetIsAddInfo());
}

void CFlow3DView::OnUpdateShowHideBoundaries(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_IsDrawBoundaries);
}

void CFlow3DView::OnSetLineStyle() 
{
	// TODO: Add your command handler code here
	MyLineStyleDialog lineStyleDialog;

	lineStyleDialog.SetLineWidth(doc->m_RegProp[DEF_REGIONS_NUM].lineWidth);
	lineStyleDialog.SetIsAntiAliasing(m_IsAntiAliasing);
	lineStyleDialog.SetIsTapering(doc->GetIsTapering());
	lineStyleDialog.SetIsTranspFunc(m_IsTranspFunc);
	lineStyleDialog.SetPeriodLength(doc->m_PeriodLength);
	lineStyleDialog.SetTranspFuncType(m_TranspFuncType);
	lineStyleDialog.SetAbsorption(m_Absorption);
	lineStyleDialog.SetIsGap(m_IsGap);
	lineStyleDialog.SetGapSize(m_GapSize);
	lineStyleDialog.SetEndTapWidth(doc->GetEndTapWidth());
	lineStyleDialog.SetIsEndTap(doc->GetIsEndTap());
	lineStyleDialog.SetIsArrows(doc->m_IsArrows);
	lineStyleDialog.SetArrowLength(m_ArrowLength);
	lineStyleDialog.SetArrowWidth(m_ArrowWidth);
	lineStyleDialog.SetArrowType(m_ArType);
	lineStyleDialog.SetArrowPeriod(m_ArPeriod);

	if(lineStyleDialog.DoModal()==IDOK)
	{
		doc->m_RegProp[DEF_REGIONS_NUM].lineWidth = 
			lineStyleDialog.GetLineWidth();
		m_IsAntiAliasing = lineStyleDialog.GetIsAntiAliasing();
		doc->SetIsTapering(lineStyleDialog.GetIsTapering());
		m_IsTranspFunc = lineStyleDialog.GetIsTranspFunc();
		doc->m_PeriodLength = lineStyleDialog.GetPeriodLength();
		m_TranspFuncType = lineStyleDialog.GetTranspFuncType();
		m_Absorption = lineStyleDialog.GetAbsorption();
		m_IsGap = lineStyleDialog.GetIsGap();
		m_GapSize = lineStyleDialog.GetGapSize();
		doc->SetEndTapWidth(lineStyleDialog.GetEndTapWidth());
		doc->SetIsEndTap(lineStyleDialog.GetIsEndTap());
		doc->m_IsArrows = lineStyleDialog.GetIsArrows();
		m_ArrowLength = lineStyleDialog.GetArrowLength();
		m_ArrowWidth = lineStyleDialog.GetArrowWidth();
		m_ArType = lineStyleDialog.GetArrowType();
		m_ArPeriod = lineStyleDialog.GetArrowPeriod();

		m_Modified = true;
	
		// Update alpha table(needed if transp function has changed).
		if(doc->m_AlphaTable != NULL)
			delete [] doc->m_AlphaTable;

		doc->m_AlphaTable = CreateAlphaTable();

		// Update end tapering table(needed if end tapering widht has changed).
		delete doc->m_EndTapTable;
		doc->m_EndTapTable = doc->CreateEndTapTable(doc->GetEndTapWidth());
		
		// Update pieces list.
		doc->UpdatePiecesLists();
		DrawScene();
	}
}

GLubyte CFlow3DView::ModFunc(int i, int n)
{
	return (GLubyte)(i%n) * DEF_OPACITY_MAX / (n-1);
}

GLubyte CFlow3DView::SinFunc(int i, int n)
{
	float sin = m_SinTable.GetSin((i%n)*DEF_ANGLE_360 / n);

	return (GLubyte)( (DEF_OPACITY_MAX + (int)(DEF_OPACITY_MAX * sin) ) >> 1);
}	

void CFlow3DView::OnShowHideAxis() 
{
	// TODO: Add your command handler code here
	m_IsShowAxis = !m_IsShowAxis;
}

void CFlow3DView::OnUpdateShowHideAxis(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_IsShowAxis);
}

void CFlow3DView::OnStatistics() 
{
	// TODO: Add your command handler code here
	
	MyStatisticsDialog statisticsDialog;

	statisticsDialog.SetStreamlinesNum(doc->GetStreamlinesNum());
	statisticsDialog.SetSamplePointsNum(doc->GetSamplePointsNum());
	statisticsDialog.SetCalculationTime(doc->GetCalculationTime());
	statisticsDialog.SetDisplSamplePNum(doc->GetDisplSamplePNum());
	statisticsDialog.SetFPS(CalcFPS());

	statisticsDialog.m_MaxXPos = doc->m_MaxPos.m_XPos;
	statisticsDialog.m_MaxYPos = doc->m_MaxPos.m_YPos;
	statisticsDialog.m_MaxZPos = doc->m_MaxPos.m_ZPos;
	
	statisticsDialog.m_MinXPos = doc->m_MinPos.m_XPos;
	statisticsDialog.m_MinYPos = doc->m_MinPos.m_YPos;
	statisticsDialog.m_MinZPos = doc->m_MinPos.m_ZPos;

	statisticsDialog.m_ScalNum = CSamplePoint::GetAdditionalSize();

	if(doc->GetCurrentField() != NULL)
	{
		statisticsDialog.m_XDim = doc->GetCurrentField()->GetXDim();
		statisticsDialog.m_YDim = doc->GetCurrentField()->GetYDim();
		statisticsDialog.m_ZDim = doc->GetCurrentField()->GetZDim();
	}
	else
	{
		statisticsDialog.m_XDim = 0;
		statisticsDialog.m_YDim = 0;
		statisticsDialog.m_ZDim = 0;
	}
	
	statisticsDialog.DoModal();
}

void CFlow3DView::OnEnableDisableAnimation() 
{
	// TODO: Add your command handler code here
	doc->m_IsAnimation = !doc->m_IsAnimation;
	
	// If Animation => enable transparency function.
	if(doc->m_IsAnimation)
		SetTimer(DEF_ANIMATIONTIMER_INDEX,doc->m_AnimSpeed,NULL);
	else
		KillTimer(DEF_ANIMATIONTIMER_INDEX);

	doc->UpdatePiecesLists();
	m_Modified = true;
}

void CFlow3DView::OnUpdateEnableDisableAnimation(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(doc->m_IsAnimation);
}

void CFlow3DView::OnSetFocus(CWnd* pOldWnd) 
{
	CView::OnSetFocus(pOldWnd);
	
	if(m_IsRotating)
		SetTimer(DEF_ROTATIONTIMER_INDEX, DEF_ROTATIONTIMER_ELAPSE, NULL);
	if(doc->m_IsAnimation)
		SetTimer(DEF_ANIMATIONTIMER_INDEX, doc->m_AnimSpeed, NULL);

	// TODO: Add your message handler code here
	m_HasFocus = true;
	
	DrawScene();
}

/*!
   m_HasFocus prevents from drawing streamlines when window does not have focus, so
   application does not crash.
*/
void CFlow3DView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);
	
	DrawScene();

	if(m_IsRotating)
		KillTimer(DEF_ROTATIONTIMER_INDEX);
	if(doc->m_IsAnimation)
		KillTimer(DEF_ANIMATIONTIMER_INDEX);

	// TODO: Add your message handler code here
	m_HasFocus = false;
}

/*! 
	 The main direction is needed for choosing streamlinepieces-list most closely 
	 resembling current view direction. Rendering the pieces in this list
	 provides correct back-to-front rendering order.
*/
int CFlow3DView::GetMainDirection()
{
	int result;

	float qX = m_View.m_XComp * m_View.m_XComp;
	float qY = m_View.m_YComp * m_View.m_YComp;
	float qZ = m_View.m_ZComp * m_View.m_ZComp;

	// Always use x-list if correct order not required
	// else use list most closely resembling view direction
	if((!doc->GetIsCorrectOrder()) || ((qX > qY) && (qX > qZ)))
	{
		if(m_View.m_XComp < 0)
			result = DEF_XLIST_INDEX;
		else 
			result = DEF_XLIST_REVERSE_INDEX;
	}
	else if((qY > qX) && (qY > qZ))
	{
		if(m_View.m_YComp < 0)
			result = DEF_YLIST_INDEX;
		else
			result = DEF_YLIST_REVERSE_INDEX;
	}
	else
	{
		if(m_View.m_ZComp < 0)
			result = DEF_ZLIST_INDEX;
		else
			result = DEF_ZLIST_REVERSE_INDEX;
	}

	return result;
}

void CFlow3DView::OnSetPerformanceOptions() 
{
	// TODO: Add your command handler code here
	MyPerformanceOptDialog performanceOptDialog;

	bool isCorrectOrder = doc->GetIsCorrectOrder();
	bool useDisplayLists = doc->GetUseDisplayLists();
	float minDistPerc = doc->GetMinDistPerc();

	performanceOptDialog.SetIsCorrectOrder(doc->GetIsCorrectOrder());
	performanceOptDialog.SetUseDisplayLists(doc->GetUseDisplayLists());
	performanceOptDialog.SetMinDistPerc(doc->GetMinDistPerc());
	
	if(performanceOptDialog.DoModal()==IDOK)
	{
		doc->SetIsCorrectOrder(performanceOptDialog.GetIsCorrectOrder());
		doc->SetUseDisplayLists(performanceOptDialog.GetUseDisplayLists());
		doc->SetMinDistPerc(performanceOptDialog.GetMinDistPerc());

		// delete lists if not needed
		if(!doc->GetUseDisplayLists())
			glDeleteLists(m_TheStreamlines,DEF_DIRECTIONS_NUM);
		
		// if ordering set from false to true or
		// mindist has changed => reorder lists.
		if(((!isCorrectOrder) && doc->GetIsCorrectOrder()) ||
		   (HasChanged(minDistPerc,doc->GetMinDistPerc())))
		{
			doc->ResortPiecesLists();
			doc->CalcMinMaxScalars();
			doc->UpdatePiecesLists();
			m_Modified = true;
		}
		// if m_UseDisplayLists has changed => set modified to true
		if(useDisplayLists == (!doc->GetUseDisplayLists()))
			m_Modified = true;

		DrawScene();
	}
	
}

void CFlow3DView::SetRotMatrix(float *matrix)
{
	glMatrixMode(GL_MODELVIEW);
	
	glPushMatrix();
	glLoadMatrixf(oldRotMatrix);
	glRotatef(m_RotAngle,m_RotAxis.m_XComp,m_RotAxis.m_YComp,m_RotAxis.m_ZComp);
	glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
	glPopMatrix();
}

void CFlow3DView::SetTextureMatrix()
{
	texMatrix[1] = m_View.m_XComp * DEF_TEXMATRIX_FACTOR;
	texMatrix[5] = m_View.m_YComp * DEF_TEXMATRIX_FACTOR;
	texMatrix[9] = m_View.m_ZComp * DEF_TEXMATRIX_FACTOR;

	texMatrix[0] = m_Light.m_XComp * DEF_TEXMATRIX_FACTOR;
	texMatrix[4] = m_Light.m_YComp * DEF_TEXMATRIX_FACTOR;
	texMatrix[8] = m_Light.m_ZComp * DEF_TEXMATRIX_FACTOR;
}

/*! 
	Calculates frame count of one DrawScene()-call by counting clock ticks and dividing
	through the macro CLK_TCK(= clock ticks per second on this computer).
*/
int CFlow3DView::CalcFPS()
{
	int result = 0;
	clock_t start,end;

	// Take time of one loop.
	start = clock();
	
	if((doc->GetIsFocusContext())||(doc->m_IsAnimation))
		doc->DoInsideTest();

	DrawScene();

	end = clock();
	
	clock_t total = end - start;

	if(total != 0)
		result = (int)(CLK_TCK/total);
	
	return result;
}

GLubyte *CFlow3DView::CreateAlphaTable()
{
	GLubyte alpha = DEF_OPACITY_MAX - m_Absorption;
	GLubyte q = DEF_OPACITY_MAX;
	GLubyte *result = NULL;
	int i=0;
	int length = doc->m_PeriodLength;
	int size = 0;
	switch(m_TranspFuncType)
	{
		case DEF_HAZE_LIN_INDEX:
			doc->m_AlphaTblWidth = length;
			result = new GLubyte[length];
			for(i=0;i<length;i++)
				result[i] = ModFunc(length-1-i,length);	
			break;
		case DEF_SIN_INDEX:
			doc->m_AlphaTblWidth = length;
			result = new GLubyte[length];
			for(i=0;i<length;i++)
				result[i] = SinFunc(length-1-i,length);
			break;
		case DEF_HAZE_EXP_INDEX:
			while((q > DEF_OPACITY_MIN) && (size < DEF_ALPHATBL_MAXSIZE))
			{
				q = q * alpha / DEF_OPACITY_MAX;
				size ++;
			}
			
			result = new GLubyte[size];
			doc->m_AlphaTblWidth = size;

			for(i=0;i<size;i++)
			{
				result[i] = q;

				if(q <= DEF_OPACITY_MIN)
					q = DEF_OPACITY_MAX;
			
				q = q * alpha / DEF_OPACITY_MAX;
			}
			break;
		default:
			doc->m_AlphaTblWidth = length;
			for(i=0;i<length;i++)
				result[i] = DEF_OPACITY_MAX;
			break;
	}
	
	return result;
}

bool CFlow3DView::HasChanged(float oldVal, float val)
{
	bool result = false;
	if(((oldVal - val) < (-DEF_FLT_CHANGE_MIN)) ||
	   ((oldVal - val) > DEF_FLT_CHANGE_MIN))
		result = true;

	return result;
}

bool CFlow3DView::HasChanged(GLubyte *oldRgba, GLubyte *rgba)
{
	bool result = false;
	if((oldRgba[0] != rgba[0]) ||
	   (oldRgba[1] != rgba[1]) ||
	   (oldRgba[2] != rgba[2]) ||
	   (oldRgba[3] != rgba[3]))
			result = true;
	
	return result;
}

void CFlow3DView::OnShowHideMagicVolume() 
{
	// TODO: Add your command handler code here
	// Switch flag.
	doc->SetIsMagicVol(!doc->GetIsMagicVol());
	DrawScene();
}

void CFlow3DView::OnUpdateShowHideMagicVolume(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(doc->GetIsMagicVol());
}

void CFlow3DView::OnSetFocusContextOptions() 
{
	// TODO: Add your command handler code here
	MyFocusContextDialog focusContextDialog;
	
	int magicVolType = doc->m_MagicVolType;

	focusContextDialog.SetContextProp(doc->m_ContextProp);
	focusContextDialog.SetIsCutLines(doc->GetIsCutLines());
	focusContextDialog.SetMagicVolType(doc->m_MagicVolType);
	focusContextDialog.m_IsIncremential = doc->GetIsIncremential();
	focusContextDialog.SetRecalcType(doc->m_RecalcType);

	if(focusContextDialog.DoModal()==IDOK)
	{
		focusContextDialog.GetContextProp(doc->m_ContextProp);
		doc->UpdateContextProp();

		doc->SetIsCutLines(focusContextDialog.GetIsCutLines());
		doc->m_MagicVolType = focusContextDialog.GetMagicVolType();
		doc->m_RecalcType = focusContextDialog.GetRecalcType();

		// Magic volume type has changed => Create new magic volume.
		if(magicVolType != doc->m_MagicVolType)
		{
			CPoint3D pos = doc->m_MagicVolume->GetPosition();

			delete doc->m_MagicVolume;
			delete m_OldMagicVolume;

			doc->m_MagicVolume = 
				MyFocusContextDialog::CreateMagicVol(doc->m_MagicVolType, pos, 
													 DEF_MAGICCUBE_SIZE, 
													 DEF_MAGICSPHERE_RADIUS,
													 DEF_MAGICRECT_LENGTH,
													 DEF_MAGICRECT_HEIGHT,
													 DEF_MAGICRECT_WIDTH);
			m_OldMagicVolume = 
				MyFocusContextDialog::CreateMagicVol(doc->m_MagicVolType, pos, 
													 DEF_MAGICCUBE_SIZE, 
													 DEF_MAGICSPHERE_RADIUS,
													 DEF_MAGICRECT_LENGTH,
													 DEF_MAGICRECT_HEIGHT,
													 DEF_MAGICRECT_WIDTH);
		}
	
		m_Modified = true;
		
		doc->UpdatePiecesLists();
		ReplaceTextures();

		DrawScene();
	}
}

void CFlow3DView::OnStartEndFocusContextMode() 
{
	// TODO: Add your command handler code here
	doc->SetIsFocusContext(!doc->GetIsFocusContext());

	// Reset field boundaries.
	if(doc->GetIsFocusContext() == false)
	{
		doc->GetCurrentField()->SetWinMinMaxPos(m_OldMinPos,m_OldMaxPos);
		doc->GetCurrentField()->SetSeedMinMaxPos(m_OldMinPos,m_OldMaxPos);

		doc->UpdateMinMaxPos();
		doc->CalcMinMaxScalars();
		InitObjects();
	}

	doc->UpdatePiecesLists();
	m_Modified = true;
	DrawScene();
}

void CFlow3DView::OnUpdateStartEndFocusContextMode(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(doc->GetIsFocusContext());	
}

bool CFlow3DView::IsKeyPressed()
{
	bool result = false;
	for(int i=0;i<DEF_KEYS_NUM;i++)
		if(m_IsKeyPressed[i])
			result = true;

	return result;
}

void CFlow3DView::DrawAxisLegend(GLint xPos, GLint yPos, GLubyte *rgba)
{
	DrawText(xPos,yPos,"X-Axis:\nY-Axis:\nZ-Axis: ", rgba);

	DrawLines(xPos + DEF_CHAR_WIDTH * strlen("X-Axis"),yPos + DEF_CHAR_HEIGHT/2,
			  DEF_LINES_LENGTH,DEF_CHAR_HEIGHT, rgba[3]);
}

void CFlow3DView::DrawText(GLint xPos, GLint yPos, char* s, GLubyte *rgba)
{
    int lines;
    char* p;

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, m_oldRect.right, 0.0, m_oldRect.bottom, -1.0f, DEF_FARPLANE);
    glMatrixMode(GL_MODELVIEW);
	
    glPushMatrix();
    glLoadIdentity();
    glColor4ubv(rgba);
    glRasterPos2i(xPos, yPos);
    
	for(p = s, lines = 0; *p; p++) 
	{
		if (*p == '\n')
		{
			lines++;
			glRasterPos2i(xPos, yPos-(lines * DEF_CHAR_HEIGHT));
		}
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *p);
    }

    glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
}

void CFlow3DView::DrawLines(GLint xPos, GLint yPos, GLint length, GLint width, GLubyte opacity)
{
	glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
   	glOrtho(0.0, m_oldRect.right, 0.0, m_oldRect.bottom, -1.0f, DEF_FARPLANE);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

	GLubyte *rgba[3];
	
	rgba[0] = xAxisRgba;
	rgba[1] = yAxisRgba;
	rgba[2] = zAxisRgba;

	glLineWidth(DEF_AXIS_LINEWIDTH);
	for(int i=0;i<3;i++)
	{
		glColor4ub(rgba[i][0],rgba[i][1],rgba[i][2],opacity);
		glBegin(GL_LINES);
		glVertex2i(xPos, yPos - i*width);
		glVertex2i(xPos + length, yPos - i*width);
		glEnd();
	}

	glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
}

void CFlow3DView::OnRButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	// is mouse clicked once rather that dragged?
	if(!m_IsRotating)
	{
		m_RButDblClkPos = point;
		m_CurrentMousePos = m_RButDblClkPos;
				
		SetTimer(DEF_ROTATIONTIMER_INDEX, DEF_ROTATIONTIMER_ELAPSE, NULL);

		m_IsRotating = true;
	}
	else
	{
		// Set rotation matrix
		SetRotMatrix(oldRotMatrix);

		// Initialise angle + axis
		m_RotAxis = DEF_AXIS_X;
		m_RotAngle = DEF_ANGLE_0F;

		KillTimer(DEF_ROTATIONTIMER_INDEX);
		m_IsRotating = false;
	}

	DrawScene();

	CView::OnRButtonDblClk(nFlags, point);
}

void CFlow3DView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	m_RButtonDownPos = point;
	m_IsRButtonDown = true;

	m_OldMagicVolume->Init(doc->m_MagicVolume);
	
	CView::OnRButtonDown(nFlags, point);

	DrawScene();
}

void CFlow3DView::RotateField()
{
	// Normal vector length = angle.
	m_RotAngle = m_RotAxis.Magnitude()*DEF_MOUSEMV_ANGLEINCR_FACTOR;

	// Clamp rotangle into [0,360].
	Clamp(m_RotAngle,DEF_ANGLE_0F,DEF_ANGLE_360F,m_RotAngle+DEF_ANGLE_360F,
		  m_RotAngle-DEF_ANGLE_360F);

	// Also rotate rotation axis.
	m_RotAxis.Mult(oldRotMatrix);

	// Rotation axis  must have mag 1.
	m_RotAxis.Normalize();
	
	// Update rotation matrix.
	SetRotMatrix(rotMatrix);
}

CPoint3D CFlow3DView::CalcNewPos(CPoint3D oldPos, CVector3D vec)
{
	CPoint3D result = oldPos;
	vec.Mult(rotMatrix);

	float factor = DEF_MOUSEMOV_POSINCR_FACTOR * CalcSizePerc() / DEF_PERC_100;	

	if(m_IsKeyPressed['X'])
		result.m_XPos += (vec.m_XComp * factor);
	if(m_IsKeyPressed['Y'])
		result.m_YPos += (vec.m_YComp * factor);
	if(m_IsKeyPressed['Z'])
		result.m_ZPos += (vec.m_ZComp * factor);
	
		// Clamps position so seedpoint position stays inside vectorfield.
	Clamp(result.m_XPos,-DEF_XRANGEDIV2,DEF_XRANGEDIV2,-DEF_XRANGEDIV2,DEF_XRANGEDIV2);
	Clamp(result.m_YPos,-DEF_YRANGEDIV2,DEF_YRANGEDIV2,-DEF_YRANGEDIV2,DEF_YRANGEDIV2);
	Clamp(result.m_ZPos,-DEF_ZRANGEDIV2,DEF_ZRANGEDIV2,-DEF_ZRANGEDIV2,DEF_ZRANGEDIV2);
	
	return result;
}

void CFlow3DView::CalcNewSize(float &size, float &radius, float &length, 
							  float &width, float &height, CVector3D vec)
{
	float factor = DEF_MOUSEMOV_POSINCR_FACTOR * CalcSizePerc() / DEF_PERC_100;	
	
	CVector3D oldVec = vec;

	vec.Mult(rotMatrix);

	if(m_IsKeyPressed['X'])
		length += (vec.m_XComp * factor);
	else if(m_IsKeyPressed['Y'])
		width += (vec.m_YComp * factor);
	else if(m_IsKeyPressed['Z'])
		height += (vec.m_ZComp * factor);
	else
	{
		size += ((oldVec.m_XComp + oldVec.m_YComp) * factor);
		radius += ((oldVec.m_XComp + oldVec.m_YComp) * factor * 0.5f);	
		length += ((oldVec.m_XComp + oldVec.m_YComp) * factor);
		width += ((oldVec.m_XComp + oldVec.m_YComp) * factor);
		height += ((oldVec.m_XComp + oldVec.m_YComp) * factor);
	}

	if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
		// Clamp size of cube, it should not be bigger than boundaries of field.
		Clamp(size,0,DEF_MINRANGE_DIM_XYZ,0,DEF_MINRANGE_DIM_XYZ);
	else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
		// Clamp radius of sphere.
		Clamp(radius,0,DEF_MINRANGE_DIM_XYZ/2,0,DEF_MINRANGE_DIM_XYZ/2);
	else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
	{
		// Clamp size of rectangle.
		Clamp(length,0,DEF_XRANGE,0,DEF_XRANGE);
		Clamp(width,0,DEF_YRANGE,0,DEF_YRANGE);
		Clamp(height,0,DEF_ZRANGE,0,DEF_ZRANGE);
	}
}

void CFlow3DView::DrawLegend()
{
	GLubyte rgba[4] = DEF_TEXT_RGBA;

	char sizeStr[100];
	char viewStr[100];
	
	if(doc->m_IsAnimation)
		DrawAnimSpeedBar(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS + DEF_CHAR_HEIGHT*10,100,rgba);

	sprintf(sizeStr,"Size: %3.2f%% of orig.", CalcSizePerc());
	DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS + DEF_CHAR_HEIGHT*6,
			 sizeStr,rgba);

	sprintf(viewStr, "Viewvec. x: %1.2f\nViewvec. y: %1.2f\nViewvec. z: %1.2f",
			m_View.m_XComp,m_View.m_YComp,m_View.m_ZComp);

	DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS + DEF_CHAR_HEIGHT*4,
			 viewStr,rgba);
	
	if(m_IsRotating)
	{
		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS, 
				 "Rotating:\nEnd with dbl r-button click",rgba);
	}

	if(m_LightType == DEF_HEADLIGHT_INDEX)
	{
		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 4, 
				 "Light type: head light",rgba);
	}
	else if(m_LightType == DEF_POINTLIGHT_INDEX)
	{
		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 4, 
				 "Light type = point light\nSet light with dbl l-button click",rgba);
	}

	if(doc->m_IsPlaceSeedPoint)
	{
		char posStr[80];
		sprintf(posStr,"Seedpoint x-pos: %3.2f\nSeedpoint y-pos: %3.2f\nSeedpoint z-pos: %3.2f",
				    doc->GetSeedPCross()->GetPosition().GetXPos()+DEF_XOFFS,
					doc->GetSeedPCross()->GetPosition().GetYPos()+DEF_YOFFS,
					doc->GetSeedPCross()->GetPosition().GetZPos()+DEF_ZOFFS);

		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 8, 
				 "Seedpoint placing mode",rgba);
		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 10, posStr, rgba);
	}
	else if(doc->GetIsFocusContext())
	{
		DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 8, "Focus context mode",rgba);
		if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
		{
			DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 10,
					 "Magic volume = cube",rgba);
		}
		else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
		{
			DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 10,
					 "Magic volume = rect. prism",rgba);
		}
		else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
		{
			DrawText(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 10,
					 "Magic volume = sphere",rgba);
		}
	}

	if(m_IsShowAxis || doc->m_IsPlaceSeedPoint || doc->GetIsFocusContext())
	{
		DrawAxisLegend(DEF_LEGEND_XPOS,DEF_LEGEND_YPOS - DEF_CHAR_HEIGHT * 14, rgba);
	}
	
}

void CFlow3DView::OnShowHideLegend() 
{
	// TODO: Add your command handler code here
	m_IsShowLegend = !m_IsShowLegend;
	DrawScene();
}

void CFlow3DView::OnUpdateShowHideLegend(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_IsShowLegend);	
}

void CFlow3DView::OnSetBackground() 
{
	// TODO: Add your command handler code here
	CColorDialog colorDialog;
	colorDialog.m_cc.rgbResult = 
		m_BKColor[0]*DEF_COLOR_MAX + m_BKColor[1]*DEF_COLOR_MAX*256 + 
		m_BKColor[2]*DEF_COLOR_MAX*256*256;

	if(colorDialog.DoModal()==IDOK)
	{
		colorDialog.GetColor();
		COLORREF c = colorDialog.GetColor();
		m_BKColor[0] = ((float)(c & 255))/DEF_COLOR_MAX;
		m_BKColor[1] = ((float)((c>>8) & 255))/DEF_COLOR_MAX;
		m_BKColor[2] = ((float)((c>>16) & 255))/DEF_COLOR_MAX;
	};
	DrawScene();

}

void CFlow3DView::OnZoomIn() 
{
	// TODO: Add your command handler code here
	if(doc->GetIsFocusContext())
		CenterFocus();	
	
	DrawScene();
}

void CFlow3DView::OnResetSize() 
{
	// TODO: Add your command handler code here
	doc->GetCurrentField()->SetWinMinMaxPos(m_OldMinPos,m_OldMaxPos);

	doc->UpdateMinMaxPos();

	doc->CalcMinMaxScalars();
	InitObjects();

	doc->UpdatePiecesLists();
	DrawScene();
}

void CFlow3DView::CenterFocus()
{
	
	CPoint3D minPoint0;
	CPoint3D minPoint1;

	CPoint3D maxPoint0;
	CPoint3D maxPoint1;
			
	if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
	{
		MyCube *cube = (MyCube *)doc->m_MagicVolume;

		minPoint0 = cube->GetMinPoint();
		maxPoint0 = cube->GetMaxPoint();
	}
	else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
	{
		MyRectangle *rect = (MyRectangle *)doc->m_MagicVolume;

		minPoint0 = rect->GetMinPoint();
		maxPoint0 = rect->GetMaxPoint();
	}
	else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
	{
		MySphere *sphere = (MySphere *)doc->m_MagicVolume;

		CPoint3D pos = sphere->GetPosition();

		minPoint0 = CPoint3D(pos.m_XPos - sphere->GetRadius(),
							 pos.m_YPos - sphere->GetRadius(),
							 pos.m_ZPos - sphere->GetRadius());

		maxPoint0 = CPoint3D(pos.m_XPos + sphere->GetRadius(),
							 pos.m_YPos + sphere->GetRadius(),
							 pos.m_ZPos + sphere->GetRadius());
	}

	minPoint1 = doc->m_Boundaries.GetMinPoint();
	maxPoint1 = doc->m_Boundaries.GetMaxPoint();

	float minXPos = max(minPoint0.m_XPos,minPoint1.m_XPos) + DEF_XOFFS;
	float minYPos = max(minPoint0.m_YPos,minPoint1.m_YPos) + DEF_YOFFS;
	float minZPos = max(minPoint0.m_ZPos,minPoint1.m_ZPos) + DEF_ZOFFS;

	float maxXPos = min(maxPoint0.m_XPos,maxPoint1.m_XPos) + DEF_XOFFS;
	float maxYPos = min(maxPoint0.m_YPos,maxPoint1.m_YPos) + DEF_YOFFS;
	float maxZPos = min(maxPoint0.m_ZPos,maxPoint1.m_ZPos) + DEF_ZOFFS;
			
	m_OldMinPos = doc->m_MinPos;
	m_OldMaxPos = doc->m_MaxPos;

	doc->GetCurrentField()->SetWinMinMaxPos(CPoint3D(minXPos,minYPos,minZPos),
											CPoint3D(maxXPos,maxYPos,maxZPos));

	doc->UpdateMinMaxPos();

	doc->CalcMinMaxScalars();
	InitObjects();

	doc->UpdatePiecesLists();
	DrawScene();
}

void CFlow3DView::InitObjects()
{
	// Initialise graphical objects.
	doc->m_Boundaries.SetPosition(CPoint3D());
	doc->m_Boundaries.SetLength(DEF_XRANGE);
	doc->m_Boundaries.SetWidth(DEF_YRANGE);
	doc->m_Boundaries.SetHeight(DEF_ZRANGE);

	m_OldSeedPCross.SetPosition(doc->GetSeedPCross()->GetPosition());
	//m_OldSeedPCross.SetPosition(CPoint3D());
	m_OldSeedPCross.SetRange(&doc->m_Boundaries);
	doc->GetSeedPCross()->SetRange(&doc->m_Boundaries);

	m_Axis.SetPosition(CPoint3D());
	m_Axis.SetRange(&doc->m_Boundaries);

	m_FoContextCross.SetPosition(CPoint3D());
	m_FoContextCross.SetRange(&doc->m_Boundaries);
		
	doc->m_IsPlaceSeedPoint = DEF_ISPLACESEEDPOINT;
	m_Modified = true;

	// Initialising magic volume parameters.
	doc->m_MagicVolume->SetPosition(CPoint3D());
	
	if(doc->m_MagicVolType == DEF_MCUBE_INDEX)
	{
		((MyCube *)doc->m_MagicVolume)->SetSize(DEF_MAGICCUBE_SIZE);
	}
	else if(doc->m_MagicVolType == DEF_MSPHERE_INDEX)
	{
		((MySphere *)doc->m_MagicVolume)->SetRadius(DEF_MAGICSPHERE_RADIUS);
	}
	else if(doc->m_MagicVolType == DEF_MRECT_INDEX)
	{
		((MyRectangle *)doc->m_MagicVolume)->SetHeight(DEF_MAGICRECT_HEIGHT);
		((MyRectangle *)doc->m_MagicVolume)->SetWidth(DEF_MAGICRECT_WIDTH);
		((MyRectangle *)doc->m_MagicVolume)->SetLength(DEF_MAGICRECT_LENGTH);
	}

	m_ZDist = m_OldZDist = 
		max(DEF_XRANGE,max(DEF_YRANGE,DEF_ZRANGE ))* DEF_Z_OFFS_FACTOR;
}

float CFlow3DView::CalcSizePerc()
{
	float result = 0.0f;

	if((doc->GetCurrentField() != NULL) && (doc->GetCurrentField()->GetXRange() != 0))
		result = DEF_XRANGE * 100 / doc->GetCurrentField()->GetXRange();

	return result;
}

void CFlow3DView::OnShowHistoDifOrient() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_DIF_ORIENT_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoOrientX() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_ORIENT_X_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoOrientY() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_ORIENT_Y_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoOrientZ() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_ORIENT_Z_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoPosX() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_POS_X_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoPosY() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_POS_Y_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoPosZ() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_POS_Z_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoVelocity() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_VELOCITY_INDEX);
		
	m_HistoDialog.DoModal();
}

void CFlow3DView::OnShowHistoDifVelocity() 
{
	// TODO: Add your command handler code here
	InitHistoDialog(DEF_DIF_VLEN_INDEX);

	m_HistoDialog.DoModal();
}

void CFlow3DView::InitHistoDialog(int scalarType)
{
	if(doc->GetCurrentField() != NULL)
	{
		m_HistoDialog.SetScalarType(scalarType);
				
		float minScalar=0;
		float maxScalar=0;
		
		doc->GetMinMaxScalar(scalarType,minScalar,maxScalar);
		m_HistoDialog.SetMinMaxScalar(minScalar,maxScalar);

		m_HistoDialog.UpdateData();
	}
}

CObList * CFlow3DView::GetPiecesList(bool &isReverse, int direction)
{
	isReverse = false;
	CObList *list = NULL;
	switch(direction)
	{
		case DEF_XLIST_INDEX:
			list = doc->GetXList(); 
			break;
		case DEF_YLIST_INDEX:
			list = doc->GetYList(); 
			break;
		case DEF_ZLIST_INDEX:
			list = doc->GetZList(); 
			break;
		case DEF_XLIST_REVERSE_INDEX:
			list = doc->GetXList(); 
			isReverse = true;
			break;
		case DEF_YLIST_REVERSE_INDEX:
			list = doc->GetYList(); 
			isReverse = true;
			break;
		case DEF_ZLIST_REVERSE_INDEX:
			list = doc->GetZList(); 
			isReverse = true;
			break;
		default:
			list = doc->GetXList(); 
			break;
	}
	return list;
}

void CFlow3DView::OnSetDepthCoding() 
{
	// TODO: Add your command handler code here
	MyDepthCodingDialog depthDialog;
	
	depthDialog.SetDepthType(m_DepthType);
	depthDialog.SetColor(m_DepthColor);
	depthDialog.SetIntensity(m_DeIntensity);

	if(depthDialog.DoModal() == IDOK)
	{
		m_DepthType = depthDialog.GetDepthType();
		depthDialog.GetColor(m_DepthColor);
		m_DeIntensity = depthDialog.GetIntensity();
		DrawScene();
	}
}

void CFlow3DView::OnSetAnimationOptions() 
{
	// TODO: Add your command handler code here
	MyAnimationDialog animDialog;

	animDialog.SetAnimSpeed(doc->m_AnimSpeed);
	animDialog.SetAnimType(doc->m_AnimType);

	if(animDialog.DoModal() == IDOK)
	{
		doc->m_AnimSpeed = animDialog.GetAnimSpeed();
		doc->m_AnimType = animDialog.GetAnimType();
		doc->UpdatePiecesLists();
		m_Modified = true;
		DrawScene();
	}
	
}

void CFlow3DView::DrawAnimSpeedBar(int xPos, int yPos, int width, GLubyte *rgba)
{
	glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
   	glOrtho(0.0, m_oldRect.right, 0.0, m_oldRect.bottom, -1.0f, DEF_FARPLANE);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

	int newXPos = xPos + 5 + (width - 10) * 
				  (DEF_SPEEDSLIDER_MAX-doc->m_AnimSpeed)/
				  (DEF_SPEEDSLIDER_MAX  - DEF_SPEEDSLIDER_MIN);
	
	DrawText(xPos,yPos + DEF_CHAR_HEIGHT,"Animation speed:",rgba);
	DrawText(xPos,yPos - DEF_CHAR_HEIGHT*2,"Slow",rgba);
	DrawText(xPos+width-DEF_CHAR_WIDTH * 2,
			 yPos - DEF_CHAR_HEIGHT*2,"Fast",rgba);

	glLineWidth(6);
	glColor4fv(m_BKColor);
	glBegin(GL_LINES);
		glVertex2i(newXPos,yPos);
		glVertex2i(xPos + width - 5,yPos);
	glEnd();

	glColor4ubv(rgba);
	glLineWidth(10);
	glBegin(GL_LINES);
		glVertex2i(xPos,yPos);
		glVertex2i(xPos + width,yPos);
	glEnd();

	glPopMatrix();
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
}

bool CFlow3DView::IsLButtonRot()
{
	bool result = true;

	if(m_IsKeyPressed['D'] || m_IsKeyPressed['R'] || m_IsKeyPressed['O'] || 
	   m_IsKeyPressed['L'] || m_IsKeyPressed['S'] || m_IsKeyPressed['I'] ||
	   m_IsKeyPressed['P'] || m_IsKeyPressed['A'] || m_IsKeyPressed['B']) 
	   result = false;
	else if((doc->m_IsPlaceSeedPoint || doc->GetIsFocusContext()) && IsKeyPressed())
		result = false;

	return result;
}

void CFlow3DView::SetDepthCueing()
{
	GLfloat fogColor[3];

	if(m_DepthType == DEF_DEPTHCUEING_INDEX)
	{
		for(int i=0;i<3;i++)
			fogColor[i] = m_BKColor[i];
	}
	else
	{
		for(int i=0;i<3;i++)
			fogColor[i] = (float)m_DepthColor[i] / (float)DEF_COLOR_MAX;
	}
	float intensityf = (float)(DEF_INTENSITY_MAX - m_DeIntensity)*DEF_DEINTENSITY_FACTOR;

	glEnable(GL_FOG);
	glFogfv(GL_FOG_COLOR,fogColor);
	glFogf(GL_FOG_MODE,GL_LINEAR);
	glFogf(GL_FOG_START,max(0,m_ZDist - DEF_ZRANGEDIV2)); 
		
	glFogf(GL_FOG_END,max(0,m_ZDist)+(DEF_ZRANGEDIV2*(1.0f + intensityf)));
}

void CFlow3DView::DrawVField()
{
	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	glPushMatrix();

	CVectorField *field = doc->GetCurrentField();
	if(field != NULL)
	{
		glTranslatef(-DEF_XRANGEDIV2,-DEF_YRANGEDIV2,-DEF_ZRANGEDIV2);

		int xDim = field->GetXDim();
		int yDim = field->GetYDim();
		int zDim = field->GetZDim();

		for(int x = 0; x < xDim; x++)
			for(int y = 0; y < yDim; y++)
				for(int z = 0; z < zDim; z++)
				{
					CSamplePoint *sp = field->GetVField(x,y,z);
					if(sp->GetMagnitude() == 0)
					{
						glPushMatrix();
						glTranslatef(sp->GetXPos(),sp->GetYPos(),sp->GetZPos());
						
						glutSolidCube(1.0f);
						glPopMatrix();
					}
				}
	}
	glPopMatrix();
}

void CFlow3DView::DrawCone(CSamplePoint *sp, CStreamlineMaterial &mat, 
						   GLubyte *rgba, float base, float height)
{
	glPushMatrix();
	glDisable(GL_TEXTURE_2D);

	glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

	// Set material and opacity.
	float opacity = (float)rgba[3] * mat.m_OpacityF / FLOATFACTOR;

	GLfloat mat_specular[] = {mat.m_SpecRGBF[0], mat.m_SpecRGBF[1], mat.m_SpecRGBF[2], 
							  opacity};

	GLfloat mat_ambient[] = {mat.m_AmbRGBF[0], mat.m_AmbRGBF[1], mat.m_AmbRGBF[2], 
							 opacity};

	GLfloat mat_diffuse[] = {mat.m_DifRGBF[0], mat.m_DifRGBF[1],mat.m_DifRGBF[2], 
							 opacity};

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
  
	// Translate cone to right position.
	glTranslatef(sp->GetXPos(),sp->GetYPos(),sp->GetZPos());
							
	// Get angle of streamline vector in cone position.
	float arc =  DEF_ANGLE_270F + acos(sp->GetUnitXVec())*ROTFACTOR;
	float arc2 = DEF_ANGLE_270F - acos(sp->GetUnitYVec())*ROTFACTOR;

	if(sp->GetUnitZVec() >= 0.0f)
		arc = DEF_ANGLE_180F - arc;
			
	// Rotate cone to same angles as vector.
	glRotatef(arc,0,1,0);
	glRotatef(arc2,1,0,0);

	// Draw cone.
	glutSolidCone(base,height,DEF_CONE_SLICES_NUM,DEF_CONE_STACKS_NUM);

	// Draw cone base.
	glRotatef(DEF_ANGLE_180F,0,1,0);
	glutSolidCone(base,0.0,DEF_CONE_SLICES_NUM,DEF_CONE_STACKS_NUM);

	glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
	glEnable(GL_TEXTURE_2D);

	glPopMatrix();
}

void CFlow3DView::GLInit()
{
	PIXELFORMATDESCRIPTOR pfd;
	int         n;
	HGLRC       hrc;
	GLfloat     fAspect;

	m_pDC = new CClientDC(this);

	ASSERT(m_pDC != NULL);

	if (!bSetupPixelFormat())
		return;

	n = ::GetPixelFormat(m_pDC->GetSafeHdc());
	::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

	hrc = wglCreateContext(m_pDC->GetSafeHdc());
	wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

	GetClientRect(&m_oldRect);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);

	if (m_oldRect.bottom)
		fAspect = (GLfloat)m_oldRect.right/m_oldRect.bottom;
	else    // Don't divide by zero, not that we should ever run into that...
		fAspect = 1.0f;

	float farPlane = DEF_NEARPLANE + DEF_MAXRANGE + DEF_FARPLANE;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(DEF_VIEWANGLE, fAspect, DEF_NEARPLANE, farPlane);
	glMatrixMode(GL_MODELVIEW);
}

void CFlow3DView::OnSetViewDirection() 
{
	// TODO: Add your command handler code here
	MyViewVecDialog viewDialog;
	
	viewDialog.m_XRot = m_XAngle;
	viewDialog.m_YRot = m_YAngle;
	viewDialog.m_ZRot = m_ZAngle;

	if(viewDialog.DoModal() == IDOK)
	{
		// Set old rotation matrix to identity.
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glGetFloatv(GL_MODELVIEW_MATRIX,oldRotMatrix);
		
			m_XAngle = viewDialog.m_XRot;
			m_YAngle = viewDialog.m_YRot;
			m_ZAngle = viewDialog.m_ZRot;

			// Update rotation matrix and old rotation matrix with current angles 
			glRotatef(viewDialog.m_XRot,1.0f,0.0f,0.0f);
			glRotatef(viewDialog.m_YRot,0.0f,1.0f,0.0f);
			glRotatef(viewDialog.m_ZRot,0.0f,0.0f,1.0f);
		
			glGetFloatv(GL_MODELVIEW_MATRIX,oldRotMatrix);
			glGetFloatv(GL_MODELVIEW_MATRIX,rotMatrix);

		glPopMatrix();
	}

	DrawScene();
}

void CFlow3DView::DrawBlock()
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);

	// Translate block so it fits into field(Which is also translated).
	float xOffs = DEF_XRANGEDIV2 + doc->GetMinXPos();
	float yOffs = DEF_YRANGEDIV2 + doc->GetMinYPos();
	float zOffs = DEF_ZRANGEDIV2 + doc->GetMinZPos();

	glTranslatef(-xOffs,-yOffs,-zOffs);

	CPoint3D blMin = doc->m_BlMinPos;
	CPoint3D blMax = doc->m_BlMaxPos;
  
	glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

	CStreamlineMaterial mat = doc->m_BlMaterial;

	// Set material and opacity.
	GLfloat mat_specular[] = {mat.m_SpecRGBF[0], mat.m_SpecRGBF[1], mat.m_SpecRGBF[2], 
							  mat.m_OpacityF};

	GLfloat mat_ambient[] = {mat.m_AmbRGBF[0], mat.m_AmbRGBF[1], mat.m_AmbRGBF[2], 
							 mat.m_OpacityF};

	GLfloat mat_diffuse[] = {mat.m_DifRGBF[0], mat.m_DifRGBF[1],mat.m_DifRGBF[2], 
							 mat.m_OpacityF};

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);

/*	blMin.m_XPos = 5.503;
	blMin.m_YPos = 5.720;
	blMin.m_ZPos = 0.834f;

	blMax.m_XPos = 6.497;
	blMax.m_YPos = 6.715;
	blMax.m_ZPos = 1.397;*/

	// Add z-offs to geometry.
	blMin.m_ZPos -= doc->m_BlZOffs;
	blMax.m_ZPos += doc->m_BlZOffs;

	// Texture coordinate points.
	CPoint3D texCo0;
	CPoint3D texCo1;

	texCo0.m_XPos = 0.0f;
	texCo0.m_YPos = 0.0f;
	texCo0.m_ZPos = 0.0f;

	texCo1.m_XPos = 1.0f;
	texCo1.m_YPos = 1.0f;

	// Save aspect ratio to keep texture proportions.
	float ratio = blMax.m_XPos - blMin.m_XPos;
	
	// Avoid division by zero.
	if(ratio == 0.0f) 
		ratio = 1.0f;

	// Now calculate z-position of texture coordinate by scaling.
	// with ratio.
	texCo1.m_ZPos = (blMax.m_ZPos - blMin.m_ZPos) / ratio;

	// If texture is enabled, assign texture to geometry.
	if(m_IsTex)
	{
		glEnable(GL_TEXTURE_2D);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glBindTexture(GL_TEXTURE_2D, blTexName);

		glMatrixMode(GL_TEXTURE);
		glPushMatrix();
		glLoadIdentity();
	}

    glBegin(GL_QUADS);
 
		glNormal3d(0.0, -1.0, 0.0);
	    glTexCoord2f(texCo0.m_XPos, texCo0.m_ZPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo1.m_XPos, texCo0.m_ZPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo1.m_XPos, texCo1.m_ZPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo1.m_ZPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMin.m_ZPos);

		glNormal3d(0.0, 0.0, -1.0);
	    glTexCoord2f(texCo1.m_XPos, texCo1.m_YPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo1.m_XPos, texCo0.m_YPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo0.m_YPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo1.m_YPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMin.m_ZPos);

		glNormal3d(1.0, 0.0, 0.0);
		glTexCoord2f(texCo1.m_YPos, texCo1.m_ZPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo0.m_YPos, texCo1.m_ZPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo0.m_YPos, texCo0.m_ZPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo1.m_YPos, texCo0.m_ZPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMax.m_ZPos);

		glNormal3d(0.0, 0.0, 1.0);
	    glTexCoord2f(texCo1.m_XPos, texCo1.m_YPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo1.m_XPos, texCo0.m_YPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo0.m_YPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo1.m_YPos);
		glVertex3f(blMax.m_XPos, blMin.m_YPos, blMax.m_ZPos);

		glNormal3d(-1.0, 0.0, 0.0);
		glTexCoord2f(texCo1.m_YPos, texCo0.m_ZPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_YPos,texCo0.m_ZPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_YPos, texCo1.m_ZPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo1.m_YPos, texCo1.m_ZPos);
		glVertex3f(blMin.m_XPos, blMin.m_YPos, blMin.m_ZPos);
	
		glNormal3d(0.0, 1.0, 0.0);
	    glTexCoord2f(texCo1.m_XPos, texCo0.m_ZPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo0.m_ZPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMax.m_ZPos);
	    
		glTexCoord2f(texCo0.m_XPos, texCo1.m_ZPos);
		glVertex3f(blMax.m_XPos, blMax.m_YPos, blMin.m_ZPos);
	    
		glTexCoord2f(texCo1.m_XPos, texCo1.m_ZPos);
		glVertex3f(blMin.m_XPos, blMax.m_YPos, blMin.m_ZPos);

    glEnd();
	glFlush();

	glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);

	if(m_IsTex)
	{
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();	
	}

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void CFlow3DView::OnSetBlockParameters() 
{
	// TODO: Add your command handler code here
	MyBlockDialog blockDialog;

	blockDialog.m_MaterialDialog.SetMaterial(doc->m_BlMaterial);
	blockDialog.SetZOffs(doc->m_BlZOffs);
	blockDialog.SetIsTex(m_IsTex);
	
	if(blockDialog.DoModal() == IDOK)
	{
		doc->m_BlMaterial = blockDialog.m_MaterialDialog.GetMaterial();
		doc->m_BlZOffs = blockDialog.GetZOffs();
		m_IsTex = blockDialog.GetIsTex();
		ReplaceBlTex();
	}

	DrawScene();
}

void CFlow3DView::OnShowHideBlock() 
{
	// TODO: Add your command handler code here
	m_IsShowBlock = !m_IsShowBlock;
}

void CFlow3DView::OnUpdateShowHideBlock(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	pCmdUI->SetCheck(m_IsShowBlock);
}

void CFlow3DView::CreateBlTex()
{
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

   glGenTextures(1, &blTexName);
   glBindTexture(GL_TEXTURE_2D, blTexName);

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
                   GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
                   GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, DEF_BLTEX_WIDTH, 
                DEF_BLTEX_HEIGHT, 0, GL_RGBA, GL_UNSIGNED_BYTE, 
				MyBlockDialog::blTex);
}

void CFlow3DView::ReplaceBlTex()
{	
	glBindTexture(GL_TEXTURE_2D, blTexName);
	
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, DEF_BLTEX_WIDTH, 
					DEF_BLTEX_HEIGHT, GL_RGBA, GL_UNSIGNED_BYTE, 
					MyBlockDialog::blTex);
}

void CFlow3DView::InitOldMembers()
{
	// Create old magic volume(needed in Mouse-move routine).
	m_OldMagicVolume = 
		MyFocusContextDialog::CreateMagicVol(doc->m_MagicVolType, CPoint3D(), 
											 DEF_MAGICCUBE_SIZE, 
											 DEF_MAGICSPHERE_RADIUS,
											 DEF_MAGICRECT_LENGTH,
											 DEF_MAGICRECT_HEIGHT,
											 DEF_MAGICRECT_WIDTH);

	m_OldContextProp.resPerc = doc->m_ContextProp.resPerc;
	m_OldContextProp.lineWidthPerc = doc->m_ContextProp.lineWidthPerc;
	m_OldContextProp.opacityPerc = doc->m_ContextProp.opacityPerc;

	m_OldAnimSpeed = doc->m_AnimSpeed;

	m_DeIntensity = DEF_DEPTH_INTENSITY;
	m_OldDeIntensity = m_DeIntensity;
	m_OldAbsorption = m_Absorption;
	m_OldPeriodLength = doc->m_PeriodLength;
	m_OldBlZOffs = doc->m_BlZOffs;

	m_OldMinPos = doc->m_MinPos;
	m_OldMaxPos = doc->m_MaxPos;
}

void CFlow3DView::InitBlockParams()
{
	//! Sould block be shown?
	m_IsShowBlock = DEF_ISSHOWBLOCK;

	//! Should block texture be used?
	m_IsTex = DEF_ISTEX;

	// White block texture as initialisation.
	MyBlockDialog::InitBlockTex();
	
	//! Load standard texture.
	MyBlockDialog::ReadBlTex(DEF_BLTEX_NAME);
	
	//! Create texture (white texture if texture loading failed).
	CreateBlTex();
}

void CFlow3DView::InitFoCoObj()
{
	// Init cross showing magic volume center.
	m_FoContextCross.SetLineWidth(DEF_AXIS_LINEWIDTH);
	m_FoContextCross.SetOffs(DEF_MAGICVOL_AXIS_OFFS);

	// Set magic volume type(one of sphere, cube, prism).
	doc->m_MagicVolType = DEF_MAGICVOL_INDEX;

	// Create magic volume.
	doc->m_MagicVolume = 
		MyFocusContextDialog::CreateMagicVol(doc->m_MagicVolType, CPoint3D(), 
											 DEF_MAGICCUBE_SIZE, 
											 DEF_MAGICSPHERE_RADIUS,
											 DEF_MAGICRECT_LENGTH,
											 DEF_MAGICRECT_HEIGHT,
											 DEF_MAGICRECT_WIDTH);
}
