// HistoData.cpp: implementation of the CHistoData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "HistoData.h"
#include "FLOAT.H"
#include "LIMITS.H"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHistoData::CHistoData()
{
	m_IntervNum = 0;
	m_Distribution = NULL;
	m_MaxScalar = FLT_MIN;
	m_MinScalar = FLT_MAX;
	m_MinDistr = INT_MAX;
	m_MaxDistr = INT_MIN;
}

CHistoData::~CHistoData()
{
	if(m_Distribution != NULL)
		delete [] m_Distribution;
}

CHistoData::CHistoData(CHistoDataStream *dataStream, int intervNum)
{
	m_IntervNum = intervNum;
	m_DataStream = dataStream;

	InitDistribution();

	m_MaxScalar = FLT_MIN;
	m_MinScalar = FLT_MAX;
	m_MinDistr = INT_MAX;
	m_MaxDistr = INT_MIN;
}

void CHistoData::CalcDistribution()
{
	//CalcMinMaxScalar();

	InitDistribution();

	float scalar;

	m_DataStream->Reset();
	while(m_DataStream->GetNext(scalar))
	{
		m_Distribution[CalcInterv(scalar)]++;
	
	}

	CalcMinMaxDistr();
}

int CHistoData::CalcInterv(float scalar)
{
	return (int)((scalar - m_MinScalar) * (float)(m_IntervNum - 1) / (m_MaxScalar - m_MinScalar))%
		   m_IntervNum;
}

void CHistoData::CalcMinMaxScalar()
{
	m_MaxScalar = FLT_MIN;
	m_MinScalar = FLT_MAX;

	float scalar;

	int oldType = m_DataStream->GetHistoType();

	m_DataStream->SetHistoType(DEF_ALL_REGIONS);

	m_DataStream->Reset();
	while(m_DataStream->GetNext(scalar))
	{
		if(scalar > m_MaxScalar)
			m_MaxScalar = scalar;
		if(scalar < m_MinScalar)
			m_MinScalar = scalar;
	}

	m_DataStream->SetHistoType(oldType);
}

void CHistoData::CalcMinMaxDistr()
{
	m_MaxDistr = INT_MIN;
	m_MinDistr = INT_MAX;

	for(int i=0; i < m_IntervNum; i++)
	{
		if(m_Distribution[i] > m_MaxDistr)
			m_MaxDistr = m_Distribution[i];

		if(m_Distribution[i] < m_MinDistr)
			m_MinDistr = m_Distribution[i];
	}
}

void CHistoData::InitDistribution()
{
	if(m_Distribution != NULL)	
		delete [] m_Distribution;		
	
	m_Distribution = new int[m_IntervNum];
	
	for(int i=0;i<m_IntervNum;i++)
	{
		m_Distribution[i] = 0;
	}
}

CHistoDataStream * CHistoData::GetDataStream()
{
	return m_DataStream;
}

void CHistoData::SetDataStream(CHistoDataStream *dataStream)
{
	m_DataStream = dataStream;
}

void CHistoData::SetIntervNum(int intervNum)
{
	m_IntervNum = intervNum;
	InitDistribution();
}

void CHistoData::SetMinMaxScalar(float minScalar, float maxScalar)
{
	m_MinScalar = minScalar;
	m_MaxScalar = maxScalar;
}
