// HistoData.h: interface for the CHistoData class.
//
//////////////////////////////////////////////////////////////////////
#include "VectorField.h"
#include "HistoDataStream.h"

#if !defined(AFX_HISTODATA_H__95F0D992_AB94_43ED_9BD3_C83C7F2F8577__INCLUDED_)
#define AFX_HISTODATA_H__95F0D992_AB94_43ED_9BD3_C83C7F2F8577__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//! Class representing the data of a histogram.
class CHistoData  
{
public:
	//! Sets minnal and maximal scalar value.
	void SetMinMaxScalar(float minScalar, float maxScalar);
	//! Sets number of histogram intervals.
	void SetIntervNum(int intervNum);
	//! Sets histogram data stream.
	void SetDataStream(CHistoDataStream *dataStream);
	//! Returns histogram data stream.
	CHistoDataStream * GetDataStream();
	//! Inits distribution array.
	void InitDistribution();
	//! Calculates minimum and maximum distribution values.
	void CalcMinMaxDistr();
	//! Calculates minimum and maximum scalar values.
	void CalcMinMaxScalar();
	//! Calculates interval index the scalar value belongs to.
	int CalcInterv(float scalar);
	//! Calculates data distribution.
	void CalcDistribution();

	//! Minimal distribution value.
	int m_MinDistr;
	//! Maximal distribution value.
	int m_MaxDistr;
	//! Minimal scalar value.
	float m_MinScalar;
	//! Maximal scalar value.
	float m_MaxScalar;
	//! Number of distribution intervals.
	int m_IntervNum;
	//! Distribution array.
	int *m_Distribution;
	//! Histogram data stream.
	CHistoDataStream *m_DataStream;
	//! Constructor setting data stream and interval number.
	CHistoData(CHistoDataStream *dataStream, int intervNum);
	//! Standard constructor.
	CHistoData();
	virtual ~CHistoData();

};

#endif // !defined(AFX_HISTODATA_H__95F0D992_AB94_43ED_9BD3_C83C7F2F8577__INCLUDED_)
