// HistoDataStream.cpp: implementation of the CHistoDataStream class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "HistoDataStream.h"
#include "Defines.h"
#include "StreamlinePiece.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHistoDataStream::CHistoDataStream()
{
	m_HistoType = DEF_ALL_REGIONS;
	m_PiecesList = NULL;
	m_CurrentPos = NULL;
	m_ScalarType = DEF_VELOCITY_INDEX;
}

CHistoDataStream::~CHistoDataStream()
{

}

bool CHistoDataStream::GetNext(float &scalar)
{
	bool result = false;
	while((m_CurrentPos != NULL) && (!result))
	{
		CStreamlinePiece * piece = 
			(CStreamlinePiece *) m_PiecesList->GetNext(m_CurrentPos);

		if(IsValid(piece))
		{
			result = true;
			scalar = GetScalar(piece);
		}
	}
	return result;
}

CHistoDataStream::HistoDataStream(CObList *piecesList, int histoType)
{
	m_HistoType = histoType;
	m_PiecesList = piecesList;
	m_CurrentPos = m_PiecesList->GetHeadPosition();
}

void CHistoDataStream::SetPiecesList(CObList *list)
{
	m_PiecesList = list;
	m_CurrentPos = list->GetHeadPosition();
}

CObList * CHistoDataStream::GetPiecesList()
{
	return m_PiecesList;
}

bool CHistoDataStream::GetFirst(float &scalar)
{
	Reset();
	return GetNext(scalar);
}

void CHistoDataStream::Reset()
{
	m_CurrentPos = m_PiecesList->GetHeadPosition();
}

float CHistoDataStream::GetScalar(CStreamlinePiece *piece)
{
	float result = 0.0f;
	CSamplePoint *start = piece->m_StartPoint;
	CSamplePoint *end = piece->m_StartPoint;
	CVector3D *tang0 = start->GetUnitVec();
	CVector3D *tang1 = end->GetUnitVec();

	CVector3D gradient;

	switch( m_ScalarType ) 
	{
		case DEF_VELOCITY_INDEX: 
			result = start->GetMagnitude();
			break;
		case DEF_ORIENT_X_INDEX: 
			result = (float)acos(tang0->m_XComp);
			break;
		case DEF_ORIENT_Y_INDEX: 
			result = (float)acos(tang0->m_YComp);
			break;
		case DEF_ORIENT_Z_INDEX: 
			result = (float)acos(tang0->m_YComp);
			break;
		case DEF_POS_X_INDEX:
			result = start->GetXPos();
			break;
		case DEF_POS_Y_INDEX:
			result = start->GetYPos();
			break;
		case DEF_POS_Z_INDEX:
			result = start->GetZPos();
			break;
		case DEF_DIF_ORIENT_INDEX:
			result = piece->CalcDifOrient();
			break;
		case DEF_DIF_VLEN_INDEX:
			result = piece->CalcDifVLen();
			break;
		default: 
			int index = m_ScalarType - DEF_SCALAR_NUM;
			result = start->GetAdditional(index);
			break;
	}
	return result;

}

void CHistoDataStream::SetScalarType(int scalarType)
{
	m_ScalarType = scalarType;
}

int CHistoDataStream::GetScalarType()
{
	return m_ScalarType;
}

void CHistoDataStream::SetHistoType(int type)
{
	m_HistoType = type;
}

bool CHistoDataStream::IsValid(CStreamlinePiece *piece)
{
	bool result = false;
	if((m_HistoType == DEF_ALL_REGIONS) ||
	   (m_HistoType == piece->m_RegIndex) ||
	   ((m_HistoType == DEF_FOCUS_INDEX) && piece->m_IsInside) ||
	   ((m_HistoType == DEF_CONTEXT_INDEX) && (!piece->m_IsInside)))
	{
		result = true;
	}

	return result;
}

int CHistoDataStream::GetHistoType()
{
	return m_HistoType;
}
