// Integrator.h: interface for the CIntegrator class.
//
//////////////////////////////////////////////////////////////////////
#include "Interpolator.h"
#include "VectorField.h"
#include "Defines.h"

#if !defined(AFX_INTEGRATOR_H__4B9E9F44_602D_4607_861B_F1572D65AFE9__INCLUDED_)
#define AFX_INTEGRATOR_H__4B9E9F44_602D_4607_861B_F1572D65AFE9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//! An abstract class that is the base class for all integrator-operators.
/*!	CIntegrator is the base class for all discrete operators like Euler-Op
	which approximate an integration of a streamline with stepsize dt
*/
class CIntegrator  
{
public:
	
	CIntegrator();
	//! Constructor with pointer to vectorfield, pointer to interpolator and dt as parameters
	CIntegrator(CVectorField *vectorField,CInterpolator *interpolator,float dt);
	virtual ~CIntegrator();

	//! Makes one integration-step and returns pointer to new samplepoint.
	virtual CSamplePoint *ComputeNext(CSamplePoint *lastPoint) = 0;
	//! Makes one integration-step in the other direction and returns pointer to new samplepoint.
	CSamplePoint *ComputeLast(CSamplePoint *nextPoint);
	//! Sets time-step value(should be little for obtaining good results).
	virtual void SetDt(float dt);
	//! Sets pointer to integrator.
	void SetInterpolator(CInterpolator *interpolator);
	//! Sets pointer to vectorfield the integrator operates on.
	void SetVectorField(CVectorField *vectorField);
protected:
	float m_Dt;
	CInterpolator *m_Interpolator;
	CVectorField *m_VectorField;
};

#endif // !defined(AFX_INTEGRATOR_H__4B9E9F44_602D_4607_861B_F1572D65AFE9__INCLUDED_)
