// LorenzSampler.cpp: implementation of the CLorenzSampler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "LorenzSampler.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CLorenzSampler::CLorenzSampler()
{
	m_Sigma = 0.0f;
	m_R = 0.0f;
	m_B = 0.0f;
	m_AdditionalSize = 0;
}

CLorenzSampler::~CLorenzSampler()
{

}

CLorenzSampler::CLorenzSampler(CPoint3D center, float sigma, float r, float b, int additionalSize)
{
	m_Center = center;
	m_Sigma = sigma;
	m_R = r;
	m_B = b;
	m_AdditionalSize = additionalSize;
}


CSamplePoint CLorenzSampler::ComputeSample()
{
	CSamplePoint result;

	float xPos = m_Pos.m_XPos - m_Center.m_XPos;
	float yPos = m_Pos.m_YPos - m_Center.m_YPos;
	float zPos = m_Pos.m_ZPos - m_Center.m_ZPos;

	m_Vector.m_XComp = m_Sigma * (yPos - xPos);
	m_Vector.m_YComp = m_R * xPos - yPos - xPos*zPos;
	m_Vector.m_ZComp = xPos * yPos - m_B*zPos;
	
	result.SetPos(m_Pos);
	result.SetVector(m_Vector);

	SetAddInfo(result.GetAdditional());
	
	return result;
}

void CLorenzSampler::SetCenter(CPoint3D center)
{
	m_Center = center;
}

CPoint3D CLorenzSampler::GetCenter()
{
	return m_Center;
}

void CLorenzSampler::SetSigma(float sigma)
{
	m_Sigma = sigma;
}

void CLorenzSampler::SetR(float r)
{
	m_R = r;
}

void CLorenzSampler::SetB(float b)
{
	m_B = b;
}