// LorenzSampler.h: interface for the CLorenzSampler class.
//
//////////////////////////////////////////////////////////////////////
#if !defined(AFX_LORENZSAMPLER_H__94532F94_F239_4AF3_908F_9C6DCC905658__INCLUDED_)
#define AFX_LORENZSAMPLER_H__94532F94_F239_4AF3_908F_9C6DCC905658__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Sampler.h"
#include "Defines.h"

//!  A class for sampling a vectorfield using the equitations of the Lorenz-attractor.
/*! The Lorenz-attractor is a chaotic attractor(meaning that it is a fractal).*/
class CLorenzSampler : public CSampler  
{
public:
	//! Initialises center with 0,0,0 and parameters with lorenz settings.
	/*! Lorenz settings: sigma = 10, r=28, b=8/3=ca. 2,667*/
	CLorenzSampler();

	//! Parameter sigma of the Lorenz-attractor.
	void SetSigma(float sigma);
	//! Parameter t of the Lorenz-attractor.
	void SetR(float r);
	//! Parameter b of the Lorenz-attractor.
	void SetB(float b);
	CPoint3D GetCenter();
	//! Sets center of the sampled field.
	void SetCenter(CPoint3D center);
	//! Computes samples using Lorenz-equitations with the parameters sigma,r and b.
	CLorenzSampler(CPoint3D center, float sigma, float r, float b, 
				   int additionalSize);
	virtual CSamplePoint ComputeSample();
	virtual ~CLorenzSampler();
private:
	
	float m_Sigma;
	float m_R;
	float m_B;

	CPoint3D m_Center;
};

#endif // !defined(AFX_LORENZSAMPLER_H__94532F94_F239_4AF3_908F_9C6DCC905658__INCLUDED_)
