// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Flow3D.h"

#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_OPTIONS_RECALC, OnRecalc)
	ON_COMMAND(ID_OPTIONS_CREATEANALYTICFIELD, OnCreateAnalyticfield)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers


void CMainFrame::OnRecalc() 
{
	// TODO: Add your command handler code here
	CFlow3DDoc * doc = (CFlow3DDoc *)GetActiveDocument();

	if(doc->GetCurrentField() != NULL)
	{
		char status[300];

		float dSep = doc->GetDSepPerc();
		float dTest = doc->GetDTest();
		float dt = doc->GetDt();

		int maxIterations = doc->GetMaxIterations();
		int caPerSP = doc->GetCaPerSP();
	
		char *interpolStr;
		char *integrStr;

		switch(doc->GetInterpolatorType())
		{
			case DEF_NN_INDEX: 
				interpolStr = "Nearest Neighbour";
				break;
			case DEF_TRILINEAR_INDEX:
				interpolStr = "Trilinear";
				break;
			case DEF_TRILINEARVL_INDEX:
				interpolStr = "Trilinear+VLength";
				break;
			default:
				interpolStr = "none";
				break;
		}

		switch(doc->GetIntegratorType())
		{
			case DEF_EULER_INDEX: 
				integrStr = "Euler";
				break;
			case DEF_RK2_INDEX:
				integrStr = "RK-2";
				break;
			case DEF_RK4_INDEX:
				integrStr = "RK-4";
				break;
			default:
				integrStr = "none";
				break;
		}

		sprintf(status,"%s, %s,dsep:%f%%,dtest:%f,max. it.:%d,ca per sp:%d, dt: %f",
				interpolStr,integrStr,dSep,dTest,maxIterations,caPerSP,dt);

		m_wndStatusBar.SetWindowText(status);
		doc->Recalc();
	}
	else
	{
		MessageBox("No vectorfield specified!",DEF_ERROR_STR,MB_OK);
	}
	
	m_wndStatusBar.SetWindowText("Ready");
}

void CMainFrame::OnCreateAnalyticfield() 
{
	// TODO: Add your command handler code here
	m_wndStatusBar.SetWindowText("Now creating field");
	CFlow3DDoc * doc = (CFlow3DDoc *)GetActiveDocument();

	doc->CreateField();	

	m_wndStatusBar.SetWindowText(DEF_READY_STR);
}
