// MyAlgoOptDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyAlgoOptDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyAlgoOptDialog dialog


MyAlgoOptDialog::MyAlgoOptDialog(CWnd* pParent /*=NULL*/)
	: CDialog(MyAlgoOptDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(MyAlgoOptDialog)
	m_IntegratorType = -1;
	m_InterpolatorType = -1;
	m_DSepPerc = 0.0f;
	m_DT = 0.0f;
	m_DTest = 0.0f;
	m_MaxIterations = 0;
	m_CaPerSP = 0;
	//}}AFX_DATA_INIT
}


void MyAlgoOptDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyAlgoOptDialog)
	DDX_Radio(pDX, IDC_EULERBUTTON, m_IntegratorType);
	DDX_Radio(pDX, IDC_NNBUTTON, m_InterpolatorType);
	DDX_Text(pDX, IDC_DSEPEDIT, m_DSepPerc);
	DDV_MinMaxFloat(pDX, m_DSepPerc, 0.5f, 100.f);
	DDX_Text(pDX, IDC_DTEDIT, m_DT);
	DDX_Text(pDX, IDC_DTESTEDIT, m_DTest);
	DDV_MinMaxFloat(pDX, m_DTest, 0.1f, 1.f);
	DDX_Text(pDX, IDC_MAXITEDIT, m_MaxIterations);
	DDV_MinMaxInt(pDX, m_MaxIterations, 1, 20000);
	DDX_Text(pDX, IDC_CAPERSPEDIT, m_CaPerSP);
	DDV_MinMaxInt(pDX, m_CaPerSP, 1, 200);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MyAlgoOptDialog, CDialog)
	//{{AFX_MSG_MAP(MyAlgoOptDialog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyAlgoOptDialog message handlers

float MyAlgoOptDialog::GetDSepPerc()
{
	return m_DSepPerc;
}

float MyAlgoOptDialog::GetDTest()
{
	return m_DTest;
}

float MyAlgoOptDialog::GetDt()
{
	return m_DT;
}

int MyAlgoOptDialog::GetMaxIterations()
{
	return m_MaxIterations;
}

int MyAlgoOptDialog::GetCaPerSP()
{
	return m_CaPerSP;
}

CIntegrator * MyAlgoOptDialog::CreateIntegrator(int integratorType,
												CVectorField *vectorField,
												float dt, CInterpolator *interpolator)
{
	CIntegrator *result;

	if(integratorType==DEF_EULER_INDEX)
	{
		result = new CEulerOp(vectorField,interpolator,dt);
	}
	else if(integratorType==DEF_RK4_INDEX)
	{
		result = new CRungeKutta4Op(vectorField,interpolator,dt);
	}
	else
	{
		result = new CRungeKutta2Op(vectorField,interpolator,dt);
	}
	return result;
}

CInterpolator * MyAlgoOptDialog::CreateInterpolator(int interpolatorType)
{
	CInterpolator *result;

	if(interpolatorType==DEF_NN_INDEX)
	{
		result = new CNearestNeighbour();
	}
	else if(interpolatorType==DEF_TRILINEAR_INDEX)
	{
		result = new CRectTrilinear();
	}
	else
	{
		result = new CRectTrilinVLength();
	}
	return result;
}

void MyAlgoOptDialog::InitAlgoOptDlg(float dSepPerc,float dTest,float dt,
									 int caPerSP,int maxIterations,int integrator,
									 int interpolator)
{

	m_DSepPerc = dSepPerc;
	m_DTest = dTest;
	m_DT = dt;
	m_CaPerSP = caPerSP;
	m_MaxIterations = maxIterations;
	m_IntegratorType = integrator;
	m_InterpolatorType = interpolator;
}

int MyAlgoOptDialog::GetInterpolatorType()
{
	return m_InterpolatorType;
}

int MyAlgoOptDialog::GetIntegratorType()
{
	return m_IntegratorType;
}
