// MyAxisCross.cpp: implementation of the MyAxisCross class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "MyAxisCross.h"
#include "Defines.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MyAxisCross::MyAxisCross()
{
	m_LineWidth = 0;
	m_Offs = 0;
	m_MinPoint = CPoint3D(0,0,0);
	m_MaxPoint = CPoint3D(0,0,0);
	m_XLineWidth = DEF_AXIS_LINEWIDTH;
	m_YLineWidth = DEF_AXIS_LINEWIDTH;
	m_ZLineWidth = DEF_AXIS_LINEWIDTH;
}

MyAxisCross::~MyAxisCross()
{

}

bool MyAxisCross::IsInside(CPoint3D position)
{
	return false;
}

void MyAxisCross::Draw()
{
	// Save old color.
	GLfloat oldRgba[4];
	glGetFloatv(GL_CURRENT_COLOR, oldRgba);

	// Save old line width.
	GLfloat oldLineWidth;
	glGetFloatv(GL_LINE_WIDTH, &oldLineWidth);

	glLineWidth(m_XLineWidth);
	glColor4ubv(m_XAxisRgba);
	glBegin(GL_LINES);
		glVertex3f(m_MinPoint.GetXPos() - m_Offs,m_Position.GetYPos(),
				   m_Position.GetZPos());
		glVertex3f(m_MaxPoint.GetXPos() + m_Offs,m_Position.GetYPos(),
				   m_Position.GetZPos());
	glEnd();

/*	glBegin(GL_LINES);
		glVertex3f(m_MinPoint.GetXPos() - m_Offs - 5,m_Position.GetYPos() - 4,
				   m_Position.GetZPos());
		glVertex3f(m_MinPoint.GetXPos() - m_Offs - 5,m_Position.GetYPos() + 4,
				   m_Position.GetZPos());
	glEnd();

	glBegin(GL_LINES);
		glVertex3f(m_MaxPoint.GetXPos() + m_Offs + 5,m_Position.GetYPos() - 4,
				   m_Position.GetZPos());
		glVertex3f(m_MaxPoint.GetXPos() + m_Offs + 5,m_Position.GetYPos() + 4,
				   m_Position.GetZPos());
	glEnd();
*/
	glLineWidth(m_YLineWidth);
	glColor4ubv(m_YAxisRgba);
	glBegin(GL_LINES);
		glVertex3f(m_Position.GetXPos(),m_MinPoint.GetYPos() - m_Offs,
				   m_Position.GetZPos());
		glVertex3f(m_Position.GetXPos(),m_MaxPoint.GetYPos() + m_Offs,
				   m_Position.GetZPos());
	glEnd();

	glLineWidth(m_ZLineWidth);
	glColor4ubv(m_ZAxisRgba);

	glBegin(GL_LINES);
		glVertex3f(m_Position.GetXPos(),m_Position.GetYPos(),
				   m_MinPoint.GetZPos() - m_Offs);
		glVertex3f(m_Position.GetXPos(),m_Position.GetYPos(),
				   m_MaxPoint.GetZPos() + m_Offs);
	glEnd();

	glColor4fv(oldRgba);
	glLineWidth(oldLineWidth);
	
}

void MyAxisCross::GetXAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		rgba[i] = m_XAxisRgba[i];
}

void MyAxisCross::SetXAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		m_XAxisRgba[i] = rgba[i];
}

void MyAxisCross::GetYAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		rgba[i] = m_YAxisRgba[i];
}

void MyAxisCross::SetYAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		m_YAxisRgba[i] = rgba[i];
}

void MyAxisCross::GetZAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		rgba[i] = m_ZAxisRgba[i];
}

void MyAxisCross::SetZAxisRgba(GLubyte *rgba)
{
	for(int i=0;i<4;i++)
		m_ZAxisRgba[i] = rgba[i];
}

MyAxisCross::MyAxisCross(CPoint3D position, CPoint3D minPoint, CPoint3D maxPoint, 
						 float xLineWidth, float yLineWidth, float zLineWidth,
						 float offs, GLubyte *xAxisRgba, GLubyte *yAxisRgba, 
						 GLubyte *zAxisRgba)
{
	m_Position = position;
	m_MinPoint = minPoint;
	m_MaxPoint = maxPoint;
	m_LineWidth = 0;
	m_XLineWidth = xLineWidth;
	m_YLineWidth = yLineWidth;
	m_ZLineWidth = zLineWidth;
	m_Offs = offs;

	SetXAxisRgba(xAxisRgba);
	SetYAxisRgba(yAxisRgba);
	SetZAxisRgba(zAxisRgba);

	for(int i=0;i<4;i++)
		m_Rgba[4] = 0;
}

void MyAxisCross::SetMinPoint(CPoint3D minPoint)
{
	m_MinPoint = minPoint;
}

void MyAxisCross::SetMaxPoint(CPoint3D maxPoint)
{
	m_MaxPoint = maxPoint;
}

CPoint3D MyAxisCross::GetMinPoint()
{
	return m_MinPoint;
}

CPoint3D MyAxisCross::GetMaxPoint()
{
	return m_MaxPoint;
}

void MyAxisCross::SetOffs(float offs)
{
	m_Offs = offs;
}

float MyAxisCross::GetOffs()
{
	return m_Offs;
}

void MyAxisCross::SetRange(MyCube *cube)
{
	m_MinPoint = cube->GetMinPoint();
	m_MaxPoint = cube->GetMaxPoint();
}

void MyAxisCross::SetRange(MyRectangle *rect)
{
	m_MinPoint = rect->GetMinPoint();
	m_MaxPoint = rect->GetMaxPoint();
}

void MyAxisCross::Init(MyObject *obj)
{
	MyAxisCross *cross = (MyAxisCross *)obj;
	m_MinPoint = cross->GetMinPoint();
	m_MaxPoint = cross->GetMaxPoint();
	m_Offs = cross->GetOffs();
	m_Position = cross->GetPosition();

	cross->GetXAxisRgba(m_XAxisRgba);
	cross->GetYAxisRgba(m_YAxisRgba);
	cross->GetZAxisRgba(m_ZAxisRgba);
	cross->GetRgba(m_Rgba);

	m_LineWidth = cross->GetLineWidth();
	m_XLineWidth = cross->GetXLineWidth();
	m_YLineWidth = cross->GetYLineWidth();
	m_ZLineWidth = cross->GetZLineWidth();
}

void MyAxisCross::SetXLineWidth(float lineWidth)
{
	m_XLineWidth = lineWidth;
}

void MyAxisCross::SetYLineWidth(float lineWidth)
{
	m_YLineWidth = lineWidth;
}

void MyAxisCross::SetZLineWidth(float lineWidth)
{
	m_ZLineWidth = lineWidth;
}

float MyAxisCross::GetXLineWidth()
{
	return m_XLineWidth;
}

float MyAxisCross::GetYLineWidth()
{
	return m_YLineWidth;
}

float MyAxisCross::GetZLineWidth()
{	
	return m_ZLineWidth;
}

void MyAxisCross::SetLineWidth(float lineWidth)
{
	m_LineWidth = lineWidth;

	m_XLineWidth = lineWidth;
	m_YLineWidth = lineWidth;
	m_ZLineWidth = lineWidth;

}

void MyAxisCross::SetRgba(GLubyte *rgba)
{
	SetXAxisRgba(rgba);
	SetYAxisRgba(rgba);
	SetZAxisRgba(rgba);

	for(int i=0;i<4;i++)
		m_Rgba[i] = rgba[i];
}