// MyAxisCross.h: interface for the MyAxisCross class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MYAXISCROSS_H__1A343C0A_D72E_4C8C_8633_29E705D2BF04__INCLUDED_)
#define AFX_MYAXISCROSS_H__1A343C0A_D72E_4C8C_8633_29E705D2BF04__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MyObject.h"
#include "Point3D.h"	// Added by ClassView
#include "MyCube.h"
#include "MyRectangle.h"
#include "gl/glut.h"

//! Class representing a coordinate axis cross.
class MyAxisCross : public MyObject  
{
public:
	virtual void SetLineWidth(float lineWidth);
	virtual void SetRgba(GLubyte *rgba);
	// Returns line width of z-axis.
	float GetZLineWidth();
	//! Returns line width of y-axis.
	float GetYLineWidth();
	//! Returns line width of x-axis.
	float GetXLineWidth();
	//! Sets line width of z-axis.
	void SetZLineWidth(float lineWidth);
	//! Sets line width of y-axis.
	void SetYLineWidth(float lineWidth);
	//! Sets line width of x-axis.
	void SetXLineWidth(float lineWidth);
	
	virtual void Init(MyObject *obj);
	//! Sets maximal and minimal point according to rect-boundaries.
	void SetRange(MyRectangle *rect);
	//! Sets maximal and minimal point according to cube-boundaries.
	void SetRange(MyCube *cube);
	//! Returns offs(=length of axis standing out of maximal/minimal point.
	float GetOffs();
	//! Sets offs(=length of axis standing out of maximal/minimal point.
	void SetOffs(float offs);
	//! Returns point with maximal xyz - coordinates.
	CPoint3D GetMaxPoint();
	//! Returns point with minimal xyz - coordinates.
	CPoint3D GetMinPoint();
	//! Sets point with maximal xyz - coordinates.
	void SetMaxPoint(CPoint3D maxPoint);
	//! Sets point with minimal xyz - coordinates.
	void SetMinPoint(CPoint3D minPoint);
		
	//! Returns rgba-value of x-axis.
	void GetXAxisRgba(GLubyte *rgba);
	//! Returns rgba-value of y-axis.
	void GetYAxisRgba(GLubyte *rgba);
	//! Returns rgba-value of z-axis.
	void GetZAxisRgba(GLubyte *rgba);
	
	//! Sets rgba-value of y-axis.
	void SetYAxisRgba(GLubyte *rgba);
	//! Sets rgba-value of z-axis.
	void SetXAxisRgba(GLubyte *rgba);
	//! Sets rgba-value of z-axis.
	void SetZAxisRgba(GLubyte *rgba);

	virtual void Draw();
	virtual bool IsInside(CPoint3D position);
	
	MyAxisCross(CPoint3D position,CPoint3D minPoint, CPoint3D maxPoint, 
				float xLineWidth, float yLineWidth, float zLineWidth,
				float offs, GLubyte *xAxisRgba,	GLubyte *yAxisRgba, 
				GLubyte *zAxisRgba);

	MyAxisCross();
	virtual ~MyAxisCross();
	
protected:
	GLubyte m_XAxisRgba[4];
	GLubyte m_YAxisRgba[4];
	GLubyte m_ZAxisRgba[4];

	float m_XLineWidth;
	float m_YLineWidth;
	float m_ZLineWidth;

	CPoint3D m_MaxPoint;
	CPoint3D m_MinPoint;

	float m_Offs;

};

#endif // !defined(AFX_MYAXISCROSS_H__1A343C0A_D72E_4C8C_8633_29E705D2BF04__INCLUDED_)
