// MyContTransfDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyContTransfDialog.h"
#include "Defines.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyContTransfDialog dialog


MyContTransfDialog::MyContTransfDialog(CWnd* pParent /*=NULL*/)
	: CDialog(MyContTransfDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(MyContTransfDialog)
	m_InterPos = 0;
	//}}AFX_DATA_INIT
}


void MyContTransfDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyContTransfDialog)
	DDX_Slider(pDX, IDC_INTERPOSSLIDER, m_InterPos);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MyContTransfDialog, CDialog)
	//{{AFX_MSG_MAP(MyContTransfDialog)
	ON_WM_PAINT()
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_INTERPOSSLIDER, OnReleasedCaptureInterPosSlider)
	ON_BN_CLICKED(IDC_MINPOSBUTTON, OnMinPosButton)
	ON_BN_CLICKED(IDC_INTERPOSBUTTON, OnInterPosButton)
	ON_BN_CLICKED(IDC_MAXPOSBUTTON, OnMaxPosButton)
	ON_BN_CLICKED(IDC_HISTOBUTTON, OnHistoButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyContTransfDialog message handlers

BOOL MyContTransfDialog::OnInitDialog() 
{	
	CSliderCtrl *pMiddleValSlider = (CSliderCtrl *)GetDlgItem( IDC_INTERPOSSLIDER );

	pMiddleValSlider->SetRange(DEF_INTERPOSSLIDER_MIN, DEF_INTERPOSSLIDER_MAX);

	m_Histo1Dialog.m_IsRegions = false;
	m_Histo1Dialog.Create(IDD_HISTO1_DIALOG,this);

	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

float MyContTransfDialog::GetInterPos()
{
	return (float)m_InterPos/DEF_INTERPOSSLIDER_MAX;
}

void MyContTransfDialog::SetInterPos(float interPos)
{
	m_InterPos = (int)(interPos * (float)DEF_INTERPOSSLIDER_MAX);
}

void MyContTransfDialog::SetRGBA(int index, GLubyte *rgba)
{
	m_PropDialog[index].m_ColorDialog.m_cc.rgbResult = 
		rgba[0] + rgba[1]*256 + rgba[2]*256*256;

	 m_PropDialog[index].m_Opacity = rgba[3];
}

void MyContTransfDialog::GetRGBA(int index, GLubyte *rgba)
{
	m_PropDialog[index].GetRgb(rgba);

	rgba[3] = m_PropDialog[index].m_Opacity;
}

void MyContTransfDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CFrameWnd * pColorImage = (CFrameWnd *)GetDlgItem( IDC_CONT_COLORIMG );
	CFrameWnd * pOpacityImage = (CFrameWnd *)GetDlgItem( IDC_CONT_OPACITYIMG );

	CSliderCtrl * pInterPosSlider = 
		(CSliderCtrl *)GetDlgItem ( IDC_INTERPOSSLIDER );
	
	CRect colorRect;
	
	pColorImage->GetWindowRect(colorRect);
	
	CPaintDC colorDC(pColorImage); // device context for painting
	CPaintDC opacityDC(pOpacityImage);

	HDC hColorDC = colorDC.GetSafeHdc();
	HDC hOpacityDC = opacityDC.GetSafeHdc();

	float interPos = 
		(float)pInterPosSlider->GetPos() / (float)DEF_INTERPOSSLIDER_MAX;

	const int size = DEF_CONTTRANSFVALUES_NUM + 1;
	int colIndex = size - 1;

	GLubyte color[size][4];
	
	for(int i = 0; i < DEF_CONTTRANSFVALUES_NUM; i++)
		GetRGBA(i,color[i]);
	
	// Draw continous color-code transfer function.
	for(i=0;i<colorRect.Height();i++)
	{
		MyHisto1Dialog::MapValue2Color_3(color[colIndex],color[0],color[1],color[2],
									     (float)i,0.0f,interPos,(float)colorRect.Height());

		for(int j=0;j<colorRect.Width();j++)
		{
			SetPixel(hColorDC,j,i,color[colIndex][0]+color[colIndex][1]*256+
					 color[colIndex][2]*256*256);
		
			// Draw opacity.
			SetPixel(hOpacityDC,j,i,color[colIndex][3]+color[colIndex][3]*256+
					 color[colIndex][3]*256*256);
		}
	}
	// Do not call CDialog::OnPaint() for painting messages
}


void MyContTransfDialog::MapValue2Prop_3(ContTransfProp &result, float value, ContTransfProp *props, 
										 float *pos)
{
	if(value < pos[1])
		MapValue2Prop(result, value,props,pos);
	else
		MapValue2Prop(result, value,&props[1],&pos[1]);
}

void MyContTransfDialog::MapValue2Prop(ContTransfProp &result, float value, ContTransfProp *props, 
									   float *pos)
{
	float factor = 1.0f;

	float dif = pos[1] - pos[0];
	if(dif != 0)
		factor = (value - pos[0]) / dif;

	for(int i=0;i<4;i++)
		result.rgba[i] = (GLubyte)(props[0].rgba[i] + 
			((float)props[1].rgba[i] - (float)props[0].rgba[i]) * factor);

	result.lineWidth = 
		(props[0].lineWidth + (props[1].lineWidth - props[0].lineWidth) * factor);
	result.res = (props[0].res + (int)((float)(props[1].res - props[0].res) * factor));
}

int MyContTransfDialog::GetRes(int index)
{
	return m_PropDialog[index].m_Res;
}

void MyContTransfDialog::SetRes(int index, int res)
{
	m_PropDialog[index].m_Res = res;
}

GLubyte MyContTransfDialog::GetOpacity(int index)
{
	return (GLubyte)m_PropDialog[index].m_Opacity;
}

void MyContTransfDialog::SetOpacity(int index, GLubyte opacity)
{
	m_PropDialog[index].m_Opacity = opacity;
}

void MyContTransfDialog::OnReleasedCaptureInterPosSlider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pInterPosSlider = (CSliderCtrl*)GetDlgItem( IDC_INTERPOSSLIDER );
		
		m_Histo1Dialog.m_InterPos = 
			pInterPosSlider->GetPos()/(float)DEF_INTERPOSSLIDER_MAX;

		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

void MyContTransfDialog::OnMinPosButton() 
{
	// TODO: Add your control notification handler code here
	m_PropDialog[0].DoModal();
	RedrawWindow();
}

void MyContTransfDialog::OnInterPosButton() 
{
	// TODO: Add your control notification handler code here
	m_PropDialog[1].DoModal();
	RedrawWindow();
}

void MyContTransfDialog::OnMaxPosButton() 
{
	// TODO: Add your control notification handler code here
	m_PropDialog[2].DoModal();
	RedrawWindow();	
}

void MyContTransfDialog::SetContProp(int index, ContTransfProp &prop)
{
	m_PropDialog[index].SetLineWidth(prop.lineWidth);
	m_PropDialog[index].m_Res = prop.res;
	SetRGBA(index,prop.rgba);
}

void MyContTransfDialog::GetContProp(int index, ContTransfProp &prop)
{
	prop.lineWidth = m_PropDialog[index].GetLineWidth();
	prop.res = m_PropDialog[index].m_Res;
	GetRGBA(index,prop.rgba);
}

void MyContTransfDialog::OnHistoButton() 
{
	// TODO: Add your control notification handler code here
	if(!m_Histo1Dialog.IsWindowVisible())
	{
		for(int i=0;i<DEF_CONTTRANSFVALUES_NUM;i++)
			GetContProp(i,m_Histo1Dialog.m_ContProp[i]);
		
		CSliderCtrl *pInterPosSlider = (CSliderCtrl *)GetDlgItem( IDC_INTERPOSSLIDER );

		m_Histo1Dialog.m_InterPos = 
			(float)pInterPosSlider->GetPos() / (float)DEF_INTERPOSSLIDER_MAX;

		m_Histo1Dialog.ShowWindow(SW_SHOW);
	}
}
