// MyCube.cpp: implementation of the MyCube class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "MyCube.h"
#include "MyRectangle.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MyCube::MyCube()
{
	for(int i=0;i<4;i++)
		m_Rgba[i] = 0;
	
	m_Size = 0;
	m_LineWidth = 0;
	m_Position = CPoint3D(0,0,0);

	UpdateMinPoint();
	UpdateMaxPoint();
}

MyCube::~MyCube()
{

}

void MyCube::SetSize(float size)
{
	m_Size = size;
	UpdateMinPoint();
	UpdateMaxPoint();
}

float MyCube::GetSize()
{
	return m_Size;
}

void MyCube::UpdateMinPoint()
{
	// Calculate magic cube minimal point.

	m_MinPoint = CPoint3D(m_Position.m_XPos - m_Size * 0.5f,
						  m_Position.m_YPos - m_Size * 0.5f,
						  m_Position.m_ZPos - m_Size * 0.5f);
}

void MyCube::UpdateMaxPoint()
{
	// Calculate magic cube maximal point.

	m_MaxPoint = CPoint3D(m_Position.m_XPos + m_Size * 0.5f,
						  m_Position.m_YPos + m_Size * 0.5f,
						  m_Position.m_ZPos + m_Size * 0.5f);
}

void MyCube::Draw()
{
	MyRectangle rect = MyRectangle(m_Position, m_Size, m_Size, 
								   m_Size, m_LineWidth, m_Rgba);

	rect.Draw();
}

MyCube::MyCube(CPoint3D position, float size, float lineWidth, 
			   GLubyte *rgba)
{
	m_Position = position;
	m_Size = size;
	m_LineWidth = lineWidth;
	SetRgba(rgba);
}

bool MyCube::IsInside(CPoint3D pos)
{
	return pos.Inside(m_MinPoint,m_MaxPoint);
}

void MyCube::SetPosition(CPoint3D position)
{
	m_Position = position;
	UpdateMinPoint();
	UpdateMaxPoint();
}

CPoint3D MyCube::GetMinPoint()
{
	return m_MinPoint;
}

CPoint3D MyCube::GetMaxPoint()
{
	return m_MaxPoint;
}

void MyCube::Init(MyObject *obj)
{
	MyCube *cube = (MyCube *)obj;

	SetPosition(cube->GetPosition());
	SetSize(cube->GetSize());
	cube->GetRgba(m_Rgba);

	m_LineWidth = cube->GetLineWidth();
}
