// MyEditRegionsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyEditRegionsDialog.h"
#include "MyHisto1Dialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyEditRegionsDialog dialog


MyEditRegionsDialog::MyEditRegionsDialog(CWnd* pParent /*=NULL*/)
	: CDialog(MyEditRegionsDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(MyEditRegionsDialog)
	m_Region2 = 0;
	m_Region3 = 0;
	m_Region4 = 0;
	m_Region5 = 0;
	m_Region1 = 0;
	//}}AFX_DATA_INIT
}


void MyEditRegionsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyEditRegionsDialog)
	DDX_Slider(pDX, IDC_REGION2SLIDER, m_Region2);
	DDX_Slider(pDX, IDC_REGION3SLIDER, m_Region3);
	DDX_Slider(pDX, IDC_REGION4SLIDER, m_Region4);
	DDX_Slider(pDX, IDC_REGION5SLIDER, m_Region5);
	DDX_Slider(pDX, IDC_REGION1SLIDER, m_Region1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MyEditRegionsDialog, CDialog)
	//{{AFX_MSG_MAP(MyEditRegionsDialog)
	ON_WM_PAINT()
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_REGION1SLIDER, OnReleasedcaptureRegion1slider)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_REGION2SLIDER, OnReleasedcaptureRegion2slider)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_REGION3SLIDER, OnReleasedcaptureRegion3slider)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_REGION4SLIDER, OnReleasedcaptureRegion4slider)
	ON_NOTIFY(NM_RELEASEDCAPTURE, IDC_REGION5SLIDER, OnReleasedcaptureRegion5slider)
	ON_BN_CLICKED(IDC_HISTOBUTTON, OnHistoButton)
	ON_BN_CLICKED(IDC_PROPREG1BUTTON, OnPropReg1Button)
	ON_BN_CLICKED(IDC_PROPREG2BUTTON, OnPropReg2Button)
	ON_BN_CLICKED(IDC_PROPREG3BUTTON, OnPropReg3Button)
	ON_BN_CLICKED(IDC_PROPREG4BUTTON, OnPropReg4Button)
	ON_BN_CLICKED(IDC_PROPREG5BUTTON, OnPropReg5Button)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyEditRegionsDialog message handlers

void MyEditRegionsDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CFrameWnd *pAmbientImage = (CFrameWnd *)GetDlgItem( IDC_REGIONSIMG_AMB );
	CFrameWnd *pDiffuseImage = (CFrameWnd *)GetDlgItem( IDC_REGIONSIMG_DIF );
	CFrameWnd *pSpecularImage = (CFrameWnd *)GetDlgItem( IDC_REGIONSIMG_SPEC );
	CFrameWnd *pOpacityImage = (CFrameWnd *)GetDlgItem( IDC_REGIONSIMG_OPACITY );

	CSliderCtrl *pRegionSlider[DEF_REGIONS_NUM];

	pRegionSlider[0] = (CSliderCtrl*)GetDlgItem( IDC_REGION1SLIDER );
	pRegionSlider[1] = (CSliderCtrl*)GetDlgItem( IDC_REGION2SLIDER );
	pRegionSlider[2] = (CSliderCtrl*)GetDlgItem( IDC_REGION3SLIDER );
	pRegionSlider[3] = (CSliderCtrl*)GetDlgItem( IDC_REGION4SLIDER );
	pRegionSlider[4] = (CSliderCtrl*)GetDlgItem( IDC_REGION5SLIDER );

	CRect ambientRect;
	CRect specularRect;
	CRect diffuseRect;
	CRect opacityRect;

	pAmbientImage->GetWindowRect(ambientRect);
	pDiffuseImage->GetWindowRect(diffuseRect);
	pSpecularImage->GetWindowRect(specularRect);
	pOpacityImage->GetWindowRect(opacityRect);

	CPaintDC ambientDC(pAmbientImage); 
	CPaintDC diffuseDC(pDiffuseImage); 
	CPaintDC specularDC(pSpecularImage); 
	CPaintDC opacityDC(pOpacityImage);

	HDC hAmbientDC = ambientDC.GetSafeHdc();
	HDC hDiffuseDC = diffuseDC.GetSafeHdc();
	HDC hSpecularDC = specularDC.GetSafeHdc();
	HDC hOpacityDC = opacityDC.GetSafeHdc();

	RegProp prop[DEF_REGIONS_NUM];
	
	for(int i=0;i<DEF_REGIONS_NUM;i++)
		prop[i].pos = (float)pRegionSlider[i]->GetPos()/(float)DEF_REGIONSLIDER_MAX;
	
	CStreamlineMaterial material;

	for(i=0;i<ambientRect.Height();i++)
	{
		int index = MapValue2PropNr(prop, DEF_REGIONS_NUM, (float)i, 
								    0.0f, (float)ambientRect.Height());

		if(index < DEF_REGIONS_NUM)
			material = GetMaterial(index);
		else
			material = m_DefaultProp.material;

		for(int j=0;j<ambientRect.Width();j++)
		{
			SetPixel(hAmbientDC,j,i,material.m_AmbientRGB[0]+
					 material.m_AmbientRGB[1]*256+material.m_AmbientRGB[2]*256*256);
		}
		for(j=0;j<diffuseRect.Width();j++)
		{
			SetPixel(hDiffuseDC,j,i,material.m_DiffuseRGB[0]+
					 material.m_DiffuseRGB[1]*256+material.m_DiffuseRGB[2]*256*256);
		}
		for(j=0;j<specularRect.Width();j++)
		{
			SetPixel(hSpecularDC,j,i,material.m_SpecularRGB[0]+
									 material.m_SpecularRGB[1]*256+
									 material.m_SpecularRGB[2]*256*256);
		}
		for(j=0;j<opacityRect.Width();j++)
		{
			SetPixel(hOpacityDC,j,i,material.m_Opacity+
									material.m_Opacity*256+
									material.m_Opacity*256*256);
		}
	}

	pAmbientImage->RedrawWindow();
	pDiffuseImage->RedrawWindow();
	pSpecularImage->RedrawWindow();

	// Do not call CDialog::OnPaint() for painting messages
}

BOOL MyEditRegionsDialog::OnInitDialog() 
{
	// TODO: Add extra initialization here
	CSliderCtrl *pRegionSlider[DEF_REGIONS_NUM];
	
	pRegionSlider[0] = (CSliderCtrl*)GetDlgItem( IDC_REGION1SLIDER );
	pRegionSlider[1] = (CSliderCtrl*)GetDlgItem( IDC_REGION2SLIDER );
	pRegionSlider[2] = (CSliderCtrl*)GetDlgItem( IDC_REGION3SLIDER );
	pRegionSlider[3] = (CSliderCtrl*)GetDlgItem( IDC_REGION4SLIDER );
	pRegionSlider[4] = (CSliderCtrl*)GetDlgItem( IDC_REGION5SLIDER );

	for(int i=0;i<DEF_REGIONS_NUM;i++)
		pRegionSlider[i]->SetRange(DEF_REGIONSLIDER_MIN,DEF_REGIONSLIDER_MAX);
	
	m_Histo1Dialog.m_IsRegions = true;
	m_Histo1Dialog.Create(IDD_HISTO1_DIALOG,this);
	CDialog::OnInitDialog();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void MyEditRegionsDialog::OnReleasedcaptureRegion1slider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	this->RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider = (CSliderCtrl*)GetDlgItem( IDC_REGION1SLIDER );
		
		m_Histo1Dialog.m_Pos[0] = 
				pRegionSlider->GetPos()/(float)DEF_REGIONSLIDER_MAX;
		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

void MyEditRegionsDialog::OnReleasedcaptureRegion2slider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	this->RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider = (CSliderCtrl*)GetDlgItem( IDC_REGION2SLIDER );
		
		m_Histo1Dialog.m_Pos[1] = 
				pRegionSlider->GetPos()/(float)DEF_REGIONSLIDER_MAX;
		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

void MyEditRegionsDialog::OnReleasedcaptureRegion3slider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	this->RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider = (CSliderCtrl*)GetDlgItem( IDC_REGION3SLIDER );
		
		m_Histo1Dialog.m_Pos[2] = 
				pRegionSlider->GetPos()/(float)DEF_REGIONSLIDER_MAX;
		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

void MyEditRegionsDialog::OnReleasedcaptureRegion4slider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	this->RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider = (CSliderCtrl*)GetDlgItem( IDC_REGION4SLIDER );
		
		m_Histo1Dialog.m_Pos[3] = 
			pRegionSlider->GetPos()/(float)DEF_REGIONSLIDER_MAX;
		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

void MyEditRegionsDialog::OnReleasedcaptureRegion5slider(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	this->RedrawWindow();
	if(m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider = (CSliderCtrl*)GetDlgItem( IDC_REGION5SLIDER );
		
		m_Histo1Dialog.m_Pos[4] = 
			pRegionSlider->GetPos()/(float)DEF_REGIONSLIDER_MAX;
		m_Histo1Dialog.RedrawWindow();		
	}
	*pResult = 0;
}

float MyEditRegionsDialog::GetRegionPos(int index)
{
	int sliderPos;

	switch(index)
	{
		case 0:
			sliderPos = m_Region1;
			break;
		case 1:
			sliderPos = m_Region2;
			break;
		case 2:
			sliderPos = m_Region3;
			break;
		case 3:
			sliderPos = m_Region4;
			break;
		case 4:
			sliderPos = m_Region5;
			break;
		default:
			sliderPos = m_Region1;
	}

	return (float)sliderPos/(float)DEF_REGIONSLIDER_MAX;
}

void MyEditRegionsDialog::SetRegionPos(int index, float pos)
{
	int sliderPos = (int)(pos*DEF_REGIONSLIDER_MAX);

	switch(index)
	{
		case 0:
			m_Region1 = sliderPos;
			break;
		case 1:
			m_Region2 = sliderPos;
			break;
		case 2:
			m_Region3 = sliderPos;
			break;
		case 3:
			m_Region4 = sliderPos;
			break;
		case 4:
			m_Region5 = sliderPos;
			break;
		default:
			m_Region1 = sliderPos;
	}
}

int MyEditRegionsDialog::MapValue2PropNr(RegProp *prop, int maxIndex, float value, 
										 float minValue, float maxValue)
{
	int result = 0;
	float factor = 1;
	if(maxValue != minValue)
		factor = (value - minValue)/(maxValue - minValue);

	while((factor > prop[result].pos) && (result < maxIndex))
	{
		result ++;
	}

	return result;
}

int MyEditRegionsDialog::GetRes(int index)
{
	return m_RegPropDialog[index].m_Res;
}

void MyEditRegionsDialog::SetRes(int index, int res)
{
	m_RegPropDialog[index].m_Res = res;
}

void MyEditRegionsDialog::OnHistoButton() 
{
	// TODO: Add your control notification handler code here
	if(!m_Histo1Dialog.IsWindowVisible())
	{
		CSliderCtrl *pRegionSlider[DEF_REGIONS_NUM];

		pRegionSlider[0] = (CSliderCtrl*)GetDlgItem( IDC_REGION1SLIDER );
		pRegionSlider[1] = (CSliderCtrl*)GetDlgItem( IDC_REGION2SLIDER );
		pRegionSlider[2] = (CSliderCtrl*)GetDlgItem( IDC_REGION3SLIDER );
		pRegionSlider[3] = (CSliderCtrl*)GetDlgItem( IDC_REGION4SLIDER );
		pRegionSlider[4] = (CSliderCtrl*)GetDlgItem( IDC_REGION5SLIDER );

		for(int i=0;i<DEF_REGIONS_NUM;i++)
			m_Histo1Dialog.m_Pos[i] = 
				pRegionSlider[i]->GetPos()/(float)DEF_REGIONSLIDER_MAX;

		m_Histo1Dialog.ShowWindow(SW_SHOW);
	}
}

CStreamlineMaterial MyEditRegionsDialog::GetMaterial(int index)
{
	return m_RegPropDialog[index].m_MaterialDialog.GetMaterial();
}

void MyEditRegionsDialog::SetMaterial(int index, CStreamlineMaterial material)
{
	m_RegPropDialog[index].m_MaterialDialog.SetMaterial(material);
}

void MyEditRegionsDialog::OnPropReg1Button() 
{
	// TODO: Add your control notification handler code here
	m_RegPropDialog[0].DoModal();
	RedrawWindow();
}


void MyEditRegionsDialog::OnPropReg2Button() 
{
	// TODO: Add your control notification handler code here
	m_RegPropDialog[1].DoModal();
	RedrawWindow();
}

void MyEditRegionsDialog::OnPropReg3Button() 
{
	// TODO: Add your control notification handler code here
	m_RegPropDialog[2].DoModal();
	RedrawWindow();
}

void MyEditRegionsDialog::OnPropReg4Button() 
{
	// TODO: Add your control notification handler code here
	m_RegPropDialog[3].DoModal();
	RedrawWindow();
}

void MyEditRegionsDialog::OnPropReg5Button() 
{
	// TODO: Add your control notification handler code here
	m_RegPropDialog[4].DoModal();
	RedrawWindow();
}

void MyEditRegionsDialog::GetRegProp(int index, RegProp &prop)
{
	prop.lineWidth = m_RegPropDialog[index].GetLineWidth();
	prop.res = m_RegPropDialog[index].m_Res;
	prop.material = m_RegPropDialog[index].
						m_MaterialDialog.GetMaterial();
	prop.pos  = GetRegionPos(index);
}

void MyEditRegionsDialog::SetRegProp(int index, RegProp &prop)
{
	m_RegPropDialog[index].SetLineWidth(prop.lineWidth);
	m_RegPropDialog[index].m_Res = prop.res;
	m_RegPropDialog[index].m_MaterialDialog.SetMaterial(prop.material);
	SetRegionPos(index, prop.pos);
}

void MyEditRegionsDialog::SetDefaultProp(RegProp &prop)
{
	m_DefaultProp.lineWidth = prop.lineWidth;
	m_DefaultProp.res = prop.res;
	m_DefaultProp.material = prop.material;
	m_DefaultProp.pos = prop.pos;
}
