// MyFocusContextDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyFocusContextDialog.h"
#include "Defines.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyFocusContextDialog dialog


MyFocusContextDialog::MyFocusContextDialog(CWnd* pParent /*=NULL*/)
	: CDialog(MyFocusContextDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(MyFocusContextDialog)
	m_Opacity = 0;
	m_IsCutLines = FALSE;
	m_MagicVolType = -1;
	m_ContextRes = 0;
	m_LineWidth = 0;
	m_RecalcType = -1;
	//}}AFX_DATA_INIT
}

int MyFocusContextDialog::GetOpacity()
{
	return m_Opacity;
}

bool MyFocusContextDialog::GetIsCutLines()
{
	return (m_IsCutLines == TRUE);
}

void MyFocusContextDialog::SetIsCutLines(bool isCutLines)
{
	m_IsCutLines = isCutLines;
}

void MyFocusContextDialog::SetOpacity(int opacity)
{
	m_Opacity = opacity;
}

int MyFocusContextDialog::GetContextRes()
{
	return m_ContextRes;
}

void MyFocusContextDialog::SetContextRes(int contextRes)
{
	m_ContextRes = contextRes;
}

void MyFocusContextDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyFocusContextDialog)
	DDX_Slider(pDX, IDC_OPACITYSLIDER, m_Opacity);
	DDX_Check(pDX, IDC_CUTLINES_CHECK, m_IsCutLines);
	DDX_Radio(pDX, IDC_CUBE_RADIO, m_MagicVolType);
	DDX_Slider(pDX, IDC_CONTEXTRESSLIDER, m_ContextRes);
	DDX_Slider(pDX, IDC_LINEWIDTHSLIDER, m_LineWidth);
	DDX_Radio(pDX, IDC_NEWBOUND_RADIO, m_RecalcType);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MyFocusContextDialog, CDialog)
	//{{AFX_MSG_MAP(MyFocusContextDialog)
	ON_BN_CLICKED(IDC_MATERIALBUTTON, OnMaterialButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyFocusContextDialog message handlers

BOOL MyFocusContextDialog::OnInitDialog() 
{
	CSliderCtrl *pOpacitySlider = (CSliderCtrl *)GetDlgItem(IDC_OPACITYSLIDER);
	CSliderCtrl *pContextResSlider = (CSliderCtrl *)GetDlgItem(IDC_CONTEXTRESSLIDER);
	CSliderCtrl *pLineWidthSlider = (CSliderCtrl *)GetDlgItem(IDC_LINEWIDTHSLIDER);

	pOpacitySlider->SetRange(DEF_PERCENTSLIDER_MIN, DEF_PERCENTSLIDER_MAX);
	pContextResSlider->SetRange(DEF_PERCENTSLIDER_MIN, DEF_PERCENTSLIDER_MAX);
	pLineWidthSlider->SetRange(DEF_PERCENTSLIDER_MIN, DEF_PERCENTSLIDER_MAX);

	pOpacitySlider->SetTic(DEF_PERCENTSLIDER_MAX/2);
	pContextResSlider->SetTic(DEF_PERCENTSLIDER_MAX/2);
	pLineWidthSlider->SetTic(DEF_PERCENTSLIDER_MAX/2);

	if(m_IsIncremential)
		(CButton *)GetDlgItem( IDC_CUTLINES_CHECK )->EnableWindow(false);
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void MyFocusContextDialog::SetMagicVolType(int magicVolType)
{
	m_MagicVolType = magicVolType;
}

int MyFocusContextDialog::GetMagicVolType()
{
	return m_MagicVolType;
}

MyObject * MyFocusContextDialog::CreateMagicVol(int magicVolType, CPoint3D pos, float size, float radius,
												float length, float height, float width)
{
	MyObject *result;

	switch(magicVolType)
	{
		case DEF_MCUBE_INDEX:
			result = new MyCube();
			((MyCube *)result)->SetSize(size);
			break;
		case DEF_MSPHERE_INDEX:
			result = new MySphere();
			((MySphere *)result)->SetRadius(radius);
			break;
		case DEF_MRECT_INDEX:
			result = new MyRectangle();
			((MyRectangle *)result)->SetLength(length);
			((MyRectangle *)result)->SetWidth(width);
			((MyRectangle *)result)->SetHeight(height);
			break;
		default:
			result = new MyCube();
			((MyCube *)result)->SetSize(size);
			break;
	}

	result->SetPosition(pos);
	result->SetLineWidth(DEF_MAGICVOL_LINEWIDTH);

	GLubyte rgba[4] = DEF_MAGICVOL_RGBA;
	result->SetRgba(rgba);

	return result;
}

int MyFocusContextDialog::GetLineWidth()
{
	return m_LineWidth;
}

void MyFocusContextDialog::SetLineWidth(int lineWidth)
{
	m_LineWidth = lineWidth;
}

void MyFocusContextDialog::GetContextProp(ContextProp &prop)
{
	prop.lineWidthPerc = m_LineWidth;
	prop.opacityPerc = m_Opacity;
	prop.resPerc = m_ContextRes;
	prop.material = m_MaterialDialog.GetMaterial();
}

void MyFocusContextDialog::SetContextProp(ContextProp &prop)
{
	m_LineWidth = prop.lineWidthPerc;
	m_Opacity = prop.opacityPerc;
	m_ContextRes = prop.resPerc;
	m_MaterialDialog.SetMaterial(prop.material);
}

void MyFocusContextDialog::OnMaterialButton() 
{
	// TODO: Add your control notification handler code here
	m_MaterialDialog.DoModal();	
}

void MyFocusContextDialog::SetRecalcType(int recalcType)
{
	m_RecalcType = recalcType;
}

int MyFocusContextDialog::GetRecalcType()
{	
	return m_RecalcType;

}
