#if !defined(AFX_MYFOCUSCONTEXTDIALOG_H__392D60D0_DB97_44AF_B5D7_1FC520EAD7D7__INCLUDED_)
#define AFX_MYFOCUSCONTEXTDIALOG_H__392D60D0_DB97_44AF_B5D7_1FC520EAD7D7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// MyFocusContextDialog.h : header file
//
#include "gl/gl.h"
#include "MyObject.h"
#include "MyCube.h"
#include "MySphere.h"
#include "MyRectangle.h"
#include "MyMaterialDialog.h"	// Added by ClassView

/////////////////////////////////////////////////////////////////////////////
// MyFocusContextDialog dialog

//! Structure which holds context streamline properties.
struct ContextProp
{
	// Context streamlines linewidth.
	float lineWidth;
	//! Context streamlines resolution index.
	int resIndex;
	//! Context streamlines opacity.
	GLubyte opacity;
	//! Context streamlines material.
	CStreamlineMaterial material;
	//! Context streamlines linewidth in percent of focus linewidth.
	int lineWidthPerc;
	//! Context opacity in percent of focus opacity.
	int opacityPerc;
	//! Context streamline resolution in percent of focus resolution.
	int resPerc;
};

//! Dialog specifying focus-context options.
class MyFocusContextDialog : public CDialog
{
// Construction
public:
	//! Returns type of field recalculation.
	int GetRecalcType();
	//! Sets type of field recalculation.
	void SetRecalcType(int recalcType);

	//! Returns context properties.
	void GetContextProp(ContextProp &prop);

	//! Sets context properties.
	void SetContextProp(ContextProp &prop);

	//! Sets context streamline linewidth.
	void SetLineWidth(int lineWidth);

	//! Returns context streamline line width.
	int GetLineWidth();

	//! Creates new instance of magic volume according to parameters.
	static MyObject * CreateMagicVol(int magicVolType, CPoint3D pos, float size, 
									 float radius, float length, float height, float width);
	
	//! Returns magic volume type.
	int GetMagicVolType();

	//! Returns context streamline resolution.
	int GetContextRes();

	//! Returns flag which indicates that context streamlines should be cut in focus.
	bool GetIsCutLines();

	//! Sets flag which indicates that context streamlines should be cut in focus. 
	void SetIsCutLines(bool isCutLines);

	//! Sets context streamline opacity.
	void SetOpacity(int opacity);

	//! Sets context streamline resolution.
	void SetContextRes(int contextRes);

	//! Sets magic volume type.
	void SetMagicVolType(int magicVolType);
	//! Returns context opacity.
	int GetOpacity();
	//! IsIncremential to determine if "cutlines"-options should be shown.
	bool m_IsIncremential;
	
	//! Dialog for specifying context streamline material.
	MyMaterialDialog m_MaterialDialog;
	MyFocusContextDialog(CWnd* pParent = NULL);   // standard constructor
	
// Dialog Data
	//{{AFX_DATA(MyFocusContextDialog)
	enum { IDD = IDD_FOCUS_CONTEXTDIALOG };
	int		m_Opacity;
	BOOL	m_IsCutLines;
	int		m_MagicVolType;
	int		m_ContextRes;
	int		m_LineWidth;
	int		m_RecalcType;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(MyFocusContextDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(MyFocusContextDialog)
	virtual BOOL OnInitDialog();
	afx_msg void OnMaterialButton();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MYFOCUSCONTEXTDIALOG_H__392D60D0_DB97_44AF_B5D7_1FC520EAD7D7__INCLUDED_)
