// MyHistoDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyHistoDialog.h"
#include "Defines.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyHistoDialog dialog


MyHistoDialog::MyHistoDialog(CWnd* pParent /*=NULL*/)
 	: CDialog(MyHistoDialog::IDD, pParent)
{
 	//{{AFX_DATA_INIT(MyHistoDialog)
 		// NOTE: the ClassWizard will add member initialization here
 	//}}AFX_DATA_INIT
	m_IntervNum = DEF_INTERV_NUM;
	m_ScalarType = DEF_VELOCITY_INDEX;

	sprintf(m_Title,"Histogram: %s",DEF_VELOCITY_STR);

	// TODO: Add your specialized creation code here
	for(int i=0;i<DEF_HISTOIMG_NUM;i++)
	{
		m_HistoData[i].SetIntervNum(m_IntervNum);
		m_HistoData[i].SetDataStream(&m_HistoDataStream[i]);
	}
}


void MyHistoDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyHistoDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(MyHistoDialog, CDialog)
	//{{AFX_MSG_MAP(MyHistoDialog)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyHistoDialog message handlers
void MyHistoDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here

	CPen myPen;
	GLubyte barColor[3] = DEF_HISTO_BARCOLOR;
	GLubyte bkColor[3] = DEF_HISTO_BKCOLOR;

	int bkColInt = bkColor[0]+bkColor[1]*256+bkColor[2]*256*256;

	// Implement focus-context later
	for(int i=0; i< DEF_HISTOIMG_NUM; i++)
		DrawHisto(i, barColor, bkColInt);

	// Do not call CDialog::OnPaint() for painting messages
}

MyHistoDialog::DrawHisto(int index, GLubyte *barColor, int bkColor)
{
	CFrameWnd *pHistoImage = GetFrame(index);

	CRect histoRect;
	
	pHistoImage->GetWindowRect(histoRect);
	CPaintDC myDC(pHistoImage); // device context for painting

	HDC hDC = myDC.GetSafeHdc();

	// Draw background
	for(int i=0;i<histoRect.Width();i++)
		for(int j=0;j<histoRect.Height();j++)
			SetPixel(hDC,i,j,bkColor);

	float width = 1.0f;
	
	if(m_IntervNum > 0)
		width = (float)histoRect.Width() / (float)m_IntervNum;

	CPen myPen;
    // Initialize pen
    if( myPen.CreatePen( PS_DOT, (int)width, RGB(barColor[0],barColor[1],barColor[2]) ) )   
    {
		CPen* pOldPen = myDC.SelectObject(&myPen);
		
		float factor = 0;

		if(m_HistoData[0].m_MaxDistr > 0)
			factor = (float)histoRect.Height() / (float)m_HistoData[0].m_MaxDistr;
   
		for(i=0;i<m_HistoData[index].m_IntervNum;i++)
		{
			// Draw with the pen
			myDC.MoveTo((int)(i * width),histoRect.Height());
			myDC.LineTo((int)(i * width),histoRect.Height() - 
						(int)(m_HistoData[index].m_Distribution[i] * factor));
		}
		myDC.SelectObject(&pOldPen);
	}
}

CFrameWnd * MyHistoDialog::GetFrame(int index)
{
	CFrameWnd *result;

	switch(index)
	{
		case 0:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_0 );
			break;
		case 1:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_1 );
			break;
		case 2:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_2 );
			break;
		case 3:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_3 );
			break;
		case 4:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_4 );
			break;
		case 5:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_5 );
			break;
		case 6:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_6 );
			break;
		case 7:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_7 );
			break;
		default:
			result = (CFrameWnd *)GetDlgItem( IDC_HISTOIMG_0 );
			break;
	}

	return result;
}

void MyHistoDialog::InitDataStreams(CObList *list)
{
	// Regions
	for(int i= 0;i < DEF_HISTOIMG_NUM;i++)
	{
		m_HistoDataStream[i].SetPiecesList(list);
		m_HistoDataStream[i].SetHistoType(i-1);
	}
}

void MyHistoDialog::UpdateData()
{
	// Implement focus-context later: DEF_HISTOIMG_NUM
	for(int i=0;i<DEF_HISTOIMG_NUM;i++)
		m_HistoData[i].CalcDistribution();	
}

void MyHistoDialog::UpdateGraphs()
{
	UpdateData();
	this->RedrawWindow();
}

void MyHistoDialog::SetScalarType(int scalarType)
{
	m_ScalarType = scalarType;

	char *str;
	char h[30];

	switch( m_ScalarType ) 
	{
		case DEF_VELOCITY_INDEX: 
			str = DEF_VELOCITY_STR;
			break;
		case DEF_ORIENT_X_INDEX: 
			str = DEF_ORIENT_X_STR;
			break;
		case DEF_ORIENT_Y_INDEX: 
			str = DEF_ORIENT_Y_STR;
			break;
		case DEF_ORIENT_Z_INDEX: 
			str = DEF_ORIENT_Z_STR;
			break;
		case DEF_POS_X_INDEX:
			str = DEF_POS_X_STR;
			break;
		case DEF_POS_Y_INDEX:
			str = DEF_POS_Y_STR;
			break;
		case DEF_POS_Z_INDEX:
			str = DEF_POS_Z_STR;
			break;
		case DEF_DIF_ORIENT_INDEX:
			str = DEF_DIF_ORIENT_STR;
			break;
		case DEF_DIF_VLEN_INDEX:
			str = DEF_DIF_VLEN_STR;
			break;
		default:
			sprintf(h,"%s %d",m_ScalarType - DEF_SCALAR_NUM);
			str = h;
			break;
	}
	
	sprintf(m_Title, "Histogram: %s",str);
	
	for(int i=0;i < DEF_HISTOIMG_NUM; i++)
		m_HistoDataStream[i].SetScalarType(scalarType);
}

BOOL MyHistoDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	SetWindowText(m_Title);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void MyHistoDialog::SetMinMaxScalar(float minScalar, float maxScalar)
{
	for(int i=0;i<DEF_HISTOIMG_NUM;i++)
		m_HistoData[i].SetMinMaxScalar(minScalar,maxScalar);
}
