// MyMaterialDialog.cpp : implementation file
//

#include "stdafx.h"
#include "flow3d.h"
#include "MyMaterialDialog.h"
#include "Defines.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// MyMaterialDialog dialog


MyMaterialDialog::MyMaterialDialog(CWnd* pParent /*=NULL*/)
	: CDialog(MyMaterialDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(MyMaterialDialog)
	m_Opacity = 0;
	//}}AFX_DATA_INIT
}

void MyMaterialDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(MyMaterialDialog)
	DDX_Slider(pDX, IDC_OPACITYSLIDER, m_Opacity);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(MyMaterialDialog, CDialog)
	//{{AFX_MSG_MAP(MyMaterialDialog)
	ON_BN_CLICKED(IDC_AMBIENTBUTTON, OnAmbientbutton)
	ON_BN_CLICKED(IDC_DIFFUSEBUTTON, OnDiffusebutton)
	ON_BN_CLICKED(IDC_SPECULARBUTTON, OnSpecularbutton)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// MyMaterialDialog message handlers

void MyMaterialDialog::OnAmbientbutton() 
{
	// TODO: Add your control notification handler code here
	m_AmbientDialog.DoModal();
	this->RedrawWindow();
}

void MyMaterialDialog::OnDiffusebutton() 
{
	// TODO: Add your control notification handler code here
	m_DiffuseDialog.DoModal();
		this->RedrawWindow();
}

void MyMaterialDialog::OnSpecularbutton() 
{
	// TODO: Add your control notification handler code here
	m_SpecularDialog.DoModal();
	this->RedrawWindow();
}

void MyMaterialDialog::GetAmbientRGB(GLubyte *ambientRGB)
{
	COLORREF c = m_AmbientDialog.GetColor();
	ambientRGB[0] = (GLubyte)(c & 255);
	ambientRGB[1] = (GLubyte)((c>>8) & 255);
	ambientRGB[2] = (GLubyte)((c>>16) & 255);
}

void MyMaterialDialog::GetDiffuseRGB(GLubyte *diffuseRGB)
{
	COLORREF c = m_DiffuseDialog.GetColor();
	diffuseRGB[0] = (GLubyte)(c & 255);
	diffuseRGB[1] = (GLubyte)((c>>8) & 255);
	diffuseRGB[2] = (GLubyte)((c>>16) & 255);
}

void MyMaterialDialog::GetSpecularRGB(GLubyte *specularRGB)
{
	COLORREF c = m_SpecularDialog.GetColor();
	specularRGB[0] = (GLubyte)(c & 255);
	specularRGB[1] = (GLubyte)((c>>8) & 255);
	specularRGB[2] = (GLubyte)((c>>16) & 255);
}


CStreamlineMaterial MyMaterialDialog::GetMaterial()
{
	GLubyte ambientRGB[3];
	GLubyte specularRGB[3];
	GLubyte diffuseRGB[3];

	GetAmbientRGB(ambientRGB);
	GetDiffuseRGB(diffuseRGB);
	GetSpecularRGB(specularRGB);
	
	return CStreamlineMaterial(ambientRGB,diffuseRGB,specularRGB,GetOpacity());
}

void MyMaterialDialog::SetMaterial(CStreamlineMaterial material)
{
	
	SetAmbientRGB(material.m_AmbientRGB);
	SetDiffuseRGB(material.m_DiffuseRGB);
	SetSpecularRGB(material.m_SpecularRGB);
	SetOpacity(material.m_Opacity);
}

void MyMaterialDialog::SetAmbientRGB(GLubyte *ambientRGB)
{
	m_AmbientDialog.m_cc.rgbResult = 
		ambientRGB[0] + ambientRGB[1]*256 + ambientRGB[2]*256*256;
}

void MyMaterialDialog::SetDiffuseRGB(GLubyte *diffuseRGB)
{
	m_DiffuseDialog.m_cc.rgbResult = 
		diffuseRGB[0] + diffuseRGB[1]*256 + diffuseRGB[2]*256*256;
}

void MyMaterialDialog::SetSpecularRGB(GLubyte *specularRGB)
{
	m_SpecularDialog.m_cc.rgbResult = 
	specularRGB[0] + specularRGB[1]*256 + specularRGB[2]*256*256;
}

BOOL MyMaterialDialog::OnInitDialog() 
{
	// TODO: Add extra initialization here
	CSliderCtrl *pOpacitySlider = (CSliderCtrl *)GetDlgItem( IDC_OPACITYSLIDER );
	pOpacitySlider->SetRange(DEF_OPACITYSLIDER_MIN, DEF_OPACITYSLIDER_MAX);
	pOpacitySlider->SetTic(DEF_OPACITYSLIDER_MAX/2);

	CDialog::OnInitDialog();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void MyMaterialDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
	CFrameWnd * pAmbientImage = (CFrameWnd *)GetDlgItem( IDC_AMBIENTIMG );
	CFrameWnd * pDiffuseImage = (CFrameWnd *)GetDlgItem( IDC_DIFFUSEIMG );
	CFrameWnd * pSpecularImage = (CFrameWnd *)GetDlgItem( IDC_SPECULARIMG );

	CRect ambientRect;
	CRect specularRect;
	CRect diffuseRect;

	pAmbientImage->GetWindowRect(ambientRect);
	pDiffuseImage->GetWindowRect(diffuseRect);
	pSpecularImage->GetWindowRect(specularRect);

	CPaintDC ambientDC(pAmbientImage); 
	CPaintDC diffuseDC(pDiffuseImage); 
	CPaintDC specularDC(pSpecularImage); 

	HDC hAmbientDC = ambientDC.GetSafeHdc();
	HDC hDiffuseDC = diffuseDC.GetSafeHdc();
	HDC hSpecularDC = specularDC.GetSafeHdc();

	GLubyte ambientRGB[3];
	GLubyte diffuseRGB[3];
	GLubyte specularRGB[3];

	GetAmbientRGB(ambientRGB);
	GetDiffuseRGB(diffuseRGB);
	GetSpecularRGB(specularRGB);

	for(int i=0;i<ambientRect.Width();i++)
	{
		for(int j=0;j<ambientRect.Height();j++)
		{
			SetPixel(hAmbientDC,i,j,ambientRGB[0]+(ambientRGB[1]<<8)+(ambientRGB[2]<<16));
		}
	}

	for(i=0;i<diffuseRect.Width();i++)
	{
		for(int j=0;j<diffuseRect.Height();j++)
		{
			SetPixel(hDiffuseDC,i,j,diffuseRGB[0]+(diffuseRGB[1]<<8)+(diffuseRGB[2]<<16));
		}
	}

	for(i=0;i<specularRect.Width();i++)
	{
		for(int j=0;j<specularRect.Height();j++)
		{
			SetPixel(hSpecularDC,i,j,specularRGB[0]+(specularRGB[1]<<8)+(specularRGB[2]<<16));
		}
	}
	// Do not call CDialog::OnPaint() for painting messages
}

GLubyte MyMaterialDialog::GetOpacity()
{
	return (GLubyte)m_Opacity;
}

void MyMaterialDialog::SetOpacity(GLubyte opacity)
{
	m_Opacity = (int)opacity;
}