// MyRectangle.cpp: implementation of the MyRectangle class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "MyRectangle.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MyRectangle::MyRectangle()
{
	for(int i=0;i<4;i++)
		m_Rgba[i] = 0;
	
	m_Length = 0;
	m_Width = 0;
	m_Height = 0;
	m_LineWidth = 0;
	m_Position = CPoint3D(0,0,0);

	UpdateMinPoint();
	UpdateMaxPoint();
}

MyRectangle::~MyRectangle()
{

}

void MyRectangle::Draw()
{
	glPushMatrix();
	glTranslatef(m_Position.GetXPos(),m_Position.GetYPos(),m_Position.GetZPos());
	
	float maxXPos = m_Length * 0.5f;
	float maxYPos = m_Width * 0.5f;
	float maxZPos = m_Height * 0.5f;
	float minXPos = - m_Length * 0.5f;
	float minYPos = - m_Width * 0.5f;
	float minZPos = - m_Height * 0.5f;

	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	// Save old color.
	GLfloat oldRgba[4];
	glGetFloatv(GL_CURRENT_COLOR, oldRgba);

	// Save old line width.
	GLfloat oldLineWidth;
	glGetFloatv(GL_LINE_WIDTH, &oldLineWidth);

	// Set color to rgba
	glColor4ubv(m_Rgba);

	// Set line width tp lineWidth.
	glLineWidth(m_LineWidth);

	glBegin(GL_QUADS);

		glVertex3d(minXPos, minYPos, minZPos);
		glVertex3d(minXPos, minYPos, maxZPos);
		glVertex3d(minXPos, maxYPos, maxZPos);
		glVertex3d(minXPos, maxYPos, minZPos);

		glVertex3d(maxXPos, minYPos, minZPos);
		glVertex3d(maxXPos, minYPos, maxZPos);
		glVertex3d(maxXPos, maxYPos, maxZPos);
		glVertex3d(maxXPos, maxYPos, minZPos);

		glVertex3d(minXPos, minYPos, minZPos);
		glVertex3d(minXPos, minYPos, maxZPos);
		glVertex3d(maxXPos, minYPos, maxZPos);
		glVertex3d(maxXPos, minYPos, minZPos);
	
		glVertex3d(minXPos, maxYPos, minZPos);
		glVertex3d(minXPos, maxYPos, maxZPos);
		glVertex3d(maxXPos, maxYPos, maxZPos);
		glVertex3d(maxXPos, maxYPos, minZPos);

		glVertex3d(minXPos, minYPos, minZPos);
		glVertex3d(minXPos, maxYPos, minZPos);
		glVertex3d(maxXPos, maxYPos, minZPos);
		glVertex3d(maxXPos, minYPos, minZPos);

		
		glVertex3d(minXPos, minYPos, maxZPos);
		glVertex3d(minXPos, maxYPos, maxZPos);
		glVertex3d(maxXPos, maxYPos, maxZPos);
		glVertex3d(maxXPos, minYPos, maxZPos);

    glEnd();

	// Set color and line width to old values.
	glColor4fv(oldRgba);
	glLineWidth(oldLineWidth);

	glPopMatrix();
}

MyRectangle::MyRectangle(CPoint3D position, float length, float width, 
						 float height, float lineWidth, GLubyte *rgba)
{
	SetRgba(rgba);
	m_Length = length;
	m_Width = width;
	m_Height = height;
	m_LineWidth = lineWidth;
	m_Position = position;

	UpdateMinPoint();
	UpdateMaxPoint();
}

void MyRectangle::UpdateMinPoint()
{
	// Calculate magic cube minimal point.

	m_MinPoint = CPoint3D(m_Position.m_XPos - m_Length * 0.5f,
						  m_Position.m_YPos - m_Width * 0.5f,
						  m_Position.m_ZPos - m_Height * 0.5f);
}

void MyRectangle::UpdateMaxPoint()
{
	// Calculate magic cube maximal point.

	m_MaxPoint = CPoint3D(m_Position.m_XPos + m_Length * 0.5f,
						  m_Position.m_YPos + m_Width * 0.5f,
						  m_Position.m_ZPos + m_Height * 0.5f);
}

void MyRectangle::SetLength(float length)
{
	m_Length = length;
	UpdateMaxPoint();
	UpdateMinPoint();
}

void MyRectangle::SetWidth(float width)
{
	m_Width = width;
	UpdateMaxPoint();
	UpdateMinPoint();
}

void MyRectangle::SetHeight(float height)
{
	m_Height = height;
	UpdateMaxPoint();
	UpdateMinPoint();	
}

void MyRectangle::SetPosition(CPoint3D position)
{
	m_Position = position;
	UpdateMaxPoint();
	UpdateMinPoint();	
}

bool MyRectangle::IsInside(CPoint3D pos)
{
	return pos.Inside(m_MinPoint,m_MaxPoint);
}

CPoint3D MyRectangle::GetMinPoint()
{
	return m_MinPoint;
}

CPoint3D MyRectangle::GetMaxPoint()
{
	return m_MaxPoint;
}


void MyRectangle::Init(MyObject *obj)
{
	MyRectangle *rect = (MyRectangle *)obj;

	SetPosition(rect->GetPosition());
	SetHeight(rect->GetHeight());
	SetLength(rect->GetLength());
	SetWidth(rect->GetWidth());

	rect->GetRgba(m_Rgba);

	m_LineWidth = rect->GetLineWidth();
}

float MyRectangle::GetHeight()
{
	return m_Height;
}

float MyRectangle::GetWidth()
{
	return m_Width;
}

float MyRectangle::GetLength()
{
	return m_Length;
}
