// MySphere.cpp: implementation of the MySphere class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "MySphere.h"
#include "Defines.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MySphere::MySphere()
{
	m_Radius = 0;
	m_QuadRadius = 0;
}

MySphere::~MySphere()
{

}

void MySphere::SetRadius(float radius)
{
	m_Radius = radius;
	m_QuadRadius = radius * radius;
}

float MySphere::GetRadius()
{
	return m_Radius;
}

void MySphere::Draw()
{
	glPushMatrix();
	glTranslatef(m_Position.GetXPos(),m_Position.GetYPos(),m_Position.GetZPos());

	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	// Save old color.
	GLfloat oldRgba[4];
	glGetFloatv(GL_CURRENT_COLOR, oldRgba);

	// Save old line width.
	GLfloat oldLineWidth;
	glGetFloatv(GL_LINE_WIDTH, &oldLineWidth);

	// Set color to rgba
	glColor4ubv(m_Rgba);

	// Set line width tp lineWidth.
	glLineWidth(m_LineWidth);

//	gluSphere(gluNewQuadric(),m_Radius,8,8);

	glBegin(GL_LINE_LOOP);
		for(int i=0; i<DEF_ANGLE_360; i++) 
		{
			float w = i * 2.0f *PI/ (float)DEF_ANGLE_360;
			glVertex3f(m_Radius * (float)sin(w),m_Radius * (float)cos(w),0);
		}
	glEnd();
	glBegin(GL_LINE_LOOP);
		for(i=0; i<DEF_ANGLE_360; i++) 
		{
			float w = i * 2.0f *PI/ (float)DEF_ANGLE_360;
			glVertex3f(m_Radius * (float)sin(w),0,m_Radius * (float)cos(w));
		}
	glEnd();
	glBegin(GL_LINE_LOOP);
		for(i=0; i<DEF_ANGLE_360; i++) 
		{
			float w = i * 2.0f *PI/ (float)DEF_ANGLE_360;
			glVertex3f(0,m_Radius * (float)sin(w),m_Radius * (float)cos(w));
		}
	glEnd();

	// Set color and line width to old values.
	glColor4fv(oldRgba);
	glLineWidth(oldLineWidth);

	glPopMatrix();
}

bool MySphere::IsInside(CPoint3D position)
{
	bool result = true;

	if(position.QuadDistance(m_Position) > m_QuadRadius)
		result = false;

	return result;
}

MySphere::MySphere(CPoint3D position, float radius, float lineWidth, GLubyte *rgba)
{
	m_Position = position;
	m_Radius = radius;
	m_LineWidth = lineWidth;

	for(int i=0;i<4;i++)
		m_Rgba[i] = rgba[i];
}

void MySphere::Init(MyObject *obj)
{
	MySphere *sphere = (MySphere *)obj;

	SetPosition(sphere->GetPosition());
	SetRadius(sphere->GetRadius());
	sphere->GetRgba(m_Rgba);

	m_LineWidth = sphere->GetLineWidth();
}