// RectTrilinVLength.cpp: implementation of the CRectTrilinVLength class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RectTrilinVLength.h"

#define X_AXIS 0
#define Y_AXIS 1
#define Z_AXIS 2

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRectTrilinVLength::CRectTrilinVLength()
{

}

CRectTrilinVLength::~CRectTrilinVLength()
{

}

void CRectTrilinVLength::LinearInterpolate(CSamplePoint *sp,int axis, CPoint3D &position, 
										   CSamplePoint *s1, CSamplePoint *s2)
{
	float factor = 0;
	
	if(axis == X_AXIS)
	{
		float range = s2->GetXPos() - s1->GetXPos();
	
		if(range != 0)
			factor = (position.m_XPos - s1->GetXPos())/range;
	}
	else if(axis == Y_AXIS)
	{
		float range = s2->GetYPos() - s1->GetYPos();
	
		if(range != 0)
			factor = (position.m_YPos - s1->GetYPos())/range;
	}
	else if(axis == Z_AXIS)
	{
		float range = s2->GetZPos() - s1->GetZPos();
	
		if(range != 0)
			factor = (position.m_ZPos - s1->GetZPos())/range;
	}

	float xComp = s1->GetUnitXVec() + (s2->GetUnitXVec() - s1->GetUnitXVec())*factor;
	float yComp = s1->GetUnitYVec() + (s2->GetUnitYVec() - s1->GetUnitYVec())*factor;
	float zComp = s1->GetUnitZVec() + (s2->GetUnitZVec() - s1->GetUnitZVec())*factor;

	float mag = s1->GetMagnitude() + (s2->GetMagnitude() - s1->GetMagnitude())*factor;
	
	for(int i=0;i<CSamplePoint::GetAdditionalSize();i++)
	{
		sp->SetAdditional(i,s1->GetAdditional(i) + 
						 (s2->GetAdditional(i) - s1->GetAdditional(i))*factor);
	}
	
	CVector3D vec(xComp,yComp,zComp);
	vec.Normalize();

	sp->SetUnitVec(vec);
	sp->SetMagnitude(mag);

	sp->SetPos(position);
}

void CRectTrilinVLength::Interpolate(CSamplePoint *sp, CPoint3D &position,CSamplePoint ** neighbour)
{
		// interpolate linear on both sides of the 2 squares in the yz-plane
	m_CurrentPos.SetPos(neighbour[B_000]->GetXPos(),neighbour[B_000]->GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[0],Z_AXIS,m_CurrentPos,neighbour[B_000],neighbour[B_001]);

	m_CurrentPos.SetPos(neighbour[B_010]->GetXPos(),neighbour[B_010]->GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[1],Z_AXIS,m_CurrentPos,neighbour[B_010],neighbour[B_011]);
	
	m_CurrentPos.SetPos(neighbour[B_100]->GetXPos(),neighbour[B_100]->GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[2],Z_AXIS,m_CurrentPos,neighbour[B_100],neighbour[B_101]);

	m_CurrentPos.SetPos(neighbour[B_110]->GetXPos(),neighbour[B_110]->GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[3],Z_AXIS,m_CurrentPos,neighbour[B_110],neighbour[B_111]);

	// linear interpolate the results on y-axis to get bilinear interpolated points on squares
	m_CurrentPos.SetPos(neighbour[B_000]->GetXPos(),position.GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[4],Y_AXIS,m_CurrentPos,&m_HSampleP[0],&m_HSampleP[1]);

	m_CurrentPos.SetPos(neighbour[B_100]->GetXPos(),position.GetYPos(),position.GetZPos());
	LinearInterpolate(&m_HSampleP[5],Y_AXIS,m_CurrentPos,&m_HSampleP[2],&m_HSampleP[3]);

	//now interpolate linear on x-axis => trilinear interpolated samplepoint
	LinearInterpolate(sp,X_AXIS,position,&m_HSampleP[4],&m_HSampleP[5]);
}
