// RoesslerSampler.cpp: implementation of the CRoesslerSampler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RoesslerSampler.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRoesslerSampler::CRoesslerSampler()
{
	m_Param1 = 0;
	m_Param2 = 0;
	m_Param3 = 0;
	m_AdditionalSize = 0;
}

CRoesslerSampler::CRoesslerSampler(CPoint3D center, float param1, float param2, float param3, 
								   int additionalSize)
{
	m_Center = center;
	m_Param1 = param1;
	m_Param2 = param2;
	m_Param3 = param3;
	m_AdditionalSize = additionalSize;
}

CRoesslerSampler::~CRoesslerSampler()
{

}

CSamplePoint CRoesslerSampler::ComputeSample()
{
	CSamplePoint result;

	float xPos = m_Pos.m_XPos - m_Center.m_XPos;
	float yPos = m_Pos.m_YPos - m_Center.m_YPos;
	float zPos = m_Pos.m_ZPos - m_Center.m_ZPos;

	m_Vector.m_XComp = -(yPos + xPos);
	m_Vector.m_YComp = xPos+m_Param1*yPos;
	m_Vector.m_ZComp = m_Param2 + zPos * xPos + m_Param3*zPos;
	
	result.SetPos(m_Pos);
	result.SetVector(m_Vector);

	SetAddInfo(result.GetAdditional());
	
	return result;
}

void CRoesslerSampler::SetCenter(CPoint3D center)
{
	m_Center = center;
}

CPoint3D CRoesslerSampler::GetCenter()
{
	return m_Center;
}

void CRoesslerSampler::SetParam3(float param3)
{
	m_Param3 = param3;
}

void CRoesslerSampler::SetParam2(float param2)
{
	m_Param2 = param2;
}

void CRoesslerSampler::SetParam1(float param1)
{
	m_Param1 = param1;
}
