// RoesslerSampler.h: interface for the CRoesslerSampler class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ROESSLERSAMPLER_H__233AF163_6FEF_4A9B_87B3_C422B81BB279__INCLUDED_)
#define AFX_ROESSLERSAMPLER_H__233AF163_6FEF_4A9B_87B3_C422B81BB279__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Sampler.h"

//! A class for sampling a vectorfield using the equitations of the Roessler-attractor.
/*!	The Lorenz-attractor is a chaotic attractor(meaning that it is a fractal). The
	Roessler-attractor is a simplified version of the Lorenz-attractor.
*/

class CRoesslerSampler : public CSampler  
{
public:
	CRoesslerSampler();
	virtual ~CRoesslerSampler();
	void SetParam1(float param1);
	void SetParam2(float param2);
	void SetParam3(float param3);
	//! Gets center of sampled field.
	CPoint3D GetCenter();
	//! Sets center of sampled field.
	void SetCenter(CPoint3D center);
	//! computes samples using Roessler-equitations with 3 parameters.
	CRoesslerSampler(CPoint3D center, float param1, float param2, 
					 float param3, int additionalSize);
	virtual CSamplePoint ComputeSample();
	
private:

	float m_Param1;
	float m_Param2;
	float m_Param3;

	CPoint3D m_Center;

};

#endif // !defined(AFX_ROESSLERSAMPLER_H__233AF163_6FEF_4A9B_87B3_C422B81BB279__INCLUDED_)
