// RungeKutta2Op.cpp: implementation of the CRungeKutta2Op class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "RungeKutta2Op.h"
#include "EulerOp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRungeKutta2Op::CRungeKutta2Op()
{
	m_Interpolator = NULL;
	m_Dt = 0;
	m_DtDiv2 = 0;
}

CRungeKutta2Op::CRungeKutta2Op(CVectorField *vectorField,CInterpolator *interpolator,float dt)
{
	m_VectorField = vectorField;
	m_Interpolator = interpolator;
	m_Dt = dt;
	m_DtDiv2 = m_Dt*0.5f;
}
	
CRungeKutta2Op::~CRungeKutta2Op()
{
}

CSamplePoint *CRungeKutta2Op::ComputeNext(CSamplePoint *lastPoint)
{
	float xPos = lastPoint->GetXPos();
	float yPos = lastPoint->GetYPos();
	float zPos = lastPoint->GetZPos();

	// Euler-step with dt/2
	
	m_Pos.m_XPos = xPos + lastPoint->GetXVec()*m_DtDiv2;
	m_Pos.m_YPos = yPos + lastPoint->GetYVec()*m_DtDiv2;
	m_Pos.m_ZPos = zPos + lastPoint->GetZVec()*m_DtDiv2;

	m_Interpolator->Interpolate(&m_InterPoint,m_Pos,
								m_VectorField->GetSamplePoints(m_Pos));

	m_Pos.m_XPos = xPos + m_InterPoint.GetXVec()*m_Dt;
	m_Pos.m_YPos = yPos + m_InterPoint.GetYVec()*m_Dt;
	m_Pos.m_ZPos = zPos + m_InterPoint.GetZVec()*m_Dt;

	return m_Interpolator->Interpolate(m_Pos,
									   m_VectorField->GetSamplePoints(m_Pos));
}

void CRungeKutta2Op::SetDt(float dt)
{
	m_Dt = dt;
	m_DtDiv2 = dt*0.5f;
}
