// RungeKutta4.cpp: implementation of the CRungeKutta4 class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "RungeKutta4Op.h"
#include "EulerOp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRungeKutta4Op::CRungeKutta4Op()
{
	m_Interpolator = NULL;
	m_Dt		   = 0.0f;
	m_DtDiv2	   = 0.0f;	
}

CRungeKutta4Op::CRungeKutta4Op(CVectorField *vectorField,CInterpolator *interpolator,float dt)
{
	m_VectorField  = vectorField;
	m_Interpolator = interpolator;
	m_Dt		   = dt;
	m_DtDiv2	   = m_Dt * 0.5f;
}

CRungeKutta4Op::~CRungeKutta4Op()
{

}

CSamplePoint *CRungeKutta4Op::ComputeNext(CSamplePoint *lastPoint)
{
	float xPos = lastPoint->GetXPos();
	float yPos = lastPoint->GetYPos();
	float zPos = lastPoint->GetZPos();

	// Computing a-component vector.
	m_AVec.m_XComp = lastPoint->GetXVec() * m_Dt;
	m_AVec.m_YComp = lastPoint->GetYVec() * m_Dt;
	m_AVec.m_ZComp = lastPoint->GetZVec() * m_Dt;
	
	// Vomputing b-component vector.
	m_Pos.m_XPos = xPos + lastPoint->GetXVec() * m_DtDiv2,
	m_Pos.m_YPos = yPos + lastPoint->GetYVec() * m_DtDiv2,
	m_Pos.m_ZPos = zPos + lastPoint->GetZVec() * m_DtDiv2;

	m_Interpolator->Interpolate(&m_SP,m_Pos,m_VectorField->GetSamplePoints(m_Pos));

	m_BVec.m_XComp = m_SP.GetXVec() * m_Dt;
	m_BVec.m_YComp = m_SP.GetYVec() * m_Dt;
	m_BVec.m_ZComp = m_SP.GetZVec() * m_Dt;

	// computing c-component vector
	m_Pos.m_XPos = xPos + m_BVec.m_XComp * 0.5f;
	m_Pos.m_YPos = yPos + m_BVec.m_YComp * 0.5f;
	m_Pos.m_ZPos = zPos + m_BVec.m_ZComp * 0.5f;
	
	m_Interpolator->Interpolate(&m_SP,m_Pos,m_VectorField->GetSamplePoints(m_Pos));
	
	m_CVec.m_XComp = m_SP.GetXVec() * m_Dt;
	m_CVec.m_YComp = m_SP.GetYVec() * m_Dt;
	m_CVec.m_ZComp = m_SP.GetZVec() * m_Dt;
	
	// computing d-component vector
	m_Pos.m_XPos = xPos + m_CVec.m_XComp;
	m_Pos.m_YPos = yPos + m_CVec.m_YComp;
	m_Pos.m_ZPos = zPos + m_CVec.m_ZComp;

	m_Interpolator->Interpolate(&m_SP,m_Pos,m_VectorField->GetSamplePoints(m_Pos));

	m_DVec.m_XComp = m_SP.GetXVec() * m_Dt;
	m_DVec.m_YComp = m_SP.GetYVec() * m_Dt;
	m_DVec.m_ZComp = m_SP.GetZVec() * m_Dt;

	// now adding all together and get vector.
	float xComp = (m_AVec.m_XComp + 2.0f * m_BVec.m_XComp + 
				   2.0f * m_CVec.m_XComp + m_DVec.m_XComp) / 6.0f;

	float yComp = (m_AVec.m_YComp + 2.0f * m_BVec.m_YComp + 
				   2.0f * m_CVec.m_YComp + m_DVec.m_YComp) / 6.0f;

	float zComp = (m_AVec.m_ZComp + 2.0f * m_BVec.m_ZComp + 
				   2.0f * m_CVec.m_ZComp + m_DVec.m_ZComp) / 6.0f;

	m_Pos.m_XPos = xPos + xComp;
	m_Pos.m_YPos = yPos + yComp;
	m_Pos.m_ZPos = zPos + zComp;

	return m_Interpolator->Interpolate(m_Pos,m_VectorField->GetSamplePoints(m_Pos));

}

void CRungeKutta4Op::SetDt(float dt)
{
	m_Dt	 = dt;
	m_DtDiv2 = dt * 0.5f;
}
