// RungeKutta4Op.h: interface for the CRungeKutta4 class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RUNGEKUTTA4_H__A40CE5C1_D4BD_48A1_9AF2_8A298CAA6FAC__INCLUDED_)
#define AFX_RUNGEKUTTA4_H__A40CE5C1_D4BD_48A1_9AF2_8A298CAA6FAC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Integrator.h"

//! Runge-Kutta Operator of 4th grade.
/*! Runge-Kutta4 is better than Euler and Runge-Kutta2, but requires
	much more computation because of 4 intermediate steps.
*/
class CRungeKutta4Op : public CIntegrator  
{
public:
	//! Sets dt(=value for one timestep)
	virtual void SetDt(float dt);
	CRungeKutta4Op();
	//! Constructor with pointer to vectorfield, pointer to interpolator and dt as parameters
	CRungeKutta4Op(CVectorField *vectorField,CInterpolator *interpolator,float dt);
	virtual ~CRungeKutta4Op();
	//! Makes one integration-step and returns pointer to new samplepoint.
	virtual CSamplePoint *ComputeNext(CSamplePoint *lastPoint);
private:
	CVector3D m_AVec,m_BVec,m_CVec,m_DVec;
	CPoint3D m_Pos;
	CSamplePoint m_SP;

	float m_DtDiv2;

};

#endif // !defined(AFX_RUNGEKUTTA4_H__A40CE5C1_D4BD_48A1_9AF2_8A298CAA6FAC__INCLUDED_)
