// CSamplePoint.cpp: implementation of the CSamplePoint class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "SamplePoint.h"
#include "FLOAT.h"
#include "Defines.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//! initialising additional array size
int CSamplePoint::m_AdditionalSize = DEF_ADDITIONAL_SIZE;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSamplePoint::CSamplePoint()
{
	m_Vector.m_XComp = 0.0f;
	m_Vector.m_YComp = 0.0f;
	m_Vector.m_ZComp = 0.0f;
	
	m_Pos.m_XPos = 0.0f;
	m_Pos.m_YPos = 0.0f;
	m_Pos.m_ZPos = 0.0f;

	if(m_AdditionalSize > 0)
		m_Additional = new float[m_AdditionalSize];
	else
		m_Additional = NULL;

	for(int i=0;i<m_AdditionalSize;i++)
		m_Additional[i] = 0.0f;

	m_QuadMinDist = FLT_MAX;
	m_Magnitude = 0.0f;
}

CSamplePoint::CSamplePoint(CPoint3D &pos3D, CVector3D &unitVec,float magnitude, float *additional)
{
	SetPos(pos3D);
	SetUnitVec(unitVec);
	SetMagnitude(magnitude);

	if(m_AdditionalSize > 0)
		m_Additional = new float[m_AdditionalSize];
	else
		m_Additional = NULL;

	SetAdditional(additional);

	m_QuadMinDist = FLT_MAX;
}

CSamplePoint::CSamplePoint(CPoint3D &pos3D, CVector3D &vector3D, float *additional)
{
	SetPos(pos3D);
	SetVector(vector3D);
	if(m_AdditionalSize > 0)
		m_Additional = new float[m_AdditionalSize];
	else
		m_Additional = NULL;

	SetAdditional(additional);

	m_QuadMinDist = FLT_MAX;
}

void CSamplePoint::SetVector(CVector3D &vector3D)
{
	m_Magnitude = vector3D.Magnitude();
	if(m_Magnitude != 0)
	{
		m_Vector.SetComp(vector3D.m_XComp/m_Magnitude,
						 vector3D.m_YComp/m_Magnitude,
						 vector3D.m_ZComp/m_Magnitude);
	}
	else m_Vector.SetComp(0,0,0);
}

void CSamplePoint::SetPos(CPoint3D &pos3D)
{
	m_Pos.SetPos(pos3D.m_XPos,pos3D.m_YPos,pos3D.m_ZPos);
}

void CSamplePoint::SetAdditional(int i, float value)
{
	m_Additional[i] = value;
}

void CSamplePoint::GetVector(CVector3D &vec)
{
	vec.m_XComp = m_Vector.m_XComp*m_Magnitude,
	vec.m_YComp = m_Vector.m_YComp*m_Magnitude,
	vec.m_ZComp = m_Vector.m_ZComp*m_Magnitude;
}

CPoint3D *CSamplePoint::GetPos()
{
	return &m_Pos;
}

float CSamplePoint::GetXPos()
{
	return m_Pos.m_XPos;
}

float CSamplePoint::GetYPos()
{
	return m_Pos.m_YPos;
}

float CSamplePoint::GetZPos()
{
	return m_Pos.m_ZPos;
}

float CSamplePoint::GetAdditional(int i)
{
	return m_Additional[i];
}

CSamplePoint::~CSamplePoint()
{
	if(m_Additional != NULL)
		delete [] m_Additional;
}

float CSamplePoint::Distance(CSamplePoint *samplePoint)
{
	return m_Pos.Distance(*samplePoint->GetPos());
}

float CSamplePoint::GetQuadDistance(CSamplePoint *samplePoint)
{
	return m_Pos.QuadDistance(*samplePoint->GetPos());
}

bool CSamplePoint::ComputeQuadMinDist(CObList *samplePoints, float validMinDist)	
{
	bool result = true;
	
	POSITION pos = samplePoints->GetHeadPosition();

	while(result && (pos != NULL))
	{
		CSamplePoint *sp = (CSamplePoint *)samplePoints->GetNext(pos);
		
		float distance = GetQuadDistance(sp);

		if(distance < validMinDist)
			result = false;
		else if(distance < m_QuadMinDist)
			m_QuadMinDist = distance;
	}		

	return result;
}

float CSamplePoint::GetXVec()
{
	return m_Vector.m_XComp*m_Magnitude;
}

float CSamplePoint::GetYVec()
{
	return m_Vector.m_YComp*m_Magnitude;
}

float CSamplePoint::GetZVec()
{
	return m_Vector.m_ZComp*m_Magnitude;
}

void CSamplePoint::SetAdditionalSize(int size)
{
 	m_AdditionalSize = size;
}

int CSamplePoint::GetAdditionalSize()
{
	return m_AdditionalSize;
}

float CSamplePoint::GetQuadMinDist()
{
	return m_QuadMinDist;
}

CSamplePoint::CSamplePoint(const CSamplePoint &samplePoint)
{
	if(m_AdditionalSize > 0)
		m_Additional = new float[m_AdditionalSize];
	else
		m_Additional = NULL;

	*this = samplePoint;
}

CSamplePoint& CSamplePoint::operator=(const CSamplePoint &samplePoint)
{
	m_QuadMinDist = samplePoint.m_QuadMinDist;
	m_Magnitude = samplePoint.m_Magnitude;

 	m_Vector.m_XComp = samplePoint.m_Vector.m_XComp;
 	m_Vector.m_YComp = samplePoint.m_Vector.m_YComp;
	m_Vector.m_ZComp = samplePoint.m_Vector.m_ZComp;
 
 	m_Pos.m_XPos = samplePoint.m_Pos.m_XPos;
 	m_Pos.m_YPos = samplePoint.m_Pos.m_YPos;
 	m_Pos.m_ZPos = samplePoint.m_Pos.m_ZPos;
 
	for(int i=0;i<m_AdditionalSize;i++)
 		m_Additional[i] = samplePoint.m_Additional[i];
	
	return *this;
}

float CSamplePoint::GetMagnitude()
{
	return m_Magnitude;
}

void CSamplePoint::SetMagnitude(float magnitude)
{
	m_Magnitude = magnitude;
}

void CSamplePoint::SetUnitVec(CVector3D &unitVec)
{
	m_Vector = unitVec;
}

CVector3D *CSamplePoint::GetUnitVec()
{
	return &m_Vector;
}

float CSamplePoint::GetUnitXVec()
{
	return m_Vector.m_XComp;
}

float CSamplePoint::GetUnitYVec()
{
	return m_Vector.m_YComp;
}

float CSamplePoint::GetUnitZVec()
{
	return m_Vector.m_ZComp;
}

void CSamplePoint::SetAdditional(float *additional)
{
	for(int i=0;i<m_AdditionalSize;i++)
		m_Additional[i] = additional[i];
}

float * CSamplePoint::GetAdditional()
{
	return m_Additional;
}
