// Streamline.cpp: implementation of the CStreamline class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "Streamline.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// defining static member m_Maxiterations for use at global scope
int CStreamline::m_MaxIterations = DEF_MAXITERATIONS;
int CStreamline::m_MaxItDiv2 = DEF_MAXITERATIONS>>1;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStreamline::CStreamline()
{
	m_Integrator = NULL;
	m_DSepGrid = NULL;
	m_Size = 0;
	m_Resolution = -1;
}

CStreamline::CStreamline(CDSepGrid *dSepGrid, CIntegrator *integrator)
{
	m_DSepGrid = dSepGrid;
	m_Integrator = integrator;
	m_Size = 0;
	m_Resolution = -1;
}

CStreamline::~CStreamline()
{
	POSITION pos = m_SamplePoints.GetHeadPosition();

	while(pos != NULL)
	{
		CSamplePoint *s = (CSamplePoint *)m_SamplePoints.GetNext(pos);
		m_SamplePoints.RemoveHead();
		delete s;
	}
}

int CStreamline::GetSize()
{
	return m_Size;
}

void CStreamline::Grow(CSamplePoint &startPoint)
{
	CSamplePoint *nextPoint = new CSamplePoint(startPoint);

	// Grow in first direction and add to tail.
	for(int i=0;((i < m_MaxItDiv2) &&
				 m_DSepGrid->TestValid(DEF_TYPE_LINE, nextPoint)); i++)
	{
		m_SamplePoints.AddHead(nextPoint);
		nextPoint = m_Integrator->ComputeNext(nextPoint); 
	}

	CSamplePoint *lastPoint = m_Integrator->ComputeLast(&startPoint);

	// Grow in other direction and add to head.
	for(i=0;((i < m_MaxItDiv2) && 
		     m_DSepGrid->TestValid(DEF_TYPE_LINE,lastPoint));i++)
	{
		m_SamplePoints.AddTail(lastPoint);
		lastPoint = m_Integrator->ComputeLast(lastPoint);
	}

	// Inserting samplepoints into dSep-grid.
	POSITION pos = m_SamplePoints.GetHeadPosition();

	while (pos != NULL)
		m_DSepGrid->Insert((CSamplePoint *)m_SamplePoints.GetNext(pos));
	

	//! Counting streamline size and storing value in m_Size.
	m_Size = m_SamplePoints.GetCount();
}

void CStreamline::SetDSepGrid(CDSepGrid *dSepGrid)
{
	m_DSepGrid = dSepGrid;
}

void CStreamline::SetIntegrator(CIntegrator *integrator)
{
	m_Integrator = integrator;
}

CObList *CStreamline::GetSamplePoints()
{
	return &m_SamplePoints;
}

void CStreamline::SetMaxIterations(int maxIterations)
{
	m_MaxIterations = maxIterations;
	m_MaxItDiv2 = maxIterations>>1;
}

int CStreamline::GetMaxIterations()
{
	return m_MaxIterations;
}

CStreamline::CStreamline(const CStreamline &streamline)
{
	m_Integrator = streamline.m_Integrator;
	m_DSepGrid = streamline.m_DSepGrid;
	m_Size = streamline.m_Size;
	m_MaxIterations = streamline.m_MaxIterations;
	m_MaxItDiv2 = streamline.m_MaxItDiv2;

	POSITION pos = streamline.m_SamplePoints.GetHeadPosition();
	
	while(pos!=NULL)
	{
		CSamplePoint *samplePoint = 
			(CSamplePoint *)streamline.m_SamplePoints.GetNext(pos);

		m_SamplePoints.AddTail(new CSamplePoint(*samplePoint));
	}
}

void CStreamline::InsertInDSep()
{
	POSITION pos = m_SamplePoints.GetHeadPosition();

	while (pos != NULL)
	{
		m_DSepGrid->Insert((CSamplePoint *)m_SamplePoints.GetNext(pos));
	}
}

void CStreamline::SetResolution(int resolution)
{
	m_Resolution = resolution;
}

int CStreamline::GetResolution()
{
	return m_Resolution;
}
