// Streamline.h: interface for the CStreamline class.
//
//////////////////////////////////////////////////////////////////////
#include "Integrator.h"
#include "Defines.h"
#include "DSepGrid.h"

#include <afxtempl.h>

#if !defined(AFX_STREAMLINE_H__73D91C57_FE46_4274_AADC_0D4DAD279942__INCLUDED_)
#define AFX_STREAMLINE_H__73D91C57_FE46_4274_AADC_0D4DAD279942__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//! A class representing a streamline in the field.
/*!	A streamline basicly consists of a list of samplepoints which
	must be traversed in order to draw the streamline to the screen.
*/

class CStreamline: public CObject
{
public:
	int GetResolution();
	void SetResolution(int resolution);
	//! Inserts streamlimne into dsepgrid.
	void InsertInDSep();
	//! Returns maximum iteration value the streamline can grow.
	static int GetMaxIterations();
	//! Sets maximum iteration value the streamline can grow.
	static void SetMaxIterations(int maxIterations);
	//! Returns list of samplepoints representing the streamline.
	CObList * GetSamplePoints();
	//! Growing streamline from startpoint in both directions.
	void Grow(CSamplePoint &startPoint);
	virtual ~CStreamline();
	//! Sets pointer to dSepGrid.
	void SetDSepGrid(CDSepGrid *dSepGrid);
	//! Sets pointer to integrator.
	void CStreamline::SetIntegrator(CIntegrator *integrator);
	//! Returns size of streamline(= number of samplepoints).
	int GetSize();

	CStreamline();
	//! Constructor with pointer to dSepGrid, pointer to integrator.
	CStreamline(CDSepGrid *dSepGrid,CIntegrator *integrator);
	//! Copy-constructor.
	CStreamline(const CStreamline &streamline);

	int m_Resolution;
	int m_Size;

private:
	CObList m_SamplePoints;
	CIntegrator *m_Integrator;
	CDSepGrid *m_DSepGrid;
	static int m_MaxIterations;
	static int m_MaxItDiv2;
};

#endif // !defined(AFX_STREAMLINE_H__73D91C57_FE46_4274_AADC_0D4DAD279942__INCLUDED_)
