// StreamlineFinder.cpp: implementation of the CStreamlineFinder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "StreamlineFinder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStreamlineFinder::CStreamlineFinder()
{
	m_DSepGrid = NULL;
	m_Integrator = NULL;
	m_Interpolator = NULL;
	m_Chooser = NULL;
}

CStreamlineFinder::CStreamlineFinder(CDSepGrid *dSepGrid, CInterpolator *interpolator,
									 CIntegrator *integrator, CCandidateChooser *chooser)
{
	m_DSepGrid = dSepGrid;
	m_Integrator = integrator;
	m_Interpolator = interpolator;
	m_Chooser = chooser;
}

CStreamlineFinder::~CStreamlineFinder()
{
	Init();
}

void CStreamlineFinder::Init()
{
	while(!m_Streamlines.IsEmpty())
	{
		CStreamline *s = m_Streamlines.Pop();
		delete s;
	}
}

void CStreamlineFinder::AddLine(CPoint3D pos)
{
	CStreamline *streamline = new CStreamline(m_DSepGrid, m_Integrator);
	CSamplePoint sp;
	
	m_Interpolator->Interpolate(&sp, pos, m_DSepGrid->GetVectorField()->
								GetSamplePoints(pos));

	int x = 0;
	int y = 0;
	int z = 0;

	if(m_DSepGrid->TestValid(DEF_TYPE_START,&sp))
	{
		streamline->Grow(sp);
		m_Streamlines.Push(streamline);
	}
}


/*! The ESS(Evenly-Spaced-Streamline)-algorithm is a method to derive streamlines
	from a vectorfield with a distance 'dSep' from an existing streamline, resulting
    in a evenly distribution of streamlines over the field
*/
CStreamline *CStreamlineFinder::ApplyESS()
{
	int x=0;
	int y=0;
	int z=0;

	CStreamline *currentLine = NULL;
	if(!m_Streamlines.IsEmpty())
	{
		currentLine = m_Streamlines.Pop();
		m_Chooser->CreateCandidates(currentLine);
		CObList *candidates =
			m_Chooser->GetCandidates();

		POSITION pos = candidates->GetHeadPosition();

		while (pos != NULL)
		{
			CSamplePoint *samplePoint = (CSamplePoint *)candidates->GetNext(pos);
			
			if(m_DSepGrid->TestValid(DEF_TYPE_START, samplePoint))
			{
				CStreamline *streamline = new CStreamline(m_DSepGrid,m_Integrator);
				streamline->Grow(*samplePoint);
				m_Streamlines.Push(streamline);
			}
		}
	
	}

	return currentLine;
}

CStreamlineQueue * CStreamlineFinder::GetStreamlines()
{
	return &m_Streamlines;
}

void CStreamlineFinder::AddStreamlines(CObList *streamlines)
{
	POSITION pos = streamlines->GetHeadPosition();

	while(pos!=NULL)
	{
		CStreamline *s = (CStreamline*)streamlines->GetNext(pos);
		m_Streamlines.Push(s);
		s->InsertInDSep();
	}
}
