// StreamlineMaterial.cpp: implementation of the CStreamlineMaterial class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "flow3d.h"
#include "StreamlineMaterial.h"
#include "Defines.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStreamlineMaterial::CStreamlineMaterial()
{
	for(int i=0;i<3;i++)
	{
		m_AmbientRGB[i] = DEF_COLOR_MAX;
		m_DiffuseRGB[i] = DEF_COLOR_MAX;
		m_SpecularRGB[i] = DEF_COLOR_MAX;
		m_AmbRGBF[i] = 1.0f;
		m_DifRGBF[i] = 1.0f;
		m_SpecRGBF[i] = 1.0f;
	}
	m_Opacity = DEF_OPACITY_MAX;
	m_OpacityF = 1.0f;
}

CStreamlineMaterial::CStreamlineMaterial(GLubyte *ambientRGB, GLubyte *diffuseRGB, 
										 GLubyte *specularRGB, GLubyte opacity)
{
	SetAmbientRGB(ambientRGB);
	SetDiffuseRGB(diffuseRGB);
	SetSpecularRGB(specularRGB);
	SetOpacity(opacity);
}

CStreamlineMaterial::~CStreamlineMaterial()
{

}

void CStreamlineMaterial::SetAmbientRGB(GLubyte *ambientRGB)
{
	for(int i=0;i<3;i++)
	{
		m_AmbientRGB[i] = ambientRGB[i];
		m_AmbRGBF[i] = (float)ambientRGB[i]/(float)DEF_COLOR_MAX;
	}

}

void CStreamlineMaterial::SetSpecularRGB(GLubyte *specularRGB)
{
	for(int i=0;i<3;i++)
	{
		m_SpecularRGB[i] = specularRGB[i];
		m_SpecRGBF[i] = (float)specularRGB[i]/(float)DEF_COLOR_MAX;
	}
}

void CStreamlineMaterial::SetDiffuseRGB(GLubyte *diffuseRGB)
{
	for(int i=0;i<3;i++)
	{
		m_DiffuseRGB[i] = diffuseRGB[i];
		m_DifRGBF[i] = (float)diffuseRGB[i]/(float)DEF_COLOR_MAX;
	}
}

void CStreamlineMaterial::SetOpacity(GLubyte opacity)
{
	m_Opacity = opacity;
	m_OpacityF = (float)opacity / (float)DEF_OPACITY_MAX;
}

void CStreamlineMaterial::GetAmbientRGB(GLubyte *ambientRGB)
{
	for(int i=0;i<3;i++)
		ambientRGB[i] = m_AmbientRGB[i];
}

void CStreamlineMaterial::GetDiffuseRGB(GLubyte *diffuseRGB)
{
	for(int i=0;i<3;i++)
		diffuseRGB[i] = m_DiffuseRGB[i];
}

void CStreamlineMaterial::GetSpecularRGB(GLubyte *specularRGB)
{
	for(int i=0;i<3;i++)
		specularRGB[i] = m_SpecularRGB[i];
}

CStreamlineMaterial::CStreamlineMaterial(const CStreamlineMaterial &material)
{
	for(int i=0;i<3;i++)
	{
		m_AmbientRGB[i] = material.m_AmbientRGB[i];
		m_DiffuseRGB[i] = material.m_DiffuseRGB[i];
		m_SpecularRGB[i] = material.m_SpecularRGB[i];

		m_AmbRGBF[i] = material.m_AmbRGBF[i];
		m_DifRGBF[i] = material.m_DifRGBF[i];
		m_SpecRGBF[i] = material.m_SpecRGBF[i];
	}

	m_Opacity = material.m_Opacity;
	m_OpacityF = material.m_OpacityF;
}

CStreamlineMaterial& CStreamlineMaterial::operator=(const CStreamlineMaterial &material)
{
	for(int i=0;i<3;i++)
	{
		m_AmbientRGB[i] = material.m_AmbientRGB[i];
		m_DiffuseRGB[i] = material.m_DiffuseRGB[i];
		m_SpecularRGB[i] = material.m_SpecularRGB[i];
				
		m_AmbRGBF[i] = material.m_AmbRGBF[i];
		m_DifRGBF[i] = material.m_DifRGBF[i];
		m_SpecRGBF[i] = material.m_SpecRGBF[i];
	}

	m_Opacity = material.m_Opacity;
	m_OpacityF = material.m_OpacityF;

	return *this;
}