// StreamlinePiece.h: interface for the CStreamlinePiece class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_STREAMLINEPIECE_H__8627EC39_B147_44B2_8A8A_BAF990DF4FB2__INCLUDED_)
#define AFX_STREAMLINEPIECE_H__8627EC39_B147_44B2_8A8A_BAF990DF4FB2__INCLUDED_

#include "SamplePoint.h"
#include "Streamline.h"
#include "gl/gl.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//! Class representing a piece of a streamline.
/*!
	Class representing a piece of a streamline consisting of a start samplepoint,
	an end samplepoint and display information
*/
class CStreamlinePiece : public CObject
{
public:
	//! Calculates difference of vector lengths of samplepoints.
	float CalcDifVLen();
	//! Calculates difference in samplepoint vector orientation.
	float CalcDifOrient();

	//! Endpoint of piece.
	CSamplePoint *m_EndPoint;
	//! Startpoint of piece.
	CSamplePoint *m_StartPoint;
	//! Pointer to streamline containing piece.
	CStreamline *m_Streamline;

	//! Piece index for transparency function table.
	int m_Index;
	
	//! Base color of streamline piece.
	GLubyte m_Color[4];

	//! Line width of streamline piece.
	float m_LineWidth;

	//! Region index of streamline piece.
	byte m_RegIndex;

	//! Should streamline be drawn ?
	bool m_IsDraw;

	//! Is streamline in context ?
	bool m_IsInside;

	//! Resolution index.
	int m_ResIndex;

	//! End-tapering index
	int m_EndTapIndex;

	//! Flag which indicates that piece is start of a streamline
	bool m_IsLineStart;

	//! Standard constructor.
	CStreamlinePiece();

	//!Initialises piece with startpoint, endpoint, streamline index, streamline pointer.
	CStreamlinePiece(CSamplePoint *startPoint, CSamplePoint *endPoint,
					 int index, CStreamline *streamline);

	//! Returns x-position of samplepoint startpoint.
	float GetStartPZPos();
	//! Returns y-position of samplepoint startpoint.
	float GetStartPYPos();
	//! Returns z-position of samplepoint startpoint.
	float GetStartPXPos();

	//! Retruns pointer to streamline containing piece.
	CStreamline *GetStreamline();

	virtual ~CStreamlinePiece();
};

#endif // !defined(AFX_STREAMLINEPIECE_H__8627EC39_B147_44B2_8A8A_BAF990DF4FB2__INCLUDED_)
