// TextureGenerator.cpp: implementation of the CTextureGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Flow3D.h"
#include "Defines.h"
#include "TextureGenerator.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTextureGenerator::CTextureGenerator()
{
	m_TextureMap = NULL;
	m_MapSize = 0;	
}

CTextureGenerator::~CTextureGenerator()
{
	if(m_TextureMap!=NULL)
	{
		delete [] m_TextureMap;
		m_TextureMap = NULL;
	}
}

int CTextureGenerator::GetMapSize()
{
	return m_MapSize;
}

void CTextureGenerator::SetMapSize(int size)
{
	m_MapSize = size;
}

GLubyte * CTextureGenerator::GetTextureMap()
{
	return m_TextureMap;
}

CTextureGenerator::CTextureGenerator(int size)
{
	m_MapSize = size;
	m_TextureMap = NULL;
}

void CTextureGenerator::GenTextureMap(CStreamlineMaterial material, float diffCoeff,
									  float specCoeff)
{
	if(m_TextureMap!=NULL)
		delete [] m_TextureMap;

	// allocate the array
	m_TextureMap = new GLubyte[m_MapSize*m_MapSize*4];

	for(int i=0;i<m_MapSize*m_MapSize*4;i+=m_MapSize*4)
	{
		float t1 = (float)i/(float)((m_MapSize-1)*m_MapSize*4);

		for(int j=0;j<m_MapSize*4;j+=4)
		{
			float t2 = (float)j/(float)((m_MapSize-1)*4);

			for(int k=0;k<3;k++)
			{
				float dot_LT = 2.0f*t1 - 1.0f;
				float dot_VT = 2.0f*t2 - 1.0f;

				float factor_t1 = (float)sqrt((1.0f-dot_LT*dot_LT));
				float factor_t2 = (float)sqrt((1.0f-dot_VT*dot_VT));

				int diffuseVal = 
					(int)((float)material.m_DiffuseRGB[k]*pow(factor_t1,diffCoeff));

				int specularVal = 
					(int)((float)material.m_SpecularRGB[k]*
									pow((factor_t1*factor_t2-dot_LT*dot_VT),specCoeff));
				
				// Specular component must be >= 0.
				if(specularVal < 0)
					specularVal = 0;

				int color = material.m_AmbientRGB[k] + diffuseVal + specularVal;

				// Color must be < maximum value.
				if(color > DEF_COLOR_MAX) 
					color = DEF_COLOR_MAX;

				m_TextureMap[i+j+k] = (GLubyte)color;
				
			}

			m_TextureMap[i+j+3] = material.m_Opacity;
		}	
	}
}
