// TextureGenerator.h: interface for the CTextureGenerator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_TEXTUREGENERATOR_H__DB9D6A46_E38A_4C5E_BE6B_10F816AE9350__INCLUDED_)
#define AFX_TEXTUREGENERATOR_H__DB9D6A46_E38A_4C5E_BE6B_10F816AE9350__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "gl\gl.h"
#include "gl\glu.h"
#include "StreamlineMaterial.h"

//! A class for generating texture map for illuminated streamlines with opengl
/*!	The generated texture is stored in quasi-2-dimensional array(in fact its
	1-dimensional) and can be used as 2-dimensional texture map with the
	gl-call:

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_TexGenerator.GetMapSize(), 
                 m_TexGenerator.GetMapSize(), 0, GL_RGBA, GL_UNSIGNED_BYTE, 
                 m_TexGenerator.GetTextureMap());
*/
class CTextureGenerator  
{
public:
	//! Generates texture map.
	/*! AmbientRGB, diffuseRGB, specularRGB store the RGB values for
		the ambient, diffuse and specular colors as 3 unsigned bytes.
		diffCoeff is a correction coefficient for diffuse colors, specCoeff
		is the highlight coefficient.
	*/
	void GenTextureMap(CStreamlineMaterial material,
					   float diffCoeff, float specCoeff);

	CTextureGenerator(int size);
	//! Returns pointer to texture map .
	GLubyte *GetTextureMap();
	//! Sets size of one dimension of map, in fact width=size and heigth = size
	void SetMapSize(int size);
	//! Returns size of one dimension of map, , in fact width=size and heigth = size
	int GetMapSize();
	
	CTextureGenerator();
	virtual ~CTextureGenerator();

private:
	GLubyte *m_TextureMap;
	int m_MapSize;
};

#endif // !defined(AFX_TEXTUREGENERATOR_H__DB9D6A46_E38A_4C5E_BE6B_10F816AE9350__INCLUDED_)
