// Vector3D.h: interface for the CVector3D class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VECTOR3D_H__F5482D1E_45D0_4FF2_8176_00B77C4862A1__INCLUDED_)
#define AFX_VECTOR3D_H__F5482D1E_45D0_4FF2_8176_00B77C4862A1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//#include <afxtempl.h>
#include "SinTable.h"
#include "CosTable.h"

//!  A class representing a Vector in 3D.
class CVector3D  
{
public:
	CVector3D();
	//! Initialises vector with 3 float values.
	CVector3D(float xComp, float yComp, float zComp);
	virtual ~CVector3D();

public:
	//! Multiplicate vector with 4-dimensional homogenious matrix.
	void Mult(float *matrix4D);
	//! Returns array with 'num' normal vectors, using unitVec as axis.
	static void Normals(CVector3D &unitVec, CVector3D *normals, int num);
	//! Tests if vector is 0-vector.
	bool IsNullVector();
	//! Sets x,y and z-component of this vector.
	void SetComp(float xComp,float yComp,float zComp);
	//! Returns array with 'num' normal vectors.
	void Normals(CVector3D *normals, int num);
	//! Returns one of the normals of this vector.
	void Normal(CVector3D &normal);
	//! Rotates vector around axis(must be unit vector!), angle is grad.
	void Rotate(int angle, CVector3D &axis);
	//! Normalizes vector.
	void Normalize();
	//! Returns magnitude of this vector.
	float Magnitude();
	//! Returns cross-product of this vector and source.
	void CrossProduct(CVector3D &source, CVector3D &dest);
	//! Returns cross-product of this vector and parameter vector3D.
	float DotProduct(CVector3D &vector3D);
	//! Components of vector.
	float m_XComp,m_YComp,m_ZComp;

private:
	static CSinTable m_SinTable;
	static CCosTable m_CosTable;
	//! Help vector array for rotation (To avoid creating new instances).
	static CVector3D m_HVec[3];
};

#endif // !defined(AFX_VECTOR3D_H__F5482D1E_45D0_4FF2_8176_00B77C4862A1__INCLUDED_)
